<?
/*
author: Ponomarev Denis <ponomarev@gmail.com>
$Date: 2004/04/27 08:37:38 $
$Revision: 1.14 $
*/

class ActionLogger {

var $descriptors = array(
	'account'=>'SELECT id, name FROM accounts WHERE id=%s',
	'account_src'=>'SELECT id, name FROM accounts WHERE id=%s',
	'account_dst'=>'SELECT id, name FROM accounts WHERE id=%s',
	'calendar_period'=>"SELECT id, DATE_FORMAT(period_begin, '%%Y-%%m-%%d') period_begin, DATE_FORMAT(period_end, '%%Y-%%m-%%d') period_end FROM calendar_periods WHERE id=%s",
	'charge_type'=>'SELECT id, name FROM charge_types WHERE id=%s',
	'client'=>'SELECT id, CONCAT(last_name, " ", first_name, " (", login, ")") name FROM clients WHERE id=%s',
	'connection_type'=>'SELECT id, name FROM connection_types WHERE id=%s',
	'connection_type_resource'=>'SELECT ctr.id, ct.name connection_type, rt.name resource_type FROM connection_type_resources ctr LEFT JOIN connection_types ct ON ct.id=ctr.connection_type LEFT JOIN resource_types rt ON rt.id=ctr.resource_type WHERE ctr.id=%s',
	'department'=>'SELECT id, name FROM departments WHERE id="%s"',
	'domain'=>'SELECT id, name FROM domains WHERE id="%s"',
	'homedir_type'=>'SELECT id, name FROM homedir_types WHERE id="%s"',
	'host'=>'SELECT id, ip FROM hosts WHERE id=%s',
	'ip_use_interval'=>'SELECT id, from_time, to_time FROM ip_use_intervals WHERE id=%s',
	'mailbox_type'=>'SELECT id, name FROM mailbox_types WHERE id="%s"',
	'mail_list'=>'SELECT id, name FROM mail_lists WHERE id=%s',
	'modem'=>'SELECT id, ip FROM modems WHERE id="%s"',
	'modem_profile'=>'SELECT id, name FROM modem_profiles WHERE id="%s"',
	'operator'=>'SELECT id, CONCAT(name, " (", c.login, ")") FROM operators WHERE id=%s',
	'personal_period'=>"SELECT pp.id, CONCAT(c.last_name, ' ', c.first_name, ' (', c.login, ')') client, DATE_FORMAT(period_begin, '%%Y-%%m-%%d') period_begin, DATE_FORMAT(period_end, '%%Y-%%m-%%d') period_end FROM personal_periods pp LEFT JOIN clients c ON c.id=pp.client WHERE pp.id=%s",
	'poll'=>'SELECT id, title FROM polls WHERE id="%s"',
	'query'=>'SELECT id, name FROM queries WHERE id=%s',
	'resource_type'=>'SELECT id, name FROM resource_types WHERE id="%s"',
	'rule'=>'SELECT id, name FROM rules WHERE id="%s"',
	'shaper'=>'SELECT id, name FROM shapers WHERE id="%s"',
	'street'=>'SELECT id, name FROM streets WHERE id="%s"',
	'system_account'=>'SELECT account, code, name FROM system_accounts WHERE account=%s',
	'system_entity'=>'SELECT id, name FROM system_entities WHERE id=%s',
	'tariff'=>'SELECT id, name FROM tariffs WHERE id=%s',
);

var $skip = array('stamp', 'created', 'created_by', 'created_name', 'modified', 'modified_by', 'modified_name');

var $db;
var $oper;
var $objects;

function ActionLogger($db, $oper){
	$this->db = $db;
	$this->oper = $oper;
	$this->objects = array();
	$res = $db->Execute("SELECT id, name FROM action_log_objects");
	while($arr = $db->Next($res)){
		$this->objects[$arr['name']] = $arr['id'];
	}
}

function getObjectId($name){
	//echo "<br><br><br><br>[$name]<br><br><br>";
	if(isset($this->objects[$name])){
		return $this->objects[$name];
	}else{
		$this->db->Execute(sprintf("INSERT INTO action_log_objects (name) VALUES ('%s')", addslashes($name)));
		$id = $this->db->getJustInserted('action_log_objects', 'id');
		$this->objects[$name] = $id;
		return $id;
	}
}

function log($action, $object, $str, $id=''){
	$query = sprintf("INSERT INTO action_log (operator, type, object, item_id, action, date) VALUES ('%s', '%s', '%s', %s, '%s', now())", 
		($this->oper->id ? $this->oper->id : CLIENT_OPERATOR),
		$action,
		$this->getObjectId($object),
		$id ? intval($id) : 'NULL',
		addslashes($str)
	);
//	echo $query;
	$this->db->Execute($query);
}

function log_inserted($name, $id, $inserted){
	$this->log(1, $name, implode("\n", _Pair($inserted,'', ': ')), $id);
}

function after_insert($table){
	$inserted = array();
	$id = '';
	foreach($table->fields as $name=>$field){
		if(!in_array($name, $this->skip) && !$field->skip_log){
			$title = isset($field->title) ? $field->title : $name;
			$log_by = $field->log_by ? $field->log_by : $name;
			if($field->key || $field->log || isset($table->new_record[$name])){
				$inserted[$title] = $table->current_record[$log_by];
			}
			if($name=='id'){
				$id = $table->current_record[$log_by];
			}
		}
	}
	$this->log_inserted($table->name, $id, $inserted);
}

function log_posted($name, $id, $desc, $post){
	$this->log(2, $name,
		implode("\n", _Pair($desc,'', ': '))."\n*diff*\n".
		implode("\n", _Pair($post,'', ': ')),
		$id);
}

function after_post($table){
	$desc = array();
	$post = array();
	$id = '';
	foreach($table->fields as $name=>$field){
		if(!in_array($name, $this->skip) && !$field->skip_log){
			$title = isset($field->title) ? $field->title : $name;
			$log_by = $field->log_by ? $field->log_by : $name;
			if($field->key || $field->desc){
				$desc[$title] = $table->old_record[$log_by];
			}
			if(isset($table->new_record[$name]) && $table->isChanged($name)){
				$post[$title] = $table->old_record[$log_by].
					'*->*'.
					$table->current_record[$log_by];
			}
			if($name=='id'){
				$id = $table->current_record[$name];
			}
		}
	}
//	if(!empty($post)){
		$this->log_posted($table->name, $id, $desc, $post);
//	}
}

function log_deleted($name, $id, $desc){
	$this->log(3, $name, implode("\n", _Pair($desc, '', ': ')), $id);
}

function after_delete($table){
	$desc = array();
	$id = '';
	foreach($table->fields as $name=>$field){
		if(!in_array($name, $this->skip) && !$field->skip_log){
			$title = isset($field->title) ? $field->title : $name;
			$log_by = $field->log_by ? $field->log_by : $name;
			if($field->key || $field->desc){
				$desc[$title] = $table->old_record[$log_by];
			}
		}
		if($name=='id'){
			$id = $table->old_record[$name];
		}
	}
	$this->log_deleted($table->name, $id, $desc);
}

function getExtValue($record, $field){
	$val = $record[$field->name];
	if($val && isset($this->descriptors[$field->lookup])){
		$desc = $this->descriptors[$field->lookup];
		$desc_record = $this->db->Next($this->db->Execute(sprintf($desc, $val)));
		return '['.implode(", ", _Pair($desc_record, array($this->db,'Pak'))).']';
	}else{
		return $this->db->Pak($val);
	}
}

function getDescriptor($name, $id){
	$desc = $this->descriptors[$name];
	$desc_record = $this->db->Next($this->db->Execute(sprintf($desc, $id)));
	return '['.implode(", ", _Pair($desc_record, array($this->db,'Pak'))).']';
}

}
?>