<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class IptAccounter {

var $host = 'localhost';
var $user;
var $logger;
var $tables = array();
var $tablesPattern;
var $workdir;
var $clientIsSource;
var $detected;
var $ready;
var $loaded;
var $processorLogged;
var $data = array();
var $profiler;
var $iptNames;

function setProfiler($obj){
	$this->profiler = $obj;
}

function setBeanName($str){
	$this->beanName = $str;
}

function setIptNames($obj){
	$this->iptNames = $obj;
}

function setLogFactory($logFactory){
	$this->logger = $logFactory->getLogger('ipt.accounter');
}

function setProcessorLogged($obj){
	$this->processor = $obj;
}

function setWorkdir($str){
	$this->workdir = $str;
}

function setTables($arr){
	$this->tables = $arr;
}

function setTablesPattern($str){
	$this->tablesPattern = $str;
}

function setClientIsSource($i){
	$this->clientIsSource = $i;
}

function initBean(){
	$this->iptNames->registerServer($this->host, $this->buildTablesListingCommand());
}

function buildListingCommand($table){
	return sprintf("/usr/local/sbin/iptaccount -l %s -f | /bin/grep 'IP: '", $table);
}

function buildTablesListingCommand(){
	return "/usr/local/sbin/iptaccount -a";
}

function getListFileName($table){
	return $table;
}

function readTables($debug=false){
	if(!$this->detected){
		if($this->tablesPattern){
			foreach($this->iptNames->getNames($this->host) as $table){
				if(preg_match($this->tablesPattern, $table)){
					if(!in_array($table, $this->tables)){
						$this->tables[] = $table;
					}
				}
			}
		}
		$this->detected = 1;
	}
}

function readCounters(){
	if(!$this->iptNames->isActive($this->host)){
		return;
	}
	if(!$this->ready){
		$this->readTables();
		$this->profiler->enter('ipt.read.counters');
		$this->logger->debug("read counters %s", implode(", ", $this->tables));
		foreach($this->tables as $table){
			$filename = $this->workdir.'/list-'.$this->getListFileName($table);
			//  
			$cmd = " > ".$filename;
			$exit_code = $this->processor->process($cmd, $this->logger);
			$this->logger->debug("listing '%s' table", $table);
			$cmd = $this->buildListingCommand($table)." > ".$filename;
			$this->logger->debug($cmd);
			$exit_code = $this->processor->process($cmd, $this->logger);
			if($exit_code){
				if($exit_code==1){
					$this->logger->debug("table '%s' is empty", $table);
				}else{
					$this->logger->error("can't list table '%s': %s, exit-code is %d", $table, $cmd, $exit_code);
				}
			}
		}
		$this->profiler->leave('ipt.read.counters');
		//$this->loadCounters();
		$this->ready = 1;
	}
}

function loadCounters(){
	if(!$this->loaded){
		$this->readTables();
		$this->profiler->enter('ipt.load.counters');
		$this->logger->debug("load counters %s", implode(", ", $this->tables));
		foreach($this->tables as $table){
			$filename = $this->workdir.'/list-'.$this->getListFileName($table);
			if($f = fopen($filename, "r")){
				while (!feof ($f)) {
					if($line = trim(fgets($f, 255))){
						$arr = preg_split("/[\s]+/", $line);
						$ip = $arr[1];
						$long = ip2long($ip);
						$bytes = $arr[$this->clientIsSource ? 6 : 11];
						$this->data[$long] = $bytes;
					}
				}
				fclose($f);
			}else{
				$this->logger->error("can't open '%s' for reading", $filename);
			}
		}
		$this->profiler->leave('ipt.load.counters');
		$this->loaded = 1;
	}
}

function getCounter($ip){
	if(strpos($ip, '/') > 0){
		list($addr, $mask) = explode("/", $ip);
		$long = ip2long($addr);
		if (is_numeric($mask)) { // style 10.10.10.0/24
			$m = ~((1<< (32-$mask))-1);
		}else{ // style 10.10.10.0/255.255.255.0
			$m = ip2long($mask);
		}
		$masked = $long & $m;
		$sum = 0;
		foreach($this->data as $l=>$bytes){
			if( ($l & $m) == $masked ){
				$sum += $bytes;
			}
		}
		return $sum;
	}else{
		$long = ip2long($ip);
		return isset($this->data[$long]) ? $this->data[$long] : '0';
	}
}

}
?>