<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class ArrayBeanFactory {

var $parent;
var $config;
var $beans;
var $aliases;
var $configurer;
var $anonym = 0;
var $stack = array();
var $allowCycling = false;

function ArrayBeanFactory($cfg = ''){
	if($cfg){
		if(is_array($cfg)){
			$this->setConfig($cfg);
		}else{
			$this->setConfigLocation($cfg);
		}
	}
}

function setConfig($array){
	$this->config = $array;
}

function setConfigLocation($location, $var_name = 'beans'){
	require_once($location);
	if(isset($configurer)){
		$this->configurer = $configurer;
	}
	$this->setConfig($$var_name);
}

function setParent($factory){
	$this->parent = $factory;
}

function setAllowCycling($bool){
	$this->allowCycling = $bool;
}

function hasDefinition($name){
	return isset($this->config[$name]);
}

function getDefinition($name){
	$config = $this->config[$name];
	if(isset($config['prototype'])){
		$config = array_merge_recursive($this->getDefinition($config['prototype']), $config);
	}
	return $config;
}

function hasBean($name){
	if($this->hasDefinition($name)){
		return true;
	}else{
		if(isset($this->parent)){
			return $this->parent->hasBean($name);
		}else{
			return false;
		}
	}
}

function getBean($name){
	if(isset($this->beans[$name])){
		return $this->beans[$name]; 
	}else{
		if($this->hasDefinition($name)){
			$def = $this->getDefinition($name);
			return $this->createBean($name, $def);
		}else{
			return $this->getParentBean($name);
		}
	}
}

function getParentBean($name){
	if(isset($this->parent)){
		return $this->parent->getBean($name);
	}else{
		die(sprintf("Bean '%s' not found", $name));
	}
}

function createBean($name, $definition){
	if(isset($definition['alias'])){
		return $this->getBean($definition['alias']);
	}else{
		$class = $definition['class'];
		if(!class_exists($class)){
			if(isset($definition['location'])){
				require_once($definition['location']);
			}else{
				if($name!='classLoader' && $this->hasBean('classLoader')){
					$cl = $this->getBean('classLoader');
					$cl->loadClass($class);
				}
			}
		}
		if(!in_array($name, $this->stack)){
			array_push($this->stack, $name);
			$bean = new $class;
			if($this->allowCycling){
				$this->beans[$name] = $bean;
			}
			$this->initializeBean($bean, $name, $definition);
			if(!$this->allowCycling){
				$this->beans[$name] = $bean;
			}
			array_pop($this->stack);
			return $bean;
		}else{
			die(sprintf("Bean dependency recursion: %s->%s", implode("->", $this->stack), $name));
		}
	}
}

function initializeBean($bean, $name, $definition){
	if(isset($definition['properties'])){
		$explicit = $this->setBeanProperties($bean, $definition['properties']);
	}else{
		$explicit = array();
	}
	if(!in_array($name, array('configurer', 'classLoader')) &&
		 $this->hasBean('configurer')){
		$cfgr = $this->getBean('configurer');
		$cfgr->configure($bean, $name);
	}
	if(isset($this->configurer) && $this->hasBean($this->configurer) && 
		!in_array($name, array($this->configurer, 'classLoader'))){
		$cfgr = $this->getBean($this->configurer);
		$cfgr->configure($bean, $name);
	}
	if(method_exists($bean, 'setBeanName')){
		$bean->setBeanName($name);
	}
	if(method_exists($bean, 'setBeanFactory')){
		$bean->setBeanFactory($this);
	}
	$this->autoWire($bean, $definition, $explicit);
	if(method_exists($bean, 'initBean')){
		$bean->initBean();
	}
}

function autoWire($bean, $definition, $explicit){
	foreach(get_class_methods($bean) as $method){
		if(substr($method, 0, 3) == 'set'){
			$ref = strtolower(substr($method, 3, 1)).substr($method, 4);
			//if(!(isset($definition['properties']) && isset($definition['properties'][$ref]))){
			if(!isset($explicit[$ref])){
				if($this->hasBean($ref)){
					$obj = $this->getBean($ref);
					$bean->$method($obj);
				}
			}
		}
	}
}

function setBeanProperties($bean, $properties){
	$explicit = array();
	foreach($properties as $key=>$val){
		@list($name, $type) = explode(":", $key);
		$explicit[$name] = 1;
		$value = $this->getPropertyValue($type, $val);
		$setter = 'set'.strtoupper(substr($name,0,1)).substr($name,1);
		$bean->$setter($value);
	}
	return $explicit;
}

function getPropertyValue($type, $value){
	switch($type){
		case 'ref':
			return $this->getBean($value);
		case 'parent':
			return $this->getParentBean($value);
		case 'bean':
			return $this->createBean('anomymous_'.($this->anonym++), $value);
		default:
			if(is_array($value)){
				$arr = array();
				foreach($value as $key=>$val){
					@list($name, $type) = explode(":", $key);
					$arr[$name] = $this->getPropertyValue($type, $val);
				}
				return $arr;
			}else{
				return $value;
			}
	}
}

function test(){
	foreach($this->config as $k=>$v){
		$this->getBean($k);
	}
}

}
?>