<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class AccountStatesDAO {

var $connection;
var $queries = array(
	'drop_actual_changes'=>"DROP TABLE IF EXISTS actual_changes",
	'create_actual_changes'=>"CREATE TEMPORARY TABLE actual_changes (account int not null primary key, amount bigint not null) ENGINE=Memory",
	'insert_actual_changes'=>"INSERT INTO actual_changes
		SELECT account, sum(amount) FROM (
		SELECT src as account, -sum(amount) amount FROM movements WHERE calendar_period=%s AND date <= FROM_UNIXTIME(%s) GROUP BY src
		UNION
		SELECT dst as account,  sum(amount) amount FROM movements WHERE calendar_period=%s AND date <= FROM_UNIXTIME(%s) GROUP BY dst
		) z GROUP BY account",
	'drop_future_changes'=>"DROP TABLE IF EXISTS future_changes",
	'create_future_changes'=>"CREATE TEMPORARY TABLE future_changes (
		account int not null primary key,
		amount bigint not null) ENGINE=Memory",
	'insert_future_changes'=>"INSERT INTO future_changes
		SELECT account, sum(amount) FROM (
		SELECT src as account, -sum(floor(amount * price / unit_size)) as amount
		FROM resource_uses
		WHERE calendar_period=%s
		AND movement IS NULL
		AND (src IS NOT NULL)
		AND (dst IS NOT NULL)
		GROUP BY src
		UNION
		SELECT dst as account, sum(floor(amount * price / unit_size)) as amount
		FROM resource_uses
		WHERE calendar_period=%s
		AND movement IS NULL
		AND (src IS NOT NULL)
		AND (dst IS NOT NULL)
		GROUP BY dst
		) z GROUP BY account",
	'load'=>"SELECT a.id as account,
		ifnull(cas.state_begin,0) begin,
		ifnull(ac.amount,0) as actual,
		ifnull(fc.amount,0) as virtual
		FROM accounts a
		LEFT JOIN calendar_account_states cas ON cas.account = a.id AND cas.calendar_period = %s
		LEFT JOIN actual_changes ac ON ac.account=a.id
		LEFT JOIN future_changes fc ON fc.account=a.id",
	'load2'=>"SELECT a.id as account, ifnull(cas.state_end,0) virtual
		FROM accounts a
		LEFT JOIN calendar_account_states cas ON cas.account = a.id AND cas.calendar_period = %s",
	'load3'=>"SELECT a.account, ifnull(cas.state_end,0) virtual
		FROM (
			SELECT account FROM clients WHERE id >= [client_from] AND id < [client_to] UNION
			SELECT account FROM bonus_accounts WHERE client >= [client_from] AND client < [client_to] UNION
			SELECT account FROM system_accounts
		) a
		LEFT JOIN calendar_account_states cas ON cas.account = a.account AND cas.calendar_period = %s",
	'future_states'=>"REPLACE INTO calendar_account_states (calendar_period, account, state_begin, state_end)
		SELECT %s, a.id as account,
		ifnull(cas.state_begin,0),
		ifnull(cas.state_begin,0) + ifnull(ac.amount,0) + ifnull(fc.amount,0)
		FROM accounts a
		LEFT JOIN calendar_account_states cas ON cas.account = a.id AND cas.calendar_period = %s
		LEFT JOIN actual_changes ac ON ac.account=a.id
		LEFT JOIN future_changes fc ON fc.account=a.id
		ORDER BY a.id
		FOR UPDATE",
	'finalize_states'=>"REPLACE INTO calendar_account_states (calendar_period, account, state_begin, state_end)
		SELECT %s, a.id as account,
		ifnull(cas.state_begin,0),
		ifnull(cas.state_begin,0) - ifnull(o.amount,0) + ifnull(i.amount,0)
		FROM accounts a
		LEFT JOIN calendar_account_states cas ON cas.account = a.id AND cas.calendar_period = %s
		LEFT JOIN (SELECT src acc, sum(amount) amount FROM movements WHERE calendar_period=%s GROUP BY src) o ON o.acc = a.id
		LEFT JOIN (SELECT dst acc, sum(amount) amount FROM movements WHERE calendar_period=%s GROUP BY dst) i ON i.acc = a.id",
	'startup_states'=>"INSERT IGNORE INTO calendar_account_states (calendar_period, account, state_begin, state_end)
		SELECT %s, id, 0, 0 FROM accounts",
	'move_states'=>"REPLACE INTO calendar_account_states (calendar_period, account, state_begin, state_end)
		SELECT %s, account, state_end, state_end FROM calendar_account_states WHERE calendar_period=%s",
);

var $splitter;

function setConnection($connection){
	$this->connection = $connection;
}

function setSplitter($splitter){
	$this->splitter = $splitter;
}

function prepare($cp, $time){
	$this->connection->query($this->queries['drop_actual_changes']);
	$this->connection->query($this->queries['create_actual_changes']);
	$this->connection->query($this->queries['insert_actual_changes'], $cp, $time, $cp, $time);
	$this->connection->query($this->queries['drop_future_changes']);
	$this->connection->query($this->queries['create_future_changes']);
	$this->connection->query($this->queries['insert_future_changes'], $cp, $cp);
}

function iterator($cp, $time){
	$q = $this->splitter->patch($this->queries['load3'], $this->queries['load2']);
	return $this->connection->select($q, $cp);
}

function futureStates($cpid, $time){
	$this->prepare($cpid, $time);
	$this->connection->update($this->queries['future_states'], $cpid, $cpid);
}

function splitFutureStates($cpid){
	$this->connection->query($this->queries['drop_future_changes']);
	$this->connection->query($this->queries['create_future_changes']);
	$nosplit = sprintf($this->queries['insert_future_changes'], $cpid, $cpid);
	$split = $cpid.' AND client >= [client_from] AND client < [client_to]';
	$split = sprintf($this->queries['insert_future_changes'], $split, $split);
	$q = $this->splitter->patch($split, $nosplit);
	$this->connection->query($q);
	$this->connection->update($this->queries['future_states'], $cpid, $cpid);
}

function finalizeStates($cpid){
	$this->connection->update($this->queries['finalize_states'], $cpid, $cpid, $cpid, $cpid);
}

function startupStates($cpid){
	$this->connection->update($this->queries['startup_states'], $cpid);
}

function moveStates($next, $prev){
	$this->connection->update($this->queries['move_states'], $next, $prev);
}

}

?>