<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class IpUseLogDAO {

var $profiler;
var $connection;
var $queries = array(
	'insert'=>"INSERT INTO ip_use_log (ip_use, ip_use_interval, bytes) VALUES %s",
	'insert_minutes'=>"INSERT INTO ip_use_time (ip_use, from_time, to_time, bytes) VALUES ",
	//     ,   LOAD DATA [LOCAL] INFILE ...
	'load_minutes'=>"LOAD DATA LOCAL INFILE '%s' INTO TABLE ip_use_time",
	'select_day'=>"SELECT iud.ip_use, iud.day, iud.bytes
		FROM ip_use_days iud
		LEFT join ip_uses iu ON iu.id=iud.ip_use
		LEFT JOIN personal_period_resource_uses ppru ON ppru.id=iu.personal_period_resource_use
		WHERE ppru.personal_period=%s AND iud.day='%s'",
	'select_hour'=>"SELECT iuh.ip_use, iuh.hour, iuh.bytes
		FROM ip_use_hours iuh
		LEFT join ip_uses iu ON iu.id=iuh.ip_use
		LEFT JOIN personal_period_resource_uses ppru ON ppru.id=iu.personal_period_resource_use
		WHERE ppru.personal_period=%s AND iuh.hour='%s'",
	'insert_day'=>"INSERT INTO ip_use_days (ip_use, day, bytes) VALUES (%s, '%s', %s)",
	'update_day'=>"UPDATE ip_use_days SET bytes=%s WHERE ip_use=%s AND day='%s'",
	'insert_hour'=>"INSERT INTO ip_use_hours (ip_use, hour, bytes) VALUES (%s, '%s', %s)",
	'update_hour'=>"UPDATE ip_use_hours SET bytes=%s WHERE ip_use=%s AND hour='%s'",
	'log_hour'=>"REPLACE INTO ip_use_hours (ip_use, hour, bytes) 
		SELECT ip_use, '%s' as hour, sum(bytes) as bytes 
		FROM ip_use_time
		WHERE from_time >='%s' AND from_time < '%s'
		GROUP BY ip_use",
	'log_day'=>"REPLACE INTO ip_use_days (ip_use, day, bytes) 
		SELECT ip_use, '%s' as day, sum(bytes) as bytes 
		FROM ip_use_hours
		WHERE hour >='%s' AND hour < '%s'
		GROUP BY ip_use",
);
var $insert_item = "(%s, %s, %s)";
var $flush = '';

function setConnection($connection){
	$this->connection = $connection;
}

function setProfiler($obj){
	$this->profiler = $obj;
}

function spreadLog($filename, $time){
	$this->loadFile($filename);
	$this->logHour($time);
	$this->logDay($time);
}

function logHour($time){
	$this->profiler->enter('iul.log.hour');
	$from = date("Y-m-d H:00:00", $time);
	$to = date("Y-m-d H:00:00", $time + 60*60);
	$this->connection->update($this->queries['log_hour'], $from, $from, $to);
	$this->profiler->leave('iul.log.hour');
}

function logDay($time){
	$this->profiler->enter('iul.log.day');
	$from = date("Y-m-d 00:00:00", $time);
	$to = date("Y-m-d 00:00:00", $time + 60*60*24);
	$this->connection->update($this->queries['log_day'], $from, $from, $to);
	$this->profiler->leave('iul.log.day');
}

function multiInsert($records){
	$values = array();
	foreach($records as $record){
		$values[] = sprintf($this->insert_item, $record[0], $record[1], $record[2]);
	}
	$this->connection->nextAsync();
	$this->connection->update($this->queries['insert'], implode(",", $values));
}

function listDay($ppid, $time){
	return $this->connection->select($this->queries['select_day'], $ppid, date("Y-m-d", $time));
}

function listHour($ppid, $time){
	return $this->connection->select($this->queries['select_hour'], $ppid, date("Y-m-d H:00:00", $time));
}

function insertDay($iu_id, $time, $bytes){
	$this->profiler->enter('iul.insert.day');
	$this->connection->nextAsync();
	$this->connection->insert($this->queries['insert_day'], $iu_id, date("Y-m-d", $time), $bytes);
	$this->profiler->leave('iul.insert.day');
}

function updateDay($iu_id, $time, $bytes){
	$this->profiler->enter('iul.update.day');
	$this->connection->nextAsync();
	$this->connection->update($this->queries['update_day'], $bytes, $iu_id, date("Y-m-d", $time));
	$this->profiler->leave('iul.update.day');
}

function insertHour($iu_id, $time, $bytes){
	$this->profiler->enter('iul.insert.hour');
	$this->connection->nextAsync();
	$this->connection->insert($this->queries['insert_hour'], $iu_id, date("Y-m-d H:00:00", $time), $bytes);
	$this->profiler->leave('iul.insert.hour');
}

function updateHour($iu_id, $time, $bytes){
	$this->profiler->enter('iul.update.hour');
	$this->connection->nextAsync();
	$this->connection->update($this->queries['update_hour'], $bytes, $iu_id, date("Y-m-d H:00:00", $time));
	$this->profiler->leave('iul.update.hour');
}

function multiInsertMinutes($records){
	//$this->profiler->enter('iul.insert.minutes');
	//$values = array();
	foreach($records as $record){
	//	$values[] = sprintf("(%s, '%s', '%s', %s)", $record[0], date("Y-m-d H:i:s", $record[1]), date("Y-m-d H:i:s", $record[2]), $record[3]);
		$this->flush .= sprintf("%s\t%s\t%s\t%s\n", $record[0], date("Y-m-d H:i:s", $record[1]), date("Y-m-d H:i:s", $record[2]), $record[3]);
	}
	//$this->connection->nextAsync();
	//$this->connection->update($this->queries['insert_minutes'].' '.implode(",", $values));
	//$this->profiler->leave('iul.insert.minutes');
}

function loadFile($filename){
	$this->connection->update($this->queries['load_minutes'], $filename);
}

function flushFile($filename){
	$this->profiler->enter('iul.flush.file');
	if($f = fopen($filename, 'a+')){
		fwrite($f, $this->flush);
		fclose($f);
	}
	$this->profiler->leave('iul.flush.file');
}

}
?>