<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class PersonalPeriodResourceUsesDAO {

var $profiler;
var $connection;
var $splitter;
var $queries = array(
	'select'=>"SELECT ppru.id, ppru.personal_period, ppru.resource_use, ppru.amount
		FROM personal_period_resource_uses ppru
		LEFT JOIN resource_uses ru ON ru.id=ppru.resource_use 
		WHERE ru.calendar_period=%s AND ru.movement IS NULL",
	'update'=>"UPDATE personal_period_resource_uses SET amount=%s WHERE id=%s",
	'insert'=>"INSERT INTO personal_period_resource_uses (personal_period, resource_use, amount) VALUES (%s, %s, %s)",
	'correct_prepare'=>"CREATE TEMPORARY TABLE ppru_correct ENGINE=Memory
		SELECT ppru.id, sum(iu.bytes) as bytes, ppru.amount
		FROM ip_uses iu
		LEFT JOIN personal_period_resource_uses ppru ON ppru.id=iu.personal_period_resource_use
		LEFT JOIN personal_periods pp ON pp.id=ppru.personal_period
		WHERE pp.period_end > SUBDATE(NOW(), INTERVAL 1 HOUR)
		GROUP BY ppru.id
		HAVING sum(iu.bytes)!=ppru.amount",
	'correct'=>"UPDATE ppru_correct pc
		LEFT JOIN personal_period_resource_uses ppru ON ppru.id=pc.id
		SET ppru.amount=pc.bytes",
);

function setConnection($connection){
	$this->connection = $connection;
}

function setProfiler($obj){
	$this->profiler = $obj;
}

function setSplitter($splitter){
	$this->splitter = $splitter;
}

function iterator($calendar_period_id){
	$q = $this->queries['select'];
	$q = $this->splitter->patch($q." AND ru.client >= [client_from] AND ru.client < [client_to]", $q);
	return $this->connection->select($q, $calendar_period_id);
}

function update($id, $amount){
//	$this->profiler->enter('dao.ppru.update');
	$this->connection->nextAsync();
	$res = $this->connection->update($this->queries['update'], $amount, $id);
//	$this->profiler->leave('dao.ppru.update');
	return $res;
}

function insert($pp_id, $ru_id, $amount){
	return $this->connection->insert_auto($this->queries['insert'], $pp_id, $ru_id, $amount);
}

function correct(){
	$this->connection->update($this->queries['correct_prepare']);
	$this->connection->update($this->queries['correct']);
}

}
?>