<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class PersonalPeriodsDAO {

var $connection;
var $splitter;
var $queries = array(
	'select'=>"SELECT id, client, connection_type, tv_type, state, marker, UNIX_TIMESTAMP(period_begin) as period_begin, UNIX_TIMESTAMP(period_end) as period_end
			FROM personal_periods WHERE marker=1",
	'select_prev'=>"SELECT id, client, connection_type, tv_type, state, marker, UNIX_TIMESTAMP(period_begin) as period_begin, UNIX_TIMESTAMP(period_end) as period_end
			FROM personal_periods WHERE next=%s",
	'select_by_client'=>"SELECT id, client, connection_type, tv_type, state, marker, UNIX_TIMESTAMP(period_begin) as period_begin, UNIX_TIMESTAMP(period_end) as period_end
			FROM personal_periods WHERE client=%s AND marker=1",
	'select_last_marked'=>"SELECT id, client, connection_type, tv_type, state, marker, UNIX_TIMESTAMP(period_begin) as period_begin, UNIX_TIMESTAMP(period_end) as period_end
			FROM personal_periods WHERE id = (SELECT min(id) FROM personal_periods WHERE client=%s AND next IS NULL)",
	'select_next_candidate'=>"SELECT id, client, connection_type, tv_type, state, marker, UNIX_TIMESTAMP(period_begin) as period_begin, UNIX_TIMESTAMP(period_end) as period_end
			FROM personal_periods WHERE client=%s AND (FROM_UNIXTIME(%s) BETWEEN period_begin AND period_end) AND (next IS NULL) AND (state=0) ORDER BY period_begin LIMIT 1",
	'append'=>"UPDATE personal_periods SET next=%s WHERE id=%s",
	'insert'=>"INSERT INTO personal_periods	(client, connection_type, tv_type, period_begin, period_end)	VALUES (%s, %s, %s, FROM_UNIXTIME(%s), FROM_UNIXTIME(%s))",
	'change_state'=> "UPDATE personal_periods SET state=%s WHERE id=%s",
	'prolong'=> "UPDATE personal_periods SET period_end=ADDDATE(period_end, INTERVAL %s SECOND) WHERE id=%s",
	'change_marker'=>"UPDATE personal_periods SET marker=%s WHERE id=%s",
	'get_used_connection_types'=>"SELECT DISTINCT connection_type FROM personal_periods WHERE client=%s",
);

function setConnection($connection){
	$this->connection = $connection;
}

function setSplitter($splitter){
	$this->splitter = $splitter;
}

function iterator(){
	$q = $this->queries['select'];
	$q = $this->splitter->patch($q." AND client >= [client_from] AND client < [client_to]", $q);
	return $this->connection->select($q);
}

function getByClient($clid) {
	$it = $this->connection->select($this->queries['select_by_client'], $clid);
	if($p = $it->next()){
		return $p;
	}
}

function getPrev($pid) {
	$it = $this->connection->select($this->queries['select_prev'], $pid);
	if($p = $it->next()){
		return $p;
	}
}


function getLastMarked($clid) {
	$it = $this->connection->select($this->queries['select_last_marked'], $clid);
	if($p = $it->next()){
		return $p;
	}
}

function getNextCandidate($clid, $time) {
	$it = $this->connection->select($this->queries['select_next_candidate'], $clid, $time);
	if($p = $it->next()){
		return $p;
	}
}

function insert($clid, $con_type_id, $tv_type_id, $period_begin, $period_end){
	return $this->connection->insert_auto($this->queries['insert'], $clid, $con_type_id, $tv_type_id ? $tv_type_id : 'null', $period_begin, $period_end);
}

function append($prev, $next){
	return $this->connection->update($this->queries['append'], $next, $prev);
}

function changeState($pid, $state){
	return $this->connection->update($this->queries['change_state'], $state, $pid);
}

function prolong($pid, $seconds){
	return $this->connection->update($this->queries['prolong'], $seconds, $pid);
}

function changeMarker($pid, $value){
	return $this->connection->update($this->queries['change_marker'], $value ? $value : "NULL", $pid);
}

function getUsedConnectionTypes($clid){
	return $this->connection->select($this->queries['get_used_connection_types'], $clid);
}


}
?>