<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class MessageParser {

var $mbox;
var $msg;
var $logger;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('message.parser');
}

function setMbox($mbox){
	$this->mbox = $mbox;
}

function parse($info){
	$this->msg = array(
		'number' => $info->msgno,
		'msg_id' => isset($info->message_id) ? $info->message_id : '',
		'in_reply_to' => isset($info->in_reply_to) ? $info->in_reply_to : '',
		'subject' => $this->mime_decode(isset($info->subject) ? $info->subject : ''),
		'addr_from' => $this->mime_decode(isset($info->from) ? $info->from : ''),
		'addr_to' => $this->mime_decode(isset($info->to) ? $info->to : ''),
		'date' => isset($info->date) ? strtotime($info->date) : '',
		'reply_to' => '',
		'answer_id' => '',
		'attachments'=>array(),
	);
	if(!$this->msg['date']){
		$this->msg['date'] = time();
	}
	$er = error_reporting(E_ALL ^ E_NOTICE);
	$header = imap_fetchheader($this->mbox, $this->msg['number']);
	error_reporting(E_ALL ^ E_NOTICE);
	if(preg_match('|^Reply-To:(.+)$|mi', $header, $m)){
		$this->msg['reply_to'] = $this->mime_decode(trim($m[1]));
	}
	if(preg_match('|^X-Answer-Id:(.+)$|mi', $header, $m)){
		$this->msg['answer_id'] = trim($m[1]);
	}
	$struct = imap_fetchstructure($this->mbox, $this->msg['number']);
	$this->processStruct($struct);
	if(!isset($this->msg['body'])){
		$this->msg['body'] = '';
	}
	return $this->msg;
}

function mime_decode($str){
	$result = '';
	foreach(imap_mime_header_decode($str) as $segment){
		if($segment->charset == 'default'){
			$result .= $segment->text;
		}else{
			$result .= @iconv($segment->charset, 'windows-1251', $segment->text);
		}
	}
	return $result;
}

function getMimeType($structure) {
	$primary_mime_type = array("text", "multipart", "message", "application", "audio", "image", "video", "other");
	if($structure->subtype) {
		return $primary_mime_type[(int) $structure->type] . '/' .strtolower($structure->subtype);
	}
	return "text/plain";
}

function packParams($params){
	$result = array();
	if(is_array($params)){
		foreach($params as $obj){
			$result[strtoupper($obj->attribute)] = $obj->value;
		}
	}
	return $result;
}

function getBody($struct, $part){
	$body = imap_fetchbody($this->mbox, $this->msg['number'], $part);
	if($struct->encoding == 3) {
		$body = imap_base64($body);
	} else if($struct->encoding == 4) {
		$body = imap_qprint($body);
	}
	return $body;
}

function getTextBody($struct, $part){
	$body = $this->getBody($struct, $part);
	$params = $this->packParams($struct->parameters);
	if(isset($params['CHARSET'])){
		$body = @iconv($params['CHARSET'], 'windows-1251', $body);
	}
	return $body;
}

function processStruct($struct, $path=''){
	switch($struct->type){
		case 0: // text
			$this->processText($struct, $path);
			break;
		case 1: //multipart
			$this->processMultipart($struct, $path);
			break;
		case 2: // message
			$this->processMessage($struct, $path);
			break;
		default: // attachment
			$this->processAttachment($struct, $path);
			break;
	}
}

function processText($struct, $path){
	if(isset($this->msg['body'])){
		$this->processAttachment($struct, $path);
	}else{
		$body = $this->getTextBody($struct, $path ? $path : 1);
		if(stristr($struct->subtype, 'html')){
			$this->processAttachment($struct, $path, 'body.html');
			$this->msg['body'] = 'HTML body saved in attachment.';
		}else{
			if(strlen($body) > 65535){
				$this->processAttachment($struct, $path, 'body.txt');
				$this->msg['body'] = 'Body too large. Saved in attachment.';
			}else{
				$this->msg['body'] = $body;
			}
		}
	}
}

function processMultipart($struct, $path){
	switch(strtolower($struct->subtype)){
		case 'alternative':
			$this->processAlternative($struct, $path);
			break;
		case 'mixed':
		case 'related':
			$this->processMixed($struct, $path);
			break;
		case 'report':
			$this->processReport($struct, $path);
			break;
		default:
			$this->msg['body'] = 'unsupported multipart content type: '.$struct->subtype;
			$this->logger->warning($this->msg['msg_id'].' has '.$this->msg['body']);
	}
}

function processAlternative($struct, $path){
	if(isset($struct->parts[0])){
		$p = $path ? $path.'.' : '';
		foreach($struct->parts as $s){
			if($this->getMimeType($s)=='text/plain'){
				$this->processStruct($s, $p.'1');
				return;
			}
		}
		$this->processStruct($struct->parts[0], $p.'1');
	}else{
		$this->msg['body'] = 'mallformed multipart alternative';
		$this->logger->warning($this->msg['msg_id'].' has '.$this->msg['body']);
	}
}

function processMixed($struct, $path){
	$p = $path ? $path.'.' : '';
	foreach($struct->parts as $i=>$s){
		$this->processStruct($s, $p.($i+1));
	}
}

function processReport($struct, $path){
	if(isset($struct->parts[0])){ 
		$p = $path ? $path.'.' : '';
		$this->processStruct($struct->parts[0], $p.'1');
	}else{
		$this->msg['body'] = 'mallformed REPORT';
		$this->logger->warning($this->msg['msg_id'].' has '.$this->msg['body']);
	}
}

function processAttachment($struct, $path, $name=''){
	$part = $path ? $path : 1;
	$body = $this->getTextBody($struct, $part);
	$type = $this->getMimeType($struct);
	$dparams = $this->packParams($struct->dparameters);
	$params = $this->packParams($struct->parameters);
	if(!$name && isset($dparams['NAME'])){
		$name = $this->mime_decode($dparams['NAME']);
	}
	if(!$name && isset($dparams['FILENAME'])){
		$name = $this->mime_decode($dparams['FILENAME']);
	}
	if(!$name && isset($params['NAME'])){
		$name = $this->mime_decode($params['NAME']);
	}
	if(!$name && isset($params['FILENAME'])){
		$name = $this->mime_decode($params['FILENAME']);
	}
	if(!$name){
		$name = 'attachment-'.$part;
	}
	$this->msg['attachments'][] = array(
		'type'=>$type,
		'name'=>$name,
		'body'=>$body,
		'size'=>strlen($body),
	);
}

function processMessage($struct, $path){
	switch(strtolower($struct->subtype)){
		case 'rfc822':
			$this->processRfc822($struct, $path);
			break;
		case 'delivery-status':
			// skip
			break;
		default:
			$this->msg['body'] = 'unsupported embeded message type: '.$struct->subtype;
			$this->logger->warning($this->msg['msg_id'].' has '.$this->msg['body']);
	}
}

function processRfc822($struct, $path){
	if(isset($struct->parts[0])){
		$p = $path ? $path.'.' : '';
		$this->processStruct($struct->parts[0], $p.'1');
	}else{
		$this->msg['body'] = 'mallformed rfc822 message';
		$this->logger->warning($this->msg['msg_id'].' has '.$this->msg['body']);
	}
}

}

?>