<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class AccountingRulesManager {

var $logger;
var $workDirOption;
var $workDir;
var $chains;

function setLogFactory($logFactory){
	$this->logger = $logFactory->getLogger('accounting.rules');
}

function setWorkDirOption($workDirOption){
	$this->workDirOption = $workDirOption;
}

function initBean(){
	$this->workdir = $this->workDirOption->getValue();
	if(substr($this->workdir, -1, 1)!='/'){
		$this->workdir .= '/';
	}
	$this->chains = array();
}

function load($chain, $reset = false){
	$this->chains[$chain] = array();
	$filename = $this->workdir.'list-'.$chain;
	$this->logger->debug("listing '%s' chain", $chain);
	$z = $reset ? "-Z" : "";
	$cmd = sprintf("iptables %s -L %s -nvx 2>&1 | grep 0.0.0.0/0 > %s", $z, $chain, $filename);
	$this->logger->debug($cmd);
	exec($cmd, $out, $exit_code);
	if(!$exit_code){
		if($f = fopen($filename, "r")){
			while (!feof ($f)) {
				$line = fgets($f, 255);
				if(preg_match('/\s+(\d+)\s+(\d+)\s+(\S*)\s+all\s+\-\-\s+\*\s+\*\s+(\S+)\s+(\S+)\s.*/', $line, $mat)){
					list($all, $packets, $bytes, $target, $src, $dst) = $mat;
					if($src){
						$is_source = $src!='0.0.0.0/0';
						$ip = $is_source ? $src : $dst;
						$this->chains[$chain][$is_source][$ip][$target] = $bytes;
					}
				}
			}
			fclose($f);
		}else{
			$this->logger->error("can't open '%s' for reading", $filename);
		}
	}else{
		if(($exit_code==1) && empty($out)){
			$this->logger->debug("chain '%s' is empty", $chain);
		}else{
			$this->logger->error("can't list chain '%s', exit-code is %d, output: %s", $chain, $exit_code, implode(", ", $out));
		}
	}
}

function begin(){
	$this->logger->debug("Begining DIFF session");
	$this->script = array();
	$this->script[] = "*filter";
}

function flush($chain){
	$this->script[] = sprintf("[0:0] -F %s", $chain);
}

function setupCounter($chain, $is_source, $ip, $block){
	$is_net = strstr($ip,'/');
	$target = $block ? 'REJECT' : ($is_net ? '' : 'RETURN');
	if(isset($this->chains[$chain][$is_source][$ip][$target])){
		unset($this->chains[$chain][$is_source][$ip][$target]);
	}else{
		$location = $is_source ? '-s': '-d';
		if($is_net){
			$action = 'I';
			$target = $block ? " -j REJECT --reject-with icmp-net-prohibited" : "";
		}else{
			$action = 'A';
			$target = $block ? " -j REJECT --reject-with icmp-host-prohibited" : " -j RETURN";
		}
		$this->script[] = sprintf("[0:0] -%s %s %s %s %s", $action, $chain, $location, $ip, $target);
	}
}

function end(){
	$this->logger->debug("Finalizing DIFF session");
	foreach($this->chains as $chain=>$c_data){
		foreach($c_data as $is_source=>$sr_data){
			foreach($sr_data as $ip=>$ip_data){
				$is_net = strstr($ip,'/');
				foreach($ip_data as $target=>$bytes){
					$location = $is_source ? '-s': '-d';
					switch($target){
						case 'REJECT':
							$target = sprintf(" -j REJECT --reject-with icmp-%s-prohibited", $is_net ? 'net': 'host');
							break;
						case 'RETURN';
							$target = " -j RETURN";
							break;
					}
					$this->script[] = sprintf("[0:0] -D %s %s %s %s", $chain, $location, $ip, $target);
				}
			}
		}
	}
	$this->script[] = "COMMIT\n";
	$count = count($this->script) - 2;
	$filename = $this->workdir.'accounting-diff';
	if($file = @fopen($filename, "w+")){
		$this->logger->debug("Writing diff to '%s'", $filename);
		fputs($file, implode("\n", $this->script));
		fclose($file);
		if($count > 0){
			$this->logger->debug("Applying %s difference rules", $count);
			$cmd = sprintf("iptables-restore --noflush 2>&1 < %s", $filename);
			$this->logger->debug($cmd);
			exec($cmd, $out, $exit_code);
			if($exit_code){
				$this->logger->error("restoring failed, exit code is %d, output: %s", $exit_code, implode(",", $out));
			}
		}else{
			$this->logger->debug("No differences");
		}
	}else{
		$this->logger->error("Can't write to '%s'", $filename);
	}
}

function getCounter($chain, $is_source, $ip){
	$target = strstr($ip,'/') ? '' : 'RETURN';
	if(isset($this->chains[$chain][$is_source][$ip][$target])){
		return $this->chains[$chain][$is_source][$ip][$target];
	}else{
		return '0';
	}
}

function show(){
	print_r($this->chains);
}

}
?>