<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class CalendarPeriodsManager {

var $logger;
var $calendarPeriodsDAO;
var $accountStatesDAO;
var $timeService;
var $current;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('calendar.period.manager');
}

function setCalendarPeriodsDAO($obj){
	$this->calendarPeriodsDAO = $obj;
}

function setAccountStatesDAO($obj){
	$this->accountStatesDAO = $obj;
}

function setTimeService($obj){
	$this->timeService = $obj;
}

function getCurrent(){
	return $this->current;
}

function initBean(){
	if(!($this->current = $this->calendarPeriodsDAO->getCurrent())){
		if($this->current = $this->calendarPeriodsDAO->getLastMarked()){
			$this->current['marker'] = 1;
			$this->calendarPeriodsDAO->changeMarker($this->current['id'], '1');
		}else{
			$this->firstCalendarPeriod();
		}
	}
	$this->logger->debug("current calendar period is [%s]  [%s - %s]", $this->current['id'],
		date("Y-m-d", $this->current['period_begin']),
		date("Y-m-d", $this->current['period_end'])
	);
}

function firstCalendarPeriod(){
	$this->logger->debug("creating first calendar period");
	$this->current = $this->createCalendarPeriod();
	$this->calendarPeriodsDAO->changeMarker($this->current['id'], '1');
	$this->calendarPeriodsDAO->changeState($this->current['id'], '1');
	$this->startupStates($this->current['id']);
}

function createCalendarPeriod(){
	$time = $this->timeService->getTime();
	//     
	$period_end = mktime(date("H", $time), 0, 0, date("m", $time) + 1, date("d", $time), date("Y",$time));
	//   
	$id = $this->calendarPeriodsDAO->insert($time, $period_end);
	$this->logger->debug("calendar period created [%s - %s]", date("Y-m-d", $time), date("Y-m-d", $period_end));
	return array (
		'id'=>$id,
		'period_begin'=>$time,
		'period_end'=>$period_end,
		'state'=>'0',
		'marker'=>'',
		'next'=>'',
	);
}

function nextCalendarPeriod(){
	$prev = $this->current;
	if($this->current = $this->calendarPeriodsDAO->getNextCandidate($this->timeService->getTime())){
		$this->logger->debug("found candidate [%s] for next calendar period", $this->current['id']);
	}else{
		$this->logger->debug("creating next calendar period");
		$this->current = $this->createCalendarPeriod();
	}
	//      
	$this->moveMarker($prev['id'], $this->current['id']);
	//        
	$this->moveStates($this->current['id'], $prev['id']);
	$this->calendarPeriodsDAO->changeState($this->current['id'], '1');
}

function closeCurrentCalendarPeriod(){
	//        
	$this->finalizeStates($this->current['id']);
	$this->calendarPeriodsDAO->changeState($this->current['id'], '2');
}

function moveMarker($from, $to){
	$this->calendarPeriodsDAO->changeMarker($from, '');
	$this->calendarPeriodsDAO->changeMarker($to, '1');
	$this->calendarPeriodsDAO->append($from, $to);
}

function startupStates($cpid){
	$this->logger->debug("startup accounts states in calendar period [%s]", $cpid);
	$this->accountStatesDAO->startupStates($cpid);
}

function finalizeStates($cpid){
	$this->logger->debug("finalize accounts states in calendar period [%s]", $cpid);
	$this->accountStatesDAO->finalizeStates($cpid);
}

function moveStates($next, $prev){
	$this->logger->debug("initilizing accounts states in calendar period [%s] by final states of period [%s]", $next, $prev);
	$this->accountStatesDAO->moveStates($next, $prev);
}

}
?>