<?/* author: Ponomarev Denis <ponomarev@gmail.com> */
class ChargeDriverDrWeb {

var $logger;
var $optionsManager;
var $options;
var $ready;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('charge.drweb');
}

function setOptionsManager($obj){
	$this->optionsManager = $obj;
}

function initBean(){
	$this->options = array();
	$this->ready = true;
	foreach(array(
		'DRWEB_API_URL',
		'DRWEB_USER',
		'DRWEB_PASSWORD',
		'DRWEB_GROUP'
	) as $id){
		if(($opt = $this->optionsManager->getOption($id)) && $opt['actual']){
			$this->options[$id] = trim($opt['value']);
		}else{
			$this->ready = false;
			$this->logger->warning("%s is not set");
		}
	}
}

function start($client, $period){
	if($init_msg = $this->init($client['login'], $period['period_end'])){
		if(strstr($init_msg, 'already exists')){
			if($prolong_msg = $this->prolong($client['login'], $period['period_end'])){
				$this->logger->error("can't prolong to [%s] drweb period for [%s]: %s", date("Y-m-d H:i:s", $period['period_end']), $client['login'], $prolong_msg);
				return false;
			}else{
				$this->logger->debug("OK prolong to [%s] drweb period for [%s]", date("Y-m-d H:i:s", $period['period_end']), $client['login']);
				return true;
			}
		}else{
			$this->logger->error("can't init to [%s] drweb period for [%s]: %s", date("Y-m-d H:i:s", $period['period_end']), $client['login'], $init_msg);
			return false;
		}
	}else{
		$this->logger->debug("OK init to [%s] drweb period for [%s]", date("Y-m-d H:i:s", $period['period_end']), $client['login']);
		return true;
	}
}

function query($query){
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $query);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array("Authorization: Basic ".base64_encode(sprintf("%s:%s", $this->options['DRWEB_USER'], $this->options['DRWEB_PASSWORD']))));
	ob_start();
	curl_exec($ch);
	$result = trim(ob_get_contents());
	ob_end_clean();
	$this->result = $result;
	return $result;
}

function init($login, $end){
	$query = sprintf('%s/add-customer.ds?count=1&id=%s&pgrp=%s&year=%s&month=%s&day=%s',
		$this->options['DRWEB_API_URL'], $login, $this->options['DRWEB_GROUP'], date("Y", $end), date("m", $end), date("d", $end));
	$this->logger->debug('init: %s', $query);
	$result = $this->query($query);
	if(!strstr($result, 'rc="true"')){
		if(preg_match('|message="([^"]*)"|imxs', $result, $m)){
			return $m[1];
		}else{
			return 'error';
		}
	}
}

function prolong($login, $end){
	$query = sprintf('%s/set-expiration.ds?id=%s&year=%s&month=%s&day=%s',
		$this->options['DRWEB_API_URL'], $login, date("Y", $end), date("m", $end), date("d", $end));
	$this->logger->debug('prolong: %s', $query);
	$result = $this->query($query);
	if(!strstr($result, 'rc="true"')){
		if(preg_match('|message="([^"]*)"|imxs', $result, $m)){
			return $m[1];
		}else{
			return 'error';
		}
	}
}

}
?>