<?/* author: Ponomarev Denis <ponomarev@gmail.com> */
class ChargeDriverDrWeb2 {

var $logger;
var $optionsManager;
var $options;
var $ready;
var $connection;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('charge.drweb');
}

function setOptionsManager($obj){
	$this->optionsManager = $obj;
}

function setConnection($obj){
	$this->connection = $obj;
}

function initBean(){
	$this->options = array();
	$this->ready = true;
	foreach(array(
		'DRWEB_API_URL',
		'DRWEB_USER',
		'DRWEB_PASSWORD',
		'DRWEB_GROUP'
	) as $id){
		if(($opt = $this->optionsManager->getOption($id)) && $opt['actual']){
			$this->options[$id] = trim($opt['value']);
		}else{
			$this->ready = false;
			$this->logger->warning("%s is not set");
		}
	}
}

function start($sub, $type, $client, $period){
	$sel = $this->connection->select("SELECT s.drweb_user, t.group_name FROM drweb_subscriptions s LEFT JOIN drweb_types t ON t.id=s.drweb_type WHERE s.subscription='%s'", $sub['id']);
	if($arr = $sel->next()){
		$user = $arr['drweb_user'];
		$group = $arr['group_name'];
	}else{
		$user = $client['login'];
		$group = $this->options['DRWEB_GROUP'];
	}
	return $this->startDrweb($user, $group, $period['period_end']);
}

function startDrweb($user, $group, $end){
	if($init_msg = $this->init($user, $group, $end)){
		if(strstr($init_msg, 'already')){
			if($prolong_msg = $this->prolong($user, $group, $end)){
				$this->logger->error("can't prolong to [%s] drweb period for [%s]: %s", date("Y-m-d H:i:s", $end), $user, $prolong_msg);
				return false;
			}else{
				$this->logger->debug("OK prolong to [%s] drweb period for [%s]", date("Y-m-d H:i:s", $end), $user);
				return true;
			}
		}else{
			$this->logger->error("can't init to [%s] drweb period for [%s]: %s", date("Y-m-d H:i:s", $end), $user, $init_msg);
			return false;
		}
	}else{
		$this->logger->debug("OK init to [%s] drweb period for [%s]", date("Y-m-d H:i:s", $end), $user);
		return true;
	}
}

function query($query){
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $query);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array("Authorization: Basic ".base64_encode(sprintf("%s:%s", $this->options['DRWEB_USER'], $this->options['DRWEB_PASSWORD']))));
	ob_start();
	curl_exec($ch);
	$result = trim(ob_get_contents());
	ob_end_clean();
	$this->result = $result;
	return $result;
}

function init($login, $group, $end){
	$query = sprintf('%s/add-customer.ds?id=%s&group=%s&expires=%s&graceperiod=no',
		$this->options['DRWEB_API_URL'], urlencode($login), urlencode($group), date("YmdHis", $end));
	$this->logger->debug('init: %s', $query);
	$result = $this->query($query);
	$result = str_replace("'", "\"", $result);
	if(!strstr($result, 'rc="true"')){
		$msg_begin = strpos($result, "<message>");
		$msg_end = strpos($result, "</message>");
		if($msg_begin > 0){
			return substr($result, $msg_begin + 9, $msg_end - $msg_begin - 9);
		}else{
			return 'error';
		}
	}
}

function prolong($login, $group, $end){
	$query = sprintf('%s/change-customer-info.ds?id=%s&expires=%s&group=%s',
		$this->options['DRWEB_API_URL'], urlencode($login), date("YmdHis", $end), urlencode($group));
	$this->logger->debug('prolong: %s', $query);
	$result = $this->query($query);
	$result = str_replace("'", "\"", $result);
	if(!strstr($result, 'rc="true"')){
		$msg_begin = strpos($result, "<message>");
		$msg_end = strpos($result, "</message>");
		if($msg_begin > 0){
			return substr($result, $msg_begin + 9, $msg_end - $msg_begin - 9);
		}else{
			return 'error';
		}
	}
}

function getUrl($login){
	$query = sprintf('%s/get-customer-info.ds?id=%s&statistics=none',
		$this->options['DRWEB_API_URL'], urlencode($login));
	$result = $this->query($query);
	$result = str_replace("'", "\"", $result);
	if(strstr($result, 'rc="true"')){
		$url_begin = strpos($result, "<url>");
		$url_end = strpos($result, "</url>");
		if($url_begin > 0){
			return $this->remove_cdata(substr($result, $url_begin + 5, $url_end - $url_begin - 5));
		}
	}
}

function remove_cdata($str){
    $begin = strpos($str, "<![CDATA[");
    if($begin!==false){
        $begin += 9;
        $end = strpos($str, "]]>", $begin);
        return substr($str, $begin, $end-$begin);
    }else{
        return $str;
    }
}

}
?>