<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class ConnectionTypesManager {

var $logger;
var $calculator;
var $connectionTypesDAO;
var $connectionTypes;
var $rules;
var $resources;
var $shapers;
var $tv;
var $variableShapers;
var $usePlans;
var $creditTypes;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('connection.types');
}

function setCalculator($obj){
	$this->calculator = $obj;
}

function setConnectionTypesDAO($obj){
	$this->connectionTypesDAO = $obj;
}

function initBean(){
	$this->loadRules();
	$this->loadResources();
	$this->loadShapers();
	$this->loadConnectionTypes();
	$this->loadTv();
	$this->loadBonuses();
}

function loadRules(){
	$this->rules = array();
	$it = $this->connectionTypesDAO->rulesIterator();
	$counter = 0;
	while($arr = $it->next()){
		$this->rules[$arr['connection_type_resource']][$arr['rule']] = $arr;
		$counter++;
	}
	$this->logger->debug("%s connection types rules loaded", $counter);
}

function loadResources(){
	$this->resources = array();
	$this->sumUsePlans = array();
	$it = $this->connectionTypesDAO->resourcesIterator();
	$counter = 0;
	while($arr = $it->next()){
		$this->resources[$arr['connection_type']][$arr['resource_type']] = $arr;
		if(isset($this->plans[$arr['connection_type']])){
			$this->usePlans[$arr['connection_type']] = $this->calculator->add($this->sum[$arr['connection_type']], $arr['use_plan']);
		}else{
			$this->usePlans[$arr['connection_type']] = $arr['use_plan'];
		}
		$counter++;
	}
	$this->logger->debug("%s connection types resources loaded", $counter);
}

function loadShapers(){
	$this->variableShapers = array();
	$this->shapers = array();
	$it = $this->connectionTypesDAO->shapersIterator();
	$counter = 0;
	while($arr = $it->next()){
		$this->shapers[$arr['connection_type']][$arr['shaper']] = $arr;
		if($arr['use_case'] > 0){
			$this->variableShapers[$arr['connection_type']] = 1;
		}
		$counter++;
	}
	$this->logger->debug("%s connection types shapers loaded", $counter);
}

function loadTv(){
	$this->tv = array();
	$it = $this->connectionTypesDAO->tvIterator();
	$counter = 0;
	while($arr = $it->next()){
		$this->tv[$arr['connection_type']][$arr['tv_type']] = $arr;
		$counter++;
	}
	$this->logger->debug("%s connection types tv loaded", $counter);
}

function loadBonuses(){
	$this->bonuses = array();
	$it = $this->connectionTypesDAO->bonusesIterator();
	$counter = 0;
	while($arr = $it->next()){
		$this->bonuses[$arr['connection_type']][$arr['account_type']] = $arr;
		$counter++;
	}
	$this->logger->debug("%s connection types bonuses loaded", $counter);
}

function hasVariableShapers($ctid){
	return isset($this->variableShapers[$ctid]);
}

function loadConnectionTypes(){
	$this->creditTypes = array();
	$this->connectionTypes = array();
	$it = $this->connectionTypesDAO->iterator();
	while($arr = $it->next()){
		$arr['credit'] = 0;
		$this->connectionTypes[$arr['id']] = $arr;
		if($arr['credit_con_type']){
			$this->creditTypes[$arr['credit_con_type']] = 1;
		}
	}
	foreach($this->creditTypes as $id=>$x){
		$this->connectionTypes[$id]['credit'] = 1;
	}
	$this->logger->debug("%s connection types loaded", count($this->connectionTypes));
}

function getConnectionType($id){
	return $this->connectionTypes[$id];
}

function getConnectionTypes(){
	return $this->connectionTypes;
}

function getResource($ctid, $rtid){
	if(isset($this->resources[$ctid][$rtid])){
		return $this->resources[$ctid][$rtid];
	}
}

function walkResources($ctid, $callback){
	if(isset($this->resources[$ctid])){
		foreach($this->resources[$ctid] as $ctResource){
			call_user_func($callback, $ctResource);
		}
	}
}

function walkRules($rtid, $callback){
	if(isset($this->rules[$rtid])){
		foreach($this->rules[$rtid] as $ctRule){
			call_user_func($callback, $ctRule);
		}
	}
}

function walkShapers($ctid, $callback){
	if(isset($this->shapers[$ctid])){
		foreach($this->shapers[$ctid] as $ctShaper){
			call_user_func($callback, $ctShaper);
		}
	}
}

function walkTv($ctid, $callback){
	if(isset($this->tv[$ctid])){
		foreach($this->tv[$ctid] as $ctTV){
			call_user_func($callback, $ctTV);
		}
	}
}

function walkBonuses($ctid, $callback){
	if(isset($this->bonuses[$ctid])){
		foreach($this->bonuses[$ctid] as $ctBonus){
			call_user_func($callback, $ctBonus);
		}
	}
}

function getTv($ctid, $tv_type){
	if(isset($this->tv[$ctid][$tv_type])){
		return $this->tv[$ctid][$tv_type];
	}
}

function getUsePlanSum($ctid){
	if(isset($this->usePlans[$ctid])){
		return $this->usePlans[$ctid];
	}else{
		return '0';
	}
}

function show(){
	print_r($this->connectionTypes);
	print_r($this->bonuses);
}

}
?>