<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class IpUsesManager {

var $logger;
var $calculator;
var $ipUsesDAO;
var $updates;
var $calendarPeriodsManager;
var $calendarPeriod;
var $registry;
var $bytes;
var $loaded;
var $profiler;

function setProfiler($obj){
	$this->profiler = $obj;
}

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('ip.uses');
}

function setCalculator($obj){
	$this->calculator = $obj;
}

function setIpUsesDAO($obj){
	$this->ipUsesDAO = $obj;
}

function setCalendarPeriodsManager($obj){
	$this->calendarPeriodsManager = $obj;
}

function load(){
	if(!$this->loaded){
		$this->calendarPeriod = $this->calendarPeriodsManager->getCurrent();
		$this->registry = array();
		$this->bytes = array();
		$it = $this->ipUsesDAO->iterator($this->calendarPeriod['id']);
		while($a = $it->next()){
			if(isset($this->registry[$a['personal_period_resource_use']][$a['host']][$a['rule']])){
				$entry = &$this->registry[$a['personal_period_resource_use']][$a['host']][$a['rule']];
				$entry['last'] = max($entry['last'], $a['id']);
				$entry['items'][] = $a['id'];
			}else{
				$this->registry[$a['personal_period_resource_use']][$a['host']][$a['rule']] = array(
					'last'=>$a['id'],
					'items'=>array($a['id']),
				);
			}
			$this->bytes[$a['id']] = $a['bytes'];
		}
		$this->logger->debug("%s ip uses loaded in calendar period [%s]", count($this->bytes), $this->calendarPeriod['id']);
		$this->loaded = true;
	}
}

function begin(){
	$this->logger->debug("begin delayed updates pak");
	$this->updates = array();
}

function apply(){
	$this->logger->debug("applying delayed update pak");
	$this->profiler->enter('iu.apply');
	ksort($this->updates);
	foreach($this->updates as $id=>$bytes){
		$this->ipUsesDAO->update($id, $bytes);
	}
	$this->profiler->leave('iu.apply');
}

function addBytes($ppru_id, $host, $rule, $bytes){
	if(isset($this->registry[$ppru_id][$host][$rule])){
		$last = $this->registry[$ppru_id][$host][$rule]['last'];
		$new_bytes = $this->calculator->add($this->bytes[$last], $bytes);
		$this->bytes[$last] = $new_bytes;
		$this->updates[$last] = $new_bytes;
		$this->logger->debug("add %s bytes to existen ip use [%s]", $bytes, $last);
		return $last;
	}else{
		$iu_id = $this->ipUsesDAO->insert($ppru_id, $host, $rule, $bytes);
		$this->registry[$ppru_id][$host][$rule] = array(
			'last'=>$iu_id,
			'items'=>array($iu_id),
		);
		$this->bytes[$iu_id] = $bytes;
		$this->logger->debug("create new ip use [%s] with %s bytes", $iu_id, $bytes);
		return $iu_id;
	}
}

function show(){
	print_r($this->registry);
}

}
?>