<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class Launcher {

var $logger;
var $locker;
var $beanFactory;
var $cli;
var $actionOption;
var $workdir;

function setWorkdir($str){
	$this->workdir = $str;
}

function setLoggerFactory($obj){
    $this->logger = $obj->getLogger('launcher');
}

function setBeanFactory($obj){
	$this->beanFactory = $obj;
}

function setCommandLineInterface($obj){
	$this->cli = $obj;
}

function setActionOption($obj){
	$this->actionOption = $obj;
}

function setLocker($obj){
    $this->locker = $obj;
}

function abortIfLocked($requested_by='', $timeout = 0, $name='caban'){
    if(!$this->locker->capture($name, $timeout)){
	    $this->logger->warning("locked, skip action: %s", $requested_by);
	    //die("system locked, aborting\n");
		exit(0);
    }
}

function launch(){
	$this->cli->parse();
	$action = $this->actionOption->getValue();
	switch($action){
		case 'fetch':
			$this->abortIfLocked('fetch');
			$fetcher = $this->beanFactory->getBean('fetcherMaster');
			$fetcher->fetch();
			return;
		case 'fetch_slave':
			$fetcher = $this->beanFactory->getBean('fetcher');
			$fetcher->fetch();
			return;
		case 'read_ipt_names':
			$rm = $this->beanFactory->getBean('rulesManager'); //   
			$iptNames = $this->beanFactory->getBean('iptNames');
			$iptNames->readServerNames();
			return;
		case 'read_counters':
			$rm = $this->beanFactory->getBean('rulesManager');
			$rm->readServerCounters();
			return;
		case 'fix':
			$this->abortIfLocked('fix');
			$fetcher = $this->beanFactory->getBean('fetcher');
			return $fetcher->fix();
		case 'make':
			$rulesBuilder = $this->beanFactory->getBean('rulesBuilder');
			$rulesBuilder->make();
			return;
		case 'make_slave':
			$rulesBuilder = $this->beanFactory->getBean('rulesBuilder');
			$rulesBuilder->makeSlave();
			return;
		case 'make_server':
			$ipSetBuildHelper = $this->beanFactory->getBean('ipSetBuildHelper');
			return $ipSetBuildHelper->processServer();
		case 'pak':
			$this->abortIfLocked('pak');
			$packer = $this->beanFactory->getBean('ipUseLogPacker');
			return $packer->pak();
		case 'clean':
			$this->abortIfLocked('clean', 900);
			$obj = $this->beanFactory->getBean('ipUseLogCleaner');
			return $obj->process();
		case 'offline':
			$this->abortIfLocked('offline');
			$offline = $this->beanFactory->getBean('offlineManager');
			$offline->load();
			return $offline->process();
		case 'mail':
			$proc = $this->beanFactory->getBean('mailProcessor');
			return $proc->process();
		case 'track':
			if(!$this->locker->capture('track', 0)){
			    $this->logger->info("track locked");
				return;
		    }else{
				$accessTrackerService = $this->beanFactory->getBean('accessTrackerService');
				$accessTrackerService->load();
				return $accessTrackerService->track();
			}
		case 'move_tasks':
			$obj = $this->beanFactory->getBean('taskMover');
			return $obj->process();
		case 'jobs':
			$obj = $this->beanFactory->getBean('jobsService');
			return $obj->process();
		case 'jobs_extras':
			$obj = $this->beanFactory->getBean('jobsExtrasService');
			return $obj->process();
		case 'shape':
			$shaper = $this->beanFactory->getBean('shapersManager');
			$shaper->load();
			return $shaper->make();
		case 'help':
			return $this->cli->showHelp();
		case 'snapshot':
			$bean = $this->beanFactory->getBean('snapshotMaker');
			return $bean->process();
		case 'update':
			$ppruDAO = $this->beanFactory->getBean('personalPeriodResourceUsesDAO');
			$ppruDAO->correct();
			$pppDAO = $this->beanFactory->getBean('personalPeriodPlansDAO');
			$pppDAO->updateActualUses();
			$proc = $this->beanFactory->getBean('stateUpdateService');
			$proc->update();
			return;
		case 'future':
			$proc = $this->beanFactory->getBean('calendarPeriodsService');
			return $proc->future();
		case 'spread_log':
			$opt = getopt('w:a:t:f:');
			$time = isset($opt['t']) ? $opt['t'] : time();
			$filename = isset($opt['f']) ? $opt['f'] : $this->workdir.'/iul.dump';
			$ipUseLogDAO = $this->beanFactory->getBean('ipUseLogDAO');
			return $ipUseLogDAO->spreadLog($filename, $time);
		case 'log_hour':
			$opt = getopt('w:a:t:');
			$time = isset($opt['t']) ? $opt['t'] : time();
			$ipUseLogDAO = $this->beanFactory->getBean('ipUseLogDAO');
			return $ipUseLogDAO->logHour($time);
		case 'log_day':
			$opt = getopt('w:a:t:');
			$time = isset($opt['t']) ? $opt['t'] : time();
			$ipUseLogDAO = $this->beanFactory->getBean('ipUseLogDAO');
			return $ipUseLogDAO->logDay($time);
		case 'call_report':
			$callReport = $this->beanFactory->getBean('callReport');
			$callReport->process();
			return;
		case 'correct_ppru':
			// -   -a update  
			return;
		case 'tv_audit':
			$this->abortIfLocked('tv_audit', 0, 'tv_audit');
			$tvNodeAuditor = $this->beanFactory->getBean('tvNodeAuditor');
			$tvNodeAuditor->process();
			return;
		case 'test':
			/*
			$cm = $this->beanFactory->getBean('clientsManager');
			$cm->replaceExpiredConnectionTypes();
			$pps = $this->beanFactory->getBean('personalPeriodsService');
			$pps->checkClient($cm->getClient(1653));
			$sm = $this->beanFactory->getBean('shapersManager');
			$sm->loadUnshapedBySubscription();
			
			$sm = $this->beanFactory->getBean('snapshotMaker');
			$sm->fixArpu();
			
			$spawner = $this->beanFactory->getBean('spawner');
			$pids = array();
			$pids[] = $spawner->spawn('sleep 1');
			$pids[] = $spawner->spawn('sleep 2');
			$pids[] = $spawner->spawn('sleep 3');
			$this->logger->info('ama do ma things');
			$spawner->waitFor($pids);
			$this->logger->info('after childs');
			
			$rb = $this->beanFactory->getBean('rulesBuilder');
			$rb->make();
			*/
			//$rp =  $this->beanFactory->getBean('ecallDelegate');
			//$filename = "22.08.2011.xls";
			//$rp->processReport(file_get_contents('/tmp/'.$filename.''), $filename, 1);
			$bean = $this->beanFactory->getBean('task.dhcp.remake');
			$bean->execute(array());
			return;
		default:
			printf("invalid action: %s\n\n", $action);
			return $this->cli->showHelp();
	}
}

}
?>