<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class OfflineManager {

var $connection;
var $personalPeriodsManager;
var $clientsManager;
var $timeService;
var $threshold;
var $logger;
var $blocksManager;
var $accountsManager;
var $hostsManager;
var $operator;
var $offlineDepartment;
var $offlineAccount;
var $calculator;
var $modemsDAO;
var $accessTrackerDAO;
var $modems = array();
var $connectionTypesManager;
var $tvTypesManager;
var $loaded;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('offline.manager');
}

function setConnection($connection){
	$this->connection = $connection;
}

function setConnectionTypesManager($obj){
	$this->connectionTypesManager = $obj;
}

function setPersonalPeriodsManager($obj){
	$this->personalPeriodsManager = $obj;
}

function setClientsManager($obj){
	$this->clientsManager = $obj;
}

function setBlocksManager($obj){
	$this->blocksManager = $obj;
}

function setAccountsManager($obj){
	$this->accountsManager = $obj;
}

function setHostsManager($obj){
	$this->hostsManager = $obj;
}

function setTimeService($obj){
	$this->timeService = $obj;
}

function setThreshold($i){
	$this->threshold = $i;
}

function setOperator($i){
	$this->operator = $i;
}

function setOfflineAccount($i){
	$this->offlineAccount = $i;
}

function setOfflineDepartment($i){
	$this->offlineDepartment = $i;
}

function setCalculator($obj){
	$this->calculator = $obj;
}

function setModemsDAO($obj){
	$this->modemsDAO = $obj;
}

function setAccessTrackerDAO($obj){
	$this->accessTrackerDAO = $obj;
}

function setOptionsManager($obj){
	$this->optionsManager = $obj;
}

function setTvTypesManager($obj){
	$this->tvTypesManager = $obj;
}

function load(){
	if(!$this->loaded){
		$this->blocksManager->load();
		$this->accountsManager->load();
		$this->personalPeriodsManager->load();
		$this->clientsManager->load();
		$this->accountsManager->load();
		$this->hostsManager->load();
		$this->count = 0;
		$this->time = $this->timeService->getTime() - $this->threshold;
		$this->loadPasswd();
		$this->forced = array();
		$sel = $this->connection->select("SELECT client FROM force_offline");
		while($arr = $sel->next()){
			$this->forced[$arr['client']] = 1;
		}
		$this->loaded = true;
	}
}

function process(){
	$this->clientsManager->walk(array($this, 'checkClient'));
	if($this->count > 0){
		$this->logger->info('%s clients moved to offline', $this->count);
	}
}

function getModem($id){
	if(!isset($this->modems[$id])){
		$this->modems[$id] = $this->modemsDAO->getById($id);
	}
	return $this->modems[$id];
}

function loadPasswd(){
	$opt = $this->optionsManager->getOption('HOMEDIR_CMD_FORMAT');
	if($opt['actual']){
		$this->homedirCmdFormat = $opt['value'];
		$cmd = sprintf($this->homedirCmdFormat, 'cat /etc/passwd');
		$this->passwd = array(
			'by_login'=>array(),
			'by_uid'=>array(),
		);
		if($fd = popen($cmd, "r")){
			while (!feof($fd)) {
				$line = fgets($fd, 4096);
				if(trim($line)){
					$item = explode(":",$line);
					$this->passwd['by_login'][$item[0]] = $item;
					$this->passwd['by_uid'][$item[2]] = $item;
				}
			}
			pclose($fd);
		}
		//print_r($this->passwd);
	}
}

function checkClient($client){
	$period = $this->personalPeriodsManager->getClientPersonalPeriod($client['id']);
	$conType = $this->connectionTypesManager->getConnectionType($client['connection_type']);
	$offline = isset($this->forced[$client['id']]);
	if(!$offline && $conType['offline_type'] &&
		!$this->blocksManager->isClientDisconnected($client['id']) &&
		($client['department']!=$this->offlineDepartment) && 
		($conType['service_type'] > 0)
	){
		if($period){
			$offline = $period['period_end'] <= $this->time;
		}else{
			$sel = $this->connection->select("SELECT UNIX_TIMESTAMP(created) created FROM stamps WHERE id='%s'", $client['stamp']);
			if($stamp = $sel->next()){
				$offline = $stamp['created'] <= $this->time;
			}
		}
	}
	if($offline){
		$this->connection->clearErrors();
		$this->connection->query("BEGIN");
		$this->logger->info('moving %s to offline', $client['login']);
		$this->count++;
		$notes = array();
		$notes[] = '  :';
		$notes[] = sprintf(" : %s", $conType['name']);
		$tvType = $this->tvTypesManager->getTvType($client['tv_type']);
		$notes[] = sprintf(" : %s", $tvType ? $tvType['name'] : '');
		//  
		if($conType['offline_type']){
			$this->connection->update("UPDATE clients SET connection_type=%s WHERE id=%s", $conType['offline_type'], $client['id']);
		}
		$this->connection->update("UPDATE clients SET department=%s, state=3, tv_type=NULL WHERE id=%s", $this->offlineDepartment, $client['id']);
		//  
		$amount = $this->accountsManager->getVirtualState($client['account']);
		if($this->calculator->cmp($amount, '0') > 0){
			$notes[] = sprintf('   %s .', $amount/100);
			$this->accountsManager->actualMove($client['account'], $this->offlineAccount, $amount);
		}elseif($this->calculator->cmp($amount, '0') < 0){
			$notes[] = sprintf('   %s .', $amount/100);
			$a = $this->calculator->sub('0', $amount);
			$this->accountsManager->actualMove($this->offlineAccount, $client['account'], $a);
		}
		//  
		$modems = array();
		$hosts = $this->hostsManager->getClientHosts($client['id']);
		foreach($hosts as $host){
			$this->connection->update("DELETE FROM hosts WHERE id=%s", $host['id']);
			$modem = $this->getModem($host['modem']);
			$notes[] = sprintf(" : ip=%s, mac=%s, modem=%s, port=%s", $host['ip'], $host['mac'], $modem['name'], $host['port']);
			$modems[] = $host['modem'];
			if($host['port']){
				$this->connection->update("REPLACE INTO offline_ports (client, modem, port, date) VALUES (%s, %s, %s, now())",
					$client['id'], $host['modem'], $host['port']);
			}
		}
		//   
		if(isset($this->homedirCmdFormat)){
			$sel = $this->connection->select("SELECT * FROM homedirs WHERE client=%s", $client['id']);
			while($arr = $sel->next()){
				$notes[] = sprintf("  : %s", $arr['login']);
				$this->connection->update("DELETE FROM homedirs WHERE id=%s", $arr['id']);
				if($arr['active']){
					$login = $arr['login'];
					if(isset($this->passwd['by_login'][$login])){
						$script = sprintf($this->homedirCmdFormat, sprintf("userdel %s", $login));
						exec($script, $out, $err);
						if($err){
							$this->logger->error('cant userdel %s: %s', $login, implode("\n", $out));
						}
					}
				}
			}
		}
		//    -
		$this->connection->update("UPDATE tv_connections SET state=3 WHERE client=%s AND state<3", $client['id']);

		//   
		$stamp = $this->connection->insert_auto("INSERT INTO stamps (created, created_by) VALUES (now(), %s)", $this->operator);
		$this->connection->update("INSERT INTO client_notes (client, note, stamp) VALUES (%s, '%s', %s)", $client['id'], implode("\n", $notes), $stamp);
		//     
		foreach($modems as $modem ){
			$this->accessTrackerDAO->appendToQueue($modem, time());
		}
		//  
		if($period){
			$this->connection->update("UPDATE personal_period_plans SET use_plan=use_actual WHERE personal_period='%s' AND use_plan > use_actual", $period['id']);
		}
		//  
		$this->connection->update("DELETE FROM charge_subscriptions WHERE client='%s'", $client['id']);
		//    0
		$this->connection->update("UPDATE clients SET block=0 WHERE id='%s'", $client['id']);
		//$this->connection->update("DELETE FROM offline_ports WHERE CONCAT(modem,':',port) IN (SELECT CONCAT(modem,':',port) FROM hosts)");
		$this->connection->update("UPDATE offline_ports o LEFT JOIN hosts h ON h.modem=o.modem AND h.port=o.port SET o.host=h.id WHERE h.id IS NOT NULL");
		$this->connection->update("DELETE FROM offline_ports WHERE host IS NOT NULL");
		
		//   force_offline
		$this->connection->update("DELETE FROM force_offline WHERE client='%s'", $client['id']);
		
		if($this->connection->hasErrors()){
			$this->connection->query("ROLLBACK");
		}else{
			$this->connection->query("COMMIT");
		}
	}
}

}
?>