<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class PersonalPeriodPlansManager {

var $logger;
var $personalPeriodPlansDAO;
var $calculator;
var $plans;
var $planUpdate;
var $loaded;
var $profiler;

function setProfiler($obj){
	$this->profiler = $obj;
}

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('personal.period.plans');
}

function setPersonalPeriodPlansDAO($obj){
	$this->personalPeriodPlansDAO = $obj;
}

function setCalculator($obj){
	$this->calculator = $obj;
}

function load(){
	if(!$this->loaded){
		$this->plans = array();
		$it = $this->personalPeriodPlansDAO->iterator();
		$counter = 0;
		while($arr = $it->next()){
			$arr['use_actual_init'] = $arr['use_actual'];
			$arr['use_value_init'] = $arr['use_value'];
			$this->plans[$arr['personal_period']][$arr['resource_type']] = $arr;
			$counter++;
		}
		$this->logger->debug("%s personal period plans loaded", $counter);
		$this->loaded = true;
	}
}

function getPlan($pid, $rtid){
	if(isset($this->plans[$pid][$rtid])){
		return $this->plans[$pid][$rtid];
	}
}

function addPlan($pid, $rtid, $plan, $limit){
	$id = $this->personalPeriodPlansDAO->insert($pid, $rtid, $plan, $limit);
	$this->plans[$pid][$rtid] = array(
		'id'=>$id,
		'personal_period'=>$pid,
		'resource_type'=>$rtid,
		'use_plan'=>$plan,
		'use_limit'=>$limit,
		'use_actual'=>'0',
		'use_actual_init'=>'0',
		'use_value'=>'0',
		'use_value_init'=>'0',
	);
}

function walkPlans($pid, $callback){
	if(isset($this->plans[$pid])){
		foreach($this->plans[$pid] as $plan){
			call_user_func($callback, $plan);
		}
	}
}

function isPlanComplete($pid, $rtid){
	if(isset($this->plans[$pid][$rtid])){
		$plan = $this->plans[$pid][$rtid];
		return $this->calculator->cmp($plan['use_plan'], $plan['use_actual']) <= 0;
	}else{
		return true;
	}
}

function isAllPlansComplete($pid){
	if(isset($this->plans[$pid])){
		foreach($this->plans[$pid] as $rtid=>$plan){
			$plan = $this->plans[$pid][$rtid];
			if($this->calculator->cmp($plan['use_plan'], $plan['use_actual']) > 0){
				return false;
			}
		}
		return true;
	}else{
		return true;
	}
}

function isJustComplete($pid){
	if(isset($this->plans[$pid])){
		$just_complete = false;
		foreach($this->plans[$pid] as $rtid=>$plan){
			$plan = $this->plans[$pid][$rtid];
			if($this->calculator->cmp($plan['use_plan'], $plan['use_actual']) > 0){
				$just_complete = $just_complete || $this->calculator->cmp($plan['use_plan'], $plan['use_actual_init']) <= 0;
			}
		}
		return $just_complete;
	}else{
		return false;
	}
}

function getPlanLack($pid, $rtid){
	$plan = $this->plans[$pid][$rtid];
	return $this->calculator->sub($plan['use_plan'], $plan['use_actual']);
}

function addUseValue($pid, $rtid, $value){
	if(!isset($this->plans[$pid][$rtid])){
		$this->addPlan($pid, $rtid, '0', '0');
	}
	$plan = &$this->plans[$pid][$rtid];
	$plan['use_value'] = $this->calculator->add($plan['use_value'], $value);
}

function addPlanUse($pid, $rtid, $amount){
	if(!isset($this->plans[$pid][$rtid])){
		$this->addPlan($pid, $rtid, '0', '0');
	}
	$plan = &$this->plans[$pid][$rtid];
	$plan['use_actual'] = $this->calculator->add($plan['use_actual'], $amount);
	$this->planUpdate[$pid][$rtid] = $plan['use_actual'];
}

function begin(){
	$this->logger->debug("begin delayed updates pak");
	$this->planUpdate = array();
}

function apply(){
	$this->logger->debug("applying delayed update pak");
	$this->profiler->enter('ppp.apply');
	ksort($this->planUpdate);
	foreach($this->planUpdate as $pid=>$parr){
		ksort($parr);
		foreach($parr as $rtid=>$amount){
			$this->personalPeriodPlansDAO->update($pid, $rtid, $amount);
		}
	}
	$this->profiler->leave('ppp.apply');
}

function updateActualUses(){
	$this->personalPeriodPlansDAO->updateActualUses();
}

}
?>