<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class PersonalPeriodResourceUsesManager {

var $logger;
var $calculator;
var $personalPeriodResourceUsesDAO;
var $updates;
var $calendarPeriodsManager;
var $calendarPeriod;
var $registry;
var $amounts;
var $loaded;
var $profiler;

function setProfiler($obj){
	$this->profiler = $obj;
}

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('personal.period.resource.uses');
}

function setCalculator($obj){
	$this->calculator = $obj;
}

function setPersonalPeriodResourceUsesDAO($obj){
	$this->personalPeriodResourceUsesDAO = $obj;
}

function setCalendarPeriodsManager($obj){
	$this->calendarPeriodsManager = $obj;
}

function load(){
	if(!$this->loaded){
		$this->calendarPeriod = $this->calendarPeriodsManager->getCurrent();
		$this->registry = array();
		$this->amounts = array();
		$it = $this->personalPeriodResourceUsesDAO->iterator($this->calendarPeriod['id']);
		while($a = $it->next()){
			if(isset($this->registry[$a['personal_period']][$a['resource_use']])){
				$entry = &$this->registry[$a['personal_period']][$a['resource_use']];
				$entry['last'] = max($entry['last'], $a['id']);
				$entry['items'][] = $a['id'];
			}else{
				$this->registry[$a['personal_period']][$a['resource_use']] = array(
					'last'=>$a['id'],
					'items'=>array($a['id']),
				);
			}
			$this->amounts[$a['id']] = $a['amount'];
		}
		$this->logger->debug("%s personal period resource uses loaded in calendar period [%s]", count($this->amounts), $this->calendarPeriod['id']);
		$this->loaded = true;
	}
}

function begin(){
	$this->logger->debug("begin delayed updates pak");
	$this->updates = array();
}

function apply(){
	$this->logger->debug("applying delayed update pak");
	$this->profiler->enter('ppru.apply');
	ksort($this->updates);
	foreach($this->updates as $id=>$amount){
		$this->personalPeriodResourceUsesDAO->update($id, $amount);
	}
	$this->profiler->leave('ppru.apply');
}

function getAmount($pid, $ruid){
	if(isset($this->registry[$pid][$ruid])){
		$entry = &$this->registry[$pid][$ruid];
		return $this->amounts[$entry['last']];
	}else{
		return '0';
	}
}

function addAmount($pid, $ruid, $amount){
	if(isset($this->registry[$pid][$ruid])){
		$last = $this->registry[$pid][$ruid]['last'];
		$new_amount = $this->calculator->add($this->amounts[$last], $amount);
		$this->amounts[$last] = $new_amount;
		$this->updates[$last] = $new_amount;
		$this->logger->debug("add %s amount to existen resource use [%s]", $amount, $last);
		return $last;
	}else{
		if($ppru_id = $this->personalPeriodResourceUsesDAO->insert($pid, $ruid, $amount)){
			$this->registry[$pid][$ruid] = array(
				'last'=>$ppru_id,
				'items'=>array($ppru_id),
			);
			$this->amounts[$ppru_id] = $amount;
			$this->logger->debug("created new resource use [%s] with %s amount", $ppru_id, $amount);
			return $ppru_id;
		}
	}
}

}
?>