<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class PersonalPeriodsNotifier {

var $notifiedPersonalPeriodsDAO;

var $personalPeriodsManager;
var $personalPeriodPlansManager;
var $connectionTypesManager;
var $resourceTypesManager;
var $chargeTypesManager;
var $subscriptionsManager;

var $logger;
var $currencyFormat;
var $calculator;
var $timeService;

var $notified;
var $loaded;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('personal.periods.notifier');
}

function setNotifiedPersonalPeriodsDAO($obj){
	$this->notifiedPersonalPeriodsDAO = $obj;
}

function setCurrencyFormat($obj){
	$this->currencyFormat = $obj;
}

function setCalculator($obj){
	$this->calculator = $obj;
}

function setTimeService($obj){
	$this->timeService = $obj;
}

function setPersonalPeriodsManager($obj){
	$this->personalPeriodsManager = $obj;
}

function setPersonalPeriodPlansManager($obj){
	$this->personalPeriodPlansManager = $obj;
}

function setConnectionTypesManager($obj){
	$this->connectionTypesManager = $obj;
}

function setResourceTypesManager($obj){
	$this->resourceTypesManager = $obj;
}

function setChargeTypesManager($obj){
	$this->chargeTypesManager = $obj;
}

function setSubscriptionsManager($obj){
	$this->subscriptionsManager = $obj;
}

function load(){
	if(!$this->loaded){
		$this->personalPeriodsManager->load();
		$this->personalPeriodPlansManager->load();
		$this->subscriptionsManager->load();
		$this->notified = array();
		$it = $this->notifiedPersonalPeriodsDAO->iterator();
		while($arr = $it->next()){
			$this->notified[$arr['client']] = $arr['personal_period'];
		}
		$this->logger->debug("%s personal period notification marks loaded", count($this->notified));
		$this->loaded = true;
	}
}

function notifyPlanStarted($plan){
	$res_type = $this->resourceTypesManager->getResourceType($plan['resource_type']);
	if($this->calculator->cmp($plan['use_plan'], '0') > 0){
		$this->notifyPlans[$res_type['name']] = $this->currencyFormat->format($plan['use_plan']);
	}
}

function notifyPlanLack($plan){
	if($this->calculator->cmp($plan['use_plan'], $plan['use_actual']) > 0){
		$lack = $this->calculator->sub($plan['use_plan'], $plan['use_actual']);
		$this->planLackSum = $this->calculator->add($this->planLackSum, $lack);
	}
}

function notifyResource($ctResource){
	$res_type = $this->resourceTypesManager->getResourceType($ctResource['resource_type']);
	if($plan = $this->personalPeriodPlansManager->getPlan($this->period['id'], $ctResource['resource_type'])){
		//     
		if($this->calculator->cmp($plan['use_plan'], $plan['use_actual']) > 0){
			$lack = $this->calculator->sub($plan['use_plan'], $plan['use_actual']);
			$this->notifySum = $this->calculator->add($this->notifySum, $lack);
			$this->planLack[$res_type['name']] = $this->currencyFormat->format($lack);
		}
	}
	if($this->calculator->cmp($ctResource['use_plan'], '0') > 0){
		$this->notifyPlans[$res_type['name']] = $this->currencyFormat->format($ctResource['use_plan']);
		$this->notifySum = $this->calculator->add($this->notifySum, $ctResource['use_plan']);
	}
}

function notifySubscription($sub){
	$type = $this->chargeTypesManager->getChargeType($sub['type']);
	$cost = $this->calculator->divLow($this->calculator->mul($sub['amount'], $type['price']), $type['unit_size']);
	$this->notifySubscriptions[] = array(
		'name'=>$type['name'],
		'price'=>$this->currencyFormat->format($type['price']),
		'unit_size'=>$type['unit_size'],
		'amount'=>$sub['amount'],
		'cost'=>$this->currencyFormat->format($cost),
	);
	$this->notifySum = $this->calculator->add($this->notifySum, $cost);
}

function notifyClient($client, &$context){
	$send = false;
	$this->planLackSum = '0';
	if(($period = $this->personalPeriodsManager->getClientPersonalPeriod($client['id'])) && $period['state']==1){
		$this->period = $period;
		$this->personalPeriodPlansManager->walkPlans($period['id'], array($this, 'notifyPlanLack'));
		if(isset($period['new'])){
			$con_type = $this->connectionTypesManager->getConnectionType($period['connection_type']);
			$this->notifyPlans = array();
			$this->notifySum = '0';
			$this->notifySubscriptions = array();
			$this->personalPeriodPlansManager->walkPlans($period['id'], array($this, 'notifyPlanStarted'));
			$this->subscriptionsManager->walkClientSubscriptions($client['id'], array($this, 'notifySubscription'));
			$abon = $con_type['abon_dst'] ? $con_type['abon_amount'] : 0;
			$context['period'] = array(
				'begin'=>$period['period_begin'],
				'end'=>$period['period_end'],
				'connection_type'=>$con_type['name'],
				'abon'=>$this->currencyFormat->format($abon),
				'subscriptions'=>$this->notifySubscriptions,
				'plans'=>$this->notifyPlans,
			);
			$send = true;
		}else{
			if(($client['notice'] > 0) && $client['email']){
				$time = $this->timeService->getTime();
				if($time + 86400 * $client['notice'] > $period['period_end']){ // day = 60*60*24 sec
					if(!isset($this->notified[$client['id']]) || $this->notified[$client['id']]!=$period['id']){
						$this->notifiedPersonalPeriodsDAO->update($client['id'], $period['id']);
						$this->planLack = array();
						$this->notifyPlans = array();
						$this->notifySum = '0';
						$this->notifySubscriptions = array();
						$con_type = $this->connectionTypesManager->getConnectionType($client['connection_type']);
						$this->connectionTypesManager->walkResources($con_type['id'], array($this, 'notifyResource'));
						$this->subscriptionsManager->walkClientSubscriptions($client['id'], array($this, 'notifySubscription'));
						$abon = $con_type['abon_dst'] ? $con_type['abon_amount'] : '0';
						$this->notifySum = $this->calculator->add($this->notifySum, $abon);
						$context['next_period'] = array(
							'begin'=>$period['period_end'],
							'connection_type'=>$con_type['name'],
							'abon'=>$this->currencyFormat->format($abon),
							'subscriptions'=>$this->notifySubscriptions,
							'plans'=>$this->notifyPlans,
							'lack'=>$this->planLack,
							'required_deposit'=>$this->currencyFormat->format($this->notifySum),
						);
						$send = true;
					}
				}
			}
		}
	}
	$context['plan_lack_sum'] = $this->planLackSum;
	return $send;
}

}
?>