<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class PersonalPeriodsService {

var $logger;

var $accessTrackerService;
var $accountsManager;
var $accountTypesManager;
var $blocksManager;
var $blocksDAO;
var $bonusesManager;
var $clientsManager;
var $chargeTypesManager;
var $connectionTypesManager;
var $depositBlocker;
var $subscriptionsManager;
var $resourceUsesManager;
var $personalPeriodResourceUsesManager;
var $transactionManager;
var $personalPeriodsDAO;
var $personalPeriodsManager;
var $personalPeriodPlansManager;
var $offlineManager;
var $payManager;

var $calculator;
var $timeService;

var $periodsChanged = false;
var $client;
var $period;
var $newPlans;
var $newPlanSum;
var $connection;
var $optionsManager;
var $stampManager;
var $profiler;

function setProfiler($obj){
	$this->profiler = $obj;
}

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('personal.periods.service');
}

function setCalculator($obj){
	$this->calculator = $obj;
}

function setTimeService($obj){
	$this->timeService = $obj;
}

function setPersonalPeriodsDAO($obj){
	$this->personalPeriodsDAO = $obj;
}

function setAccessTrackerService($obj){
	$this->accessTrackerService = $obj;
}

function setAccountsManager($obj){
	$this->accountsManager = $obj;
}

function setAccountTypesManager($obj){
	$this->accountTypesManager = $obj;
}

function setBlocksManager($obj){
	$this->blocksManager = $obj;
}

function setBlocksDAO($obj){
	$this->blocksDAO = $obj;
}

function setBonusesManager($obj){
	$this->bonusesManager = $obj;
}

function setChargeTypesManager($obj){
	$this->chargeTypesManager = $obj;
}

function setClientsManager($obj){
	$this->clientsManager = $obj;
}

function setConnectionTypesManager($obj){
	$this->connectionTypesManager = $obj;
}

function setDepositBlocker($obj){
	$this->depositBlocker = $obj;
}

function setSubscriptionsManager($obj){
	$this->subscriptionsManager = $obj;
}

function setResourceUsesManager($obj){
	$this->resourceUsesManager = $obj;
}

function setPersonalPeriodResourceUsesManager($obj){
	$this->personalPeriodResourceUsesManager = $obj;
}

function setTransactionManager($obj){
	$this->transactionManager = $obj;
}

function setPersonalPeriodsManager($obj){
	$this->personalPeriodsManager = $obj;
}

function setPersonalPeriodPlansManager($obj){
	$this->personalPeriodPlansManager = $obj;
}

function setConnection($connection){
	$this->connection = $connection;
}

function setOptionsManager($obj){
	$this->optionsManager = $obj;
}

function setStampManager($obj){
	$this->stampManager = $obj;
}

function setOfflineManager($obj){
	$this->offlineManager = $obj;
}

function setPayManager($obj){
	$this->payManager = $obj;
}

function hasPeriodsChanged(){
	return $this->periodsChanged;
}

function checkPeriods(){
	$this->clientsManager->load();
	$this->bonusesManager->load();
	$this->resourceUsesManager->load();
	$this->personalPeriodsManager->load();
	$this->personalPeriodResourceUsesManager->load();
	$this->personalPeriodPlansManager->load();
	$this->subscriptionsManager->load();
	$this->offlineManager->load();
	$this->depositBlocker->load();
	$this->accessTrackerService->load();
	$this->blocksManager->load();
	$this->logger->debug("begin check");
	$this->profiler->enter('personal.periods.check');
	$this->periodsChanged = false;
	$this->clientsManager->walk(array($this, 'checkClient'));
	$this->profiler->leave('personal.periods.check');
	$this->logger->debug("end check");
}

function checkClient($client){
	$this->client = $client;
	if(!$this->blocksManager->isClientDisconnected($client['id'])){
		if($period = $this->personalPeriodsManager->getClientPersonalPeriod($client['id'])){
			$this->checkPeriod($period);
		}else{
			if($period = $this->getLostClientPersonalPeriod($client['id'])){
				$this->checkPeriod($period);
			}else{
				$this->firstPersonalPeriod();
			}
		}
	}else{
		$this->logger->debug("%s disconnected - skip period checking", $client['login']);
	}
	$this->offlineManager->checkClient($client);
}

function checkPeriod($period){
	$this->period = $period;
	$time = $this->timeService->getTime();
	if($this->period['period_end'] <= $time){
		if($this->prolong($period) == 0){ //     
			$this->logger->debug("%s's personal period expired", $this->client['login']);
			if($this->period['state']==1){
				$this->closePersonalPeriod($period);
			}
			if($candidate = $this->personalPeriodsDAO->getNextCandidate($this->client['id'], $time)){
				$this->nextCandidatePeriod($candidate);
			}else{
				$this->nextPersonalPeriod();
			}
		}
	}else{
		//       
		if($prev = $this->accessTrackerService->getState($period['client'])){
			if($prev['state']==0){
				$this->prolong($period);
			}
		}
	}
	unset($this->period);
}

//  -        ,
//        
//   -        ,     
function prolong($period){
	$this->prolong = 0;
	$connection_type = $this->connectionTypesManager->getConnectionType($period['connection_type']);
	if(!$connection_type['credit']){ //   
		$to = 0;
		$it = $this->blocksDAO->iteratorFinishedByClient($period['client'], $period['period_begin']);
		while($block = $it->next()){
			if($block['actual_from'] < $block['actual_to']){
				$this->blocksDAO->markProlonged($block['id']);
				if($block['actual_to'] > $to){
					$from = max($block['actual_from'], $to);
					$to = $block['actual_to'];
					$this->prolong += $to - $from;
				}
			}
		}
		if($this->prolong > 0){
			$this->period['period_end'] += $this->prolong;
			$this->subscriptionsManager->walkClientSubscriptions($this->client['id'], array($this, 'prolongSubscription'));
			$this->personalPeriodsDAO->prolong($period['id'], $this->prolong);
			$this->logger->info("personal period [%s] of client [%s] prolonged on [%s] seconds", $period['id'], $this->client['login'], $this->prolong);
		}
	}
	return $this->prolong;
}

function prolongSubscription($sub){
	$type = $this->chargeTypesManager->getChargeType($sub['type']);
	if(!$this->chargeTypesManager->driverStart($sub, $this->client, $this->period)){
		$this->logger->warning('driver cant prolong subscription [%s] for client [%s]', $sub['type'], $this->client['login']);
	}
}

function closePersonalPeriod(&$period){
	$this->closingPeriod = $period;
	$this->logger->debug("closing %s's personal period", $this->client['login']);
	$this->transactionManager->begin();
	$this->connectionTypesManager->walkResources($this->period['connection_type'], array($this, 'closeResource'));
	$this->connectionTypesManager->walkBonuses($this->period['connection_type'], array($this, 'closeBonus'));
	$this->personalPeriodsDAO->changeState($period['id'], '2');
	$period['state'] = '2';
	$this->personalPeriodsManager->setClientPersonalPeriod($period);
	$this->periodsChanged = true;
	$this->transactionManager->end();
}

function closeBonus($ctBonus){
	//  
	if($ctBonus['end_dst']){
		if($bonus = $this->bonusesManager->getBonus($this->client['id'], $ctBonus['account_type'])){
			$rest = $this->accountsManager->getVirtualState($bonus['account']);
			if($this->calculator->cmp($rest, $ctBonus['end_amount']) > 0){
				$type = $this->accountTypesManager->getAccountType($ctBonus['account_type']);
				$sub = $this->calculator->sub($rest, $ctBonus['end_amount']);
				$this->logger->debug("%s's [%s] bonus left, clearing to %s", $this->client['login'], $type['name'], $ctBonus['end_amount']);
				$this->accountsManager->actualMove($bonus['account'], $ctBonus['end_dst'], $sub);
			}
		}
	}
}

function closeResource($ctResource){
	//   
	if($plan = $this->personalPeriodPlansManager->getPlan($this->closingPeriod['id'], $ctResource['resource_type'])){
		//      
		if($ctResource['penalty_dst']){
			//  
			if($ctResource['penalty_type']==1){
				//   
				if($this->calculator->cmp($plan['use_plan'], $plan['use_actual']) > 0){
					// 
					$this->logger->debug("%s's resource [%s] use plan incomplete, charging", $this->client['login'], $ctResource['resource_type']);
					$diff = $this->calculator->sub($plan['use_plan'], $plan['use_actual']);
					$this->accountsManager->actualMove($this->client['account'], $ctResource['penalty_dst'], $diff);
				}
			}
			//  
			if($ctResource['penalty_type']==2){
				$diff = $this->calculator->sub($plan['use_plan'], $plan['use_actual']);
				//      ,    
				if($this->calculator->cmp($diff, $ctResource['use_plan']) > 0){
					// 
					$penalty = $this->calculator->sub($diff, $ctResource['use_plan']);
					$this->logger->debug("%s's resource [%s] use plan [%s] incomplete [%s], charging [%s]", 
						$this->client['login'], $ctResource['resource_type'], $plan['use_plan'], $plan['use_actual'], $penalty);
					$this->accountsManager->actualMove($this->client['account'], $ctResource['penalty_dst'], $penalty);
				}
			}
		}
	}
}

function firstPersonalPeriod(){
	$connection_type = $this->connectionTypesManager->getConnectionType($this->client['connection_type']);
	$tv_id = $this->client['tv_type'];
	$this->logger->debug("trying to start first personal period for client '%s', connection type: [%s] '%s'", $this->client['login'], $connection_type['id'], $connection_type['name']);
	$this->transactionManager->begin();
	$this->newPlans = array();
	$this->newPlanSum = '0';
	//   
	$this->connectionTypesManager->walkResources($connection_type['id'], array($this, 'calcFirstPlan'));
	$need_first_charge = !$connection_type['credit'];
	if($this->canStartPersonalPeriod($connection_type, $tv_id, $this->newPlanSum, $need_first_charge)){
		$new = $this->createPersonalPeriod($connection_type, $tv_id);
		$this->openPeriod($new, $need_first_charge);
		$this->personalPeriodsDAO->changeMarker($new['id'], '1');
		$this->personalPeriodsManager->setClientPersonalPeriod($new);
		$this->periodsChanged = true;
	}else{
		$this->logger->debug("can't start personal period for '%s'", $this->client['login']);
	}
	$this->transactionManager->end();
}

function nextCandidatePeriod($candidate){
	$connection_type = $this->connectionTypesManager->getConnectionType($candidate['connection_type']);
	$tv_id = $candidate['tv_type'];
	$this->logger->debug("trying to open candidate personal period for client '%s', connection type [%s]", $this->client['login'], $connection_type['id']);
	$this->transactionManager->begin();
	$this->newPlans = array();
	$this->newPlanSum = '0';
	//   
	$this->connectionTypesManager->walkResources($connection_type['id'], array($this, 'calcNextPlan'));
	if($this->canStartPersonalPeriod($connection_type, $tv_id, $this->newPlanSum)){
		$this->moveMarker($this->period['id'], $candidate['id']);
		$candidate['marker'] = '1';
		$this->openPeriod($candidate);
		$this->personalPeriodsManager->setClientPersonalPeriod($candidate);
		$this->periodsChanged = true;
	}else{
		$this->logger->debug("can't start personal period for '%s'", $this->client['login']);
	}
	$this->transactionManager->end();
}

function hasNonCreditPeriod($clid){
	$it = $this->personalPeriodsDAO->getUsedConnectionTypes($clid);
	while($arr = $it->next()){
		$ct = $this->connectionTypesManager->getConnectionType($arr['connection_type']);
		if(!$ct['credit']){
			return true;
		}
	}
}

function nextPersonalPeriod(){
	$connection_type = $this->connectionTypesManager->getConnectionType($this->client['connection_type']);
	$tv_id = $this->client['tv_type'];
	$this->logger->debug("trying to start next personal period for client '%s', connection type [%s]", $this->client['login'], $connection_type['id']);
	$this->transactionManager->begin();
	$this->newPlans = array();
	$this->newPlanSum = '0';
	//   
	$this->connectionTypesManager->walkResources($connection_type['id'], array($this, 'calcNextPlan'));
	$need_first_charge = !($connection_type['credit'] || $this->hasNonCreditPeriod($this->client['id']));
	if($this->canStartPersonalPeriod($connection_type, $tv_id, $this->newPlanSum, $need_first_charge)){
		//  -      
		$prev_con_type = $this->connectionTypesManager->getConnectionType($this->period['connection_type']);
		//   ,        
		$cut = $prev_con_type['credit'] ? ($this->period['period_end'] - $this->period['period_begin']) : 0;
		$new = $this->createPersonalPeriod($connection_type, $tv_id, $cut);
		$this->moveMarker($this->period['id'], $new['id']);
		$new['marker'] = '1';
		$this->openPeriod($new, $need_first_charge);
		$this->personalPeriodsManager->setClientPersonalPeriod($new);
		$this->periodsChanged = true;
	}else{
		$this->logger->debug("can't start personal period for '%s'", $this->client['login']);
	}
	$this->transactionManager->end();
}

function compareBonuses($tid1, $tid2){
	$t1 = $this->accountTypesManager->getAccountType($tid1);
	$t2 = $this->accountTypesManager->getAccountType($tid2);
	$p1 = $t1['priority'];
	$p2 = $t2['priority'];
	return $p2 - $p1;
}

function prepareCanStarter($connection_type){
	$this->canStarter = array(
		'deposit' => $this->accountsManager->getVirtualState($this->client['account']),
		'bonuses'=>array(),
	);
	$this->bonusesManager->walkClientActualBonuses($this->client['id'], array($this, 'prepareCanStarterBonus'));
	$this->connectionTypesManager->walkBonuses($connection_type['id'], array($this, 'prepareCanStarterBonus2'));
	uksort($this->canStarter['bonuses'], array($this, 'compareBonuses'));
}

function prepareCanStarterBonus($bonus){
	$this->canStarter['bonuses'][$bonus['account_type']] = $this->accountsManager->getVirtualState($bonus['account']);
}

function prepareCanStarterBonus2($ctBonus){
	if($ctBonus['start_src'] && ($ctBonus['start_amount'] > 0)){
		if(isset($this->canStarter['bonuses'][$ctBonus['account_type']])){
			$this->canStarter['bonuses'][$ctBonus['account_type']] = $this->calculator->add($this->canStarter['bonuses'][$ctBonus['account_type']], $ctBonus['start_amount']);
		}else{
			$this->canStarter['bonuses'][$ctBonus['account_type']] = $ctBonus['start_amount'];
		}
	}
}

function canStartThis($rtid, $cost, $nochange = false){
	$this->canStartNoChange = $nochange;
	$this->canStartRest = $cost;
	$this->canStartResourceType =  $rtid;
	if($rtid){
		foreach($this->canStarter['bonuses'] as $tid=>$amount){
			$this->canStartThisBonus($tid);
		}
	}
	$deposit = $this->calculator->sub($this->canStarter['deposit'], $this->canStartRest);
	if(!$this->canStartNoChange){
		$this->canStarter['deposit'] = $deposit;
	}
	$this->canStartRest = 0;
	$result = !$this->depositBlocker->isDepositBlocked($this->client, $deposit);
	//$this->logger->debug('can start? rt [%s], amount [%s]: %s', $rtid, $cost, $result ? 'yes' : 'no');
	//$this->logger->debug(print_r($this->canStarter, true));
	return $result;
}

function canStartThisBonus($account_type){
	if($this->calculator->cmp($this->canStartRest, 0) > 0){
		if($this->accountTypesManager->isApplicable($account_type, $this->canStartResourceType)){
			$state = $this->canStarter['bonuses'][$account_type];
			if($this->calculator->cmp($state, 0) > 0){
				$fit = $this->calculator->min($state, $this->canStartRest);
				$this->canStartRest = $this->calculator->sub($this->canStartRest, $fit);
				if(!$this->canStartNoChange){
					$this->canStarter['bonuses'][$account_type] = $this->calculator->sub($state, $fit);
				}
			}
		}
	}
}

function canStartSubscriptions($sub){
	if($this->subResult){
		$type = $this->chargeTypesManager->getChargeType($sub['type']);
		$cost = $this->calculator->divLow($this->calculator->mul($sub['amount'], $type['price']), $type['unit_size']);
		$this->subResult = $this->canStartThis($type['resource_type'], $cost);
	}
}

function canStartPersonalPeriod($connection_type, $tv_id, $plan, $need_first_charge=false){
	if($connection_type['start_always']){
		return true;
	}
	$deposit = $this->accountsManager->getVirtualState($this->client['account']);
	$this->prepareCanStarter($connection_type);
	//$this->logger->debug(print_r($this->canStarter, true));
	//   
	if(!$this->canStartThis('', 0)){
		//     
		$this->logger->debug('%s lack for any, depo %s', $this->client['login'], $this->canStarter['deposit']);
		return false;
	}
	// 
	if($connection_type['abon_resource_type'] && $connection_type['abon_dst']){
		if(!$this->canStartThis($connection_type['abon_resource_type'], $connection_type['abon_amount'])){
			//    
			$this->logger->debug('%s lack for abon', $this->client['login']);
			return false;
		}
	}
	// 
	$this->subResult = true;
	//     ,      
	if(!$connection_type['credit']){
		$this->subscriptionsManager->walkClientSubscriptions($this->client['id'], array($this, 'canStartSubscriptions'));
	}
	if(!$this->subResult){
		//    
		$this->logger->debug('%s lack for subs', $this->client['login']);
		return false;
	}
	// 
	if($tv_id && !$connection_type['credit']){ //     
		if($tv = $this->connectionTypesManager->getTv($connection_type['id'], $tv_id)){
			if(!$this->canStartThis($this->optionsManager->getTrimOptionValue('TV_RESOURCE_TYPE'), $tv['cost'])){
				//    
				$this->logger->debug('%s lack for tv', $this->client['login']);
				return false;
			}
		}else{
			$this->logger->warning('bad tv type [%s] for connection type [%s]', $tv_id, $connection_type['id']);
		}
	}
	// 
	if(!$this->canStartThis('', $plan)){
		//    , 
		$this->logger->debug('%s lack for plan %s, depo %s', $this->client['login'], $plan, $this->canStarter['deposit']);
		return false;
	}
	//         :
	if($need_first_charge && ($this->calculator->cmp($connection_type['first_charge_amount'], '0') > 0)){
		//     
		if($this->calculator->cmp($connection_type['first_free_amount'], 0) > 0 ) { 
			//       "  ":
			$test_deposit = $this->calculator->sub($deposit, $connection_type['first_free_amount']);
			if(!$this->depositBlocker->isDepositBlocked($this->client, $test_deposit)){
				return true;
			}
		}
		//    - 
		if($this->calculator->cmp($connection_type['first_free_limit'], 0) > 0 ) { 
			//     "  "
			$test_deposit = $this->calculator->sub($deposit, $connection_type['first_free_limit']);
			if(!$this->depositBlocker->isDepositBlocked($this->client, $test_deposit)){
				return true;
			}
		}
		//     -     :
		if(!$this->canStartThis($this->optionsManager->getTrimOptionValue('CONNECTION_RESOURCE_TYPE'), $connection_type['first_charge_amount'])){
			//     
			$this->logger->debug('%s lack for connection-fee', $this->client['login']);
			return false;
		}
	}
	return true;
}

function createPersonalPeriod($connection_type, $tv_id, $cut = 0){
	$time = $this->timeService->getTime();
	//   
	$period_end = mktime(
		date("H", $time),
		date("i", $time),
		date("s", $time),
		date("m", $time) + $connection_type['period_months'],
		date("d", $time) + $connection_type['period_days'],
		date("Y", $time));
	$period_end -= $cut;
	$id = $this->personalPeriodsDAO->insert($this->client['id'], $connection_type['id'], $tv_id, $time, $period_end);
	$this->logger->debug("created %s's personal period [%s - %s]", $this->client['login'], date("Y-m-d", $time), date("Y-m-d", $period_end));
	return array(
		'id'=>$id,
		'client'=>$this->client['id'],
		'connection_type'=>$connection_type['id'],
		'tv_type'=>$tv_id,
		'state'=>0,
		'period_begin'=>$time,
		'period_end'=>$period_end,
		'new'=>1,
	);
}

function payThis2($dst, $rtid, $unit, $price, $amount){
	$this->payManager->payThis($this->client, $this->openingPeriod['id'], $dst, $rtid, $unit, $price, $amount);
}

function makeFirstCharge($connection_type){
	$deposit = $this->accountsManager->getVirtualState($this->client['account']);
	//      
	if($this->calculator->cmp($connection_type['first_charge_amount'], '0') > 0){
		//     
		if($this->calculator->cmp($connection_type['first_free_amount'], 0) > 0 ) { 
			//       "  ":
			$test_deposit = $this->calculator->sub($deposit, $connection_type['first_free_amount']);
			if(!$this->depositBlocker->isDepositBlocked($this->client, $test_deposit)){
				$this->logger->debug("%s is free from first charge", $this->client['login']);
				//   
				return;
			}
		}
		//    - 
		if($this->calculator->cmp($connection_type['first_free_limit'], 0) > 0 ) { 
			//     "  "
			$test_deposit = $this->calculator->sub($deposit, $connection_type['first_free_limit']);
			if(!$this->depositBlocker->isDepositBlocked($this->client, $test_deposit)){
				$this->logger->debug("%s is free from first charge, but got a debt", $this->client['login']);
				//   c     
				$diff = $this->calculator->sub($connection_type['first_free_amount'], $deposit);
				$stamp = $this->stampManager->createStamp();
				$this->connection->update("INSERT INTO client_notes (client, note, stamp) VALUES (%s, '%s', '%s')", $this->client['id'], "   : ". ($diff / 100)." .", $stamp);
				//   
				return;
			}
		}
		//    ,      
		$this->logger->debug("%s got first charging", $this->client['login']);
		$rt = $this->optionsManager->getTrimOptionValue('CONNECTION_RESOURCE_TYPE');
		if(!$rt){
			$this->logger->error('unconfigured CONNECTION_RESOURCE_TYPE variable, cant charge %s', $this->client['login']);
			return;
		}
		$acc = $this->optionsManager->getTrimOptionValue('CONNECTION_ACCOUNT');
		if(!$acc){
			$this->logger->error('unconfigured CONNECTION_ACCOUNT variable, cant charge %s', $this->client['login']);
			return;
		}
		$this->payThis2($acc, $rt, 1, $connection_type['first_charge_amount'], 1);
	}
}

function payTv($connection_type, $tv_id){
	if($tv_id){
		$tv = $this->connectionTypesManager->getTv($connection_type['id'], $tv_id);
		if(!$tv){
			$this->logger->error('bad tv type [%s] for connection type [%s]', $tv_id, $connection_type['id']);
			return;
		}
		if($this->calculator->cmp($tv['cost'], '0') > 0){
			$rt = $this->optionsManager->getTrimOptionValue('TV_RESOURCE_TYPE');
			if(!$rt){
				$this->logger->error('unconfigured TV_RESOURCE_TYPE variable, cant charge %s', $this->client['login']);
				return;
			}
			$acc = $this->optionsManager->getTrimOptionValue('TV_ACCOUNT');
			if(!$acc){
				$this->logger->error('unconfigured TV_ACCOUNT variable, cant charge %s', $this->client['login']);
				return;
			}
			$this->payThis2($acc, $rt, 1, $tv['cost'], 1);
		}
	}
}

function openPeriod(&$period, $need_first_charge = false){
	$this->openingPeriod = &$period;
	$connection_type = $this->connectionTypesManager->getConnectionType($period['connection_type']);
	//  
	$this->connectionTypesManager->walkBonuses($connection_type['id'], array($this, 'openBonus'));
	//   
	$this->connectionTypesManager->walkResources($connection_type['id'], array($this, 'openResource'));
	$this->resourceUsesManager->begin();
	$this->personalPeriodResourceUsesManager->begin();
	//   -    
	if($need_first_charge){
		$this->makeFirstCharge($connection_type);
	}
	//  
	if($connection_type['abon_resource_type'] && $connection_type['abon_dst'] && ($this->calculator->cmp($connection_type['abon_amount'], '0') > 0)){
		$this->payThis2($connection_type['abon_dst'], $connection_type['abon_resource_type'], 1, $connection_type['abon_amount'], 1);
	}
	$this->isCreditOpening = $connection_type['credit'];
	//   TV
	if(!$this->isCreditOpening){
		$this->payTv($connection_type, $period['tv_type']);
	}
	//    
	$this->subscriptionsManager->walkClientSubscriptions($this->client['id'], array($this, 'moveSubscription'));
	$this->personalPeriodResourceUsesManager->apply();
	$this->resourceUsesManager->apply();
	$this->personalPeriodsDAO->changeState($this->openingPeriod['id'], 1);
	$this->openingPeriod['state'] = 1;
	//      
	if($connection_type['replace_by'] && ($connection_type['replace_after'] > 0)){
		$replace = true;
		$prev = $this->openingPeriod;
		$i = 1;
		while($i < $connection_type['replace_after']){
			if($prev = $this->personalPeriodsDAO->getPrev($prev['id'])){
				if($prev['connection_type']==$this->openingPeriod['connection_type']){
					$i++;
					continue;
				}
				if($prev['connection_type']==$connection_type['credit_con_type']){
					continue;
				}
			}
			$replace = false;
			break;
		}
		if($replace){
			$this->clientsManager->changeConnectionType($this->client['id'], $connection_type['replace_by']);
		}
	}
	//      
	if($this->client['tv_type']){
		if($connection_type_tv = $this->connectionTypesManager->getTv($connection_type['id'], $this->client['tv_type'])){
			if($connection_type_tv['replace_by'] && ($connection_type_tv['replace_after'] > 0)){
				$replace = true;
				$prev = $this->openingPeriod;
				$i = 1;
				while($i < $connection_type_tv['replace_after']){
					if($prev = $this->personalPeriodsDAO->getPrev($prev['id'])){
						if(($prev['connection_type']==$this->openingPeriod['connection_type']) && ($prev['tv_type']==$this->openingPeriod['tv_type'])){
							$i++;
							continue;
						}
						if($prev['connection_type']==$connection_type['credit_con_type']){
							continue;
						}
					}
					$replace = false;
					break;
				}
				if($replace){
					$this->clientsManager->changeTvType($this->client['id'], $connection_type_tv['replace_by']);
				}
			}
		}else{
			$this->logger->error('bad tv type [%s] for connection type [%s]', $this->client['tv_type'], $connection_type['id']);
		}
	}
}

function moveSubscription($sub){
	$type = $this->chargeTypesManager->getChargeType($sub['type']);
	if($this->chargeTypesManager->driverStart($sub, $this->client, $this->openingPeriod)){
		if(!$this->isCreditOpening){
			//$cost = $this->calculator->divLow($this->calculator->mul($sub['amount'], $type['price']), $type['unit_size']);
			$this->payThis2($type['account'], $type['resource_type'], $type['unit_size'], $type['price'], $sub['amount']);
		}
	}else{
		$this->logger->warning('driver cant process subscription [%s] for client [%s]', $sub['type'], $this->client['login']);
	}
}

function openBonus($ctBonus){
	if($ctBonus['start_src'] && ($ctBonus['start_amount'] > 0)){
		$type = $this->accountTypesManager->getAccountType($ctBonus['account_type']);
		$bonus = $this->bonusesManager->getBonus($this->client['id'], $ctBonus['account_type']);
		if(!$bonus){
			$account = $this->accountsManager->createAccount(
				'B', $this->client['last_name'].' '.$this->client['first_name'].' ('.$this->client['login'].') - '.$type['name']);
			$bonus = $this->bonusesManager->createBonus($account['id'], $ctBonus['account_type'], $this->client['id']);
		}
		$this->accountsManager->actualMove($ctBonus['start_src'], $bonus['account'], $ctBonus['start_amount']);
	}
}

function openResource($ctResource){
	//  
	$plan = $this->newPlans[$ctResource['resource_type']]; //    
	$this->personalPeriodPlansManager->addPlan($this->openingPeriod['id'], $ctResource['resource_type'], $plan, $ctResource['use_limit']);
}

function calcFirstPlan($ctResource){
	$this->newPlans[$ctResource['resource_type']] = $ctResource['use_plan'];
	$this->newPlanSum = $this->calculator->add($this->newPlanSum, $ctResource['use_plan']);
}

function calcNextPlan($ctResource){
	$new_plan = $ctResource['use_plan'];
	//      
	if($old_plan = $this->personalPeriodPlansManager->getPlan($this->period['id'], $ctResource['resource_type'])){
		//     
		if($this->calculator->cmp($old_plan['use_plan'], $old_plan['use_actual']) > 0){
			if($oldResource = $this->connectionTypesManager->getResource($this->period['connection_type'], $ctResource['resource_type'])){
				$diff = $this->calculator->sub($old_plan['use_plan'], $old_plan['use_actual']);
				if(!$oldResource['penalty_dst'] || ($oldResource['penalty_type']==0)){
					//      
					$this->logger->debug("%s's plan enlargement for resource type [%s]: %s+%s", $this->client['login'], $oldResource['resource_type'], $new_plan, $diff);
					$new_plan = $this->calculator->add($new_plan, $diff);
				}
				if($oldResource['penalty_dst'] && ($oldResource['penalty_type']==2)){
					if($this->calculator->cmp($diff, $oldResource['use_plan']) > 0){
						$up = $oldResource['use_plan'];
					}else{
						$up = $diff;
					}
					//      
					$this->logger->debug("%s's plan enlargement for resource type [%s]: %s+%s", $this->client['login'], $oldResource['resource_type'], $new_plan, $up);
					$new_plan = $this->calculator->add($new_plan, $up);
				}
			}
		}else{
			//   
			if($this->calculator->cmp($old_plan['use_limit'], '0') > 0){
				//    
				if($this->calculator->cmp($old_plan['use_actual'], $old_plan['use_limit']) > 0){
					//    
					$diff = $this->calculator->sub($old_plan['use_actual'], $old_plan['use_limit']);
					$this->logger->debug("%s's plan reduction for resource type [%s]: %s-%s", $this->client['login'], $ctResource['resource_type'], $new_plan, $diff);
					$new_plan = $this->calculator->sub($new_plan, $diff);
				}
			}
		}
	}
	$this->newPlans[$ctResource['resource_type']] = $new_plan;
	$this->newPlanSum = $this->calculator->add($this->newPlanSum, $new_plan);
}

function getLostClientPersonalPeriod($clid){
	if($period = $this->personalPeriodsDAO->getLastMarked($clid)){
		$period['marker'] = 1;
		$this->personalPeriodsDAO->changeMarker($period['id'], '1');
		$this->personalPeriodsManager->setClientPersonalPeriod($period);
		$this->periodsChanged = true;
		return $period;
	}
}

function moveMarker($from, $to){
	$this->personalPeriodsDAO->changeMarker($from, '');
	$this->personalPeriodsDAO->changeMarker($to, '1');
	$this->personalPeriodsDAO->append($from, $to);
}


function fixPeriods(){
	$this->clientsManager->walk(array($this, 'fixClient'));
}

function fixClient($client){
	$this->client = $client;
	$period = $this->personalPeriodsManager->getClientPersonalPeriod($client['id']);
	if(!$period){
		$period = $this->getLostClientPersonalPeriod($client['id']);
	}
	if($period){
		if($period['state']==2){
			$this->fixingPeriod = $period;
			$this->transactionManager->begin();
			$this->connectionTypesManager->walkResources($period['connection_type'], array($this, 'fixResource'));
			$this->transactionManager->end();
		}
	}
}

function fixResource($ctResource){
	//   
	if($plan = $this->personalPeriodPlansManager->getPlan($this->fixingPeriod['id'], $ctResource['resource_type'])){
		//      
		if($ctResource['penalty_dst']){
			//  
			if($ctResource['penalty_type']==2){
				$diff = $this->calculator->sub($plan['use_plan'], $plan['use_actual']);
				//      ,    
				if($this->calculator->cmp($diff, $ctResource['use_plan']) > 0){
					// 
					$penalty = $this->calculator->sub($diff, $ctResource['use_plan']);
					$this->logger->info("%s's resource [%s] use plan [%s] incomplete [%s], charging [%s]", 
						$this->client['login'], $ctResource['resource_type'], $plan['use_plan'], $plan['use_actual'], $penalty);
					$this->accountsManager->actualMove($this->client['account'], $ctResource['penalty_dst'], $penalty);
				}
			}
		}
	}
}

}
?>