<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class ProfileBuilder {

var $logger;
var $optionsManager;
var $connectionPolicy;
var $clientsDAO;
var $modemsDAO;
var $modemProfilesDAO;
var $hostsDAO;
var $profiles;
var $clients;
var $connectionTypesManager;
var $blocksManager;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('profile.builder');
}

function setClientsDAO($obj){
	$this->clientsDAO = $obj;
}

function setHostsDAO($obj){
	$this->hostsDAO = $obj;
}

function setModemsDAO($obj){
	$this->modemsDAO = $obj;
}

function setModemProfilesDAO($obj){
	$this->modemProfilesDAO = $obj;
}

function setOptionsManager($obj){
	$this->optionsManager = $obj;
}

function setConnectionTypesManager($obj){
	$this->connectionTypesManager = $obj;
}

function setConnectionPolicy($obj){
	$this->connectionPolicy = $obj;
}

function setBlocksManager($obj){
	$this->blocksManager = $obj;
}

function getProfile($pid){
	if(isset($this->profiles[$pid])){
		return $this->profiles[$pid];
	}else{
		if($profile = $this->modemProfilesDAO->getById($pid)){
			$this->profiles[$pid] = $profile;
			return $profile;
		}else{
			$this->logger->error("Modem profile with id '%s' not found", $pid);
		}
	}
}

function getClient($clid){
	if(isset($this->clients[$clid])){
		return $this->clients[$clid];
	}else{
		if($client = $this->clientsDAO->getById($clid)){
			$client['dead'] = $this->connectionPolicy->isClientDisconnected($clid) ? 1 : 0;
			$client['admin_block'] = $this->blocksManager->isClientAdminDisconnected($clid) ? 1 : 0;
			$this->clients[$clid] = $client;
			return $client;
		}else{
			$this->logger->error("Client with id '%s' not found", $clid);
		}
	}
}

function render($template, $context, &$result){
	$err = 1;
	ob_start();
	extract($context);
	eval('?>'.$template.'<?;$err = 0;');
	$result = ob_get_contents();
	ob_end_clean();
	return $err==0;
}

function buildModem($mid){
//    echo $mid."\n";
	if($modem = $this->modemsDAO->getById($mid)){
//	    print_r($modem);
//	    return;
	    return $this->doProfiling($modem, $modem);
	}else{
		$this->logger->error("Modem with id '%s' not found", $mid);
		return 1;
	}
}

function buildChangedModem($mid, $old_modem){
	if($modem = $this->modemsDAO->getById($mid)){
		return $this->doProfiling($modem, $old_modem);
	}else{
		$this->logger->error("Modem with id '%s' not found", $mid);
		return 1;
	}
}

function doProfiling($modem, $old_modem){
	$opt = $this->optionsManager->getOption('MODEMS_CONFDIR');
	if($opt['actual']){
		$confDir = $opt['value'];
		if(substr($confDir,-1)!='/'){
		    $confDir .= '/';
		}
		$this->logger->debug("profiling modem [%s] %s", $modem['id'], $modem['ip']);
		$modem['hosts'] = array();
		$it = $this->hostsDAO->listByModem($modem['id']);
		while($host = $it->next()){
			$host['client'] = $this->getClient($host['client']);
			$modem['hosts'][] = $host;
		}
		$modem['modems'] = array();
		$it = $this->modemsDAO->listByModem($modem['id']);
		while($m = $it->next()){
			$modem['modems'][] = $m;
		}
		if($modem['modem'] > 0){
			$modem['modem'] = $this->modemsDAO->getById($modem['modem']);
		}
		$connection_types = $this->connectionTypesManager->getConnectionTypes();
		$profile = $this->getProfile($modem['profile']);
		if($this->render($profile['template'], array('modem'=>$modem, 'connection_types'=>$connection_types), $content)){
			$filename = $confDir.str_replace(array(':','-'),'',$modem['mac']).'.docsis';
			if($this->saveProfile($filename, $content)){
				 if($this->render($profile['executor'],array('modem'=>$modem,'old_modem'=>$old_modem,'filename'=>$filename), $executor)){
				 	if($exit_code = $this->execProfile($executor)){
						$this->logger->debug("Profiling modem [%s] failed", $modem['id']);
					}else{
						$this->logger->debug("Profiling modem [%s] succeed", $modem['id']);
					}
					return $exit_code;
				 }else{
				 	$this->logger->error("Can't render modem [%s] profile executor", $modem['id']);
				 }
			}
		}else{
			$this->logger->error("Can't render modem [%s] profile body", $modem['id']);
		}
	}else{
		$this->logger->debug("profiling disabled by configuration");
		return 0;
	}
	return 1;
}

function saveProfile($filename, $content){
	if($fd = @fopen($filename,"w")){
		fwrite($fd, $content);
		fclose($fd);
		@chown($filename, 'apache');
		$this->logger->debug("profile saved: %s", $filename);
		return true;
	}else{
		$this->logger->error("can't save %s", $filename);
		return false;
	}
}

function execProfile($executor){
	$exit_code = 0;
	$line = 0;
	foreach(explode("\n",$executor) as $command){
		$line++;
		$command = trim($command);
		if($command && $command[0]!='#'){
			$this->logger->debug("executing line %s: %s", $line, $command);
			if($silent = ($command[0]=='@')){
				$command = substr($command, 1);
			}
			$out = array();
//			echo $command;
			exec($command, $out, $exit_code);
			echo implode("\n", $out);
			if(!$silent && ($exit_code > 0)){
				$this->logger->debug("line %s returned exit code %d", $line, $exit_code);
				break;
			}else{
				$exit_code = 0;
			}
		}
	}
	return $exit_code;
}

}
?>