<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class ResourceUsesManager {

var $logger;
var $calculator;
var $resourceUsesDAO;
var $accountsManager;
var $updates;
var $calendarPeriodsManager;
var $calendarPeriod;
var $amounts;
var $registry;
var $loaded;
var $profiler;

function setProfiler($obj){
	$this->profiler = $obj;
}

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('resource.uses');
}

function setCalculator($obj){
	$this->calculator = $obj;
}

function setResourceUsesDAO($obj){
	$this->resourceUsesDAO = $obj;
}

function setAccountsManager($obj){
	$this->accountsManager = $obj;
}

function setCalendarPeriodsManager($obj){
	$this->calendarPeriodsManager = $obj;
}

function load(){
	if(!$this->loaded){
		$this->accountsManager->load();
		$this->calendarPeriod = $this->calendarPeriodsManager->getCurrent();
		$this->registry = array();
		$this->amounts = array();
		$it = $this->resourceUsesDAO->iterator($this->calendarPeriod['id']);
		while($a = $it->next()){
			if(isset($this->registry[$a['client']][$a['src']][$a['dst']][$a['resource_type']][$a['unit_size']][$a['price']])){
				$entry = &$this->registry[$a['client']][$a['src']][$a['dst']][$a['resource_type']][$a['unit_size']][$a['price']];
				$entry['last'] = max($entry['last'], $a['id']);
				$entry['items'][] = $a['id'];
			}else{
				$this->registry[$a['client']][$a['src']][$a['dst']][$a['resource_type']][$a['unit_size']][$a['price']] = array(
					'last'=>$a['id'],
					'items'=>array($a['id']),
				);
			}
			$this->amounts[$a['id']] = $a['amount'];
		}
		$this->logger->debug("%s resource uses loaded in calendar period [%s]", count($this->amounts), $this->calendarPeriod['id']);
		$this->loaded = true;
	}
}

function cost($amount, $price, $unit_size){
	$m = $this->calculator->mul($amount, $price);
	return $this->calculator->divLow($m, $unit_size);
}

function payResourceUses(){
	$this->logger->debug("paying uses in calendar period [%s]", $this->calendarPeriod['id']);
	foreach($this->registry as $clid=>$carr){
		foreach($carr as $src=>$sarr){
			if($src){
				foreach($sarr as $dst=>$darr){
					if($dst){
						foreach($darr as $rt=>$rarr){
							foreach($rarr as $unit_size=>$uarr){
								foreach($uarr as $price=>$entry){
									foreach($entry['items'] as $ru_id){
										$amount = $this->amounts[$ru_id];
										$ru_cost = $this->cost($amount, $price, $unit_size);
										if($this->calculator->cmp($ru_cost, '0') > 0){
											if($mov_id = $this->accountsManager->actualMove($src, $dst, $ru_cost)){
												//   
												$this->accountsManager->virtualMove($dst, $src, $ru_cost);
												$this->resourceUsesDAO->markMoved($ru_id, $mov_id);
												$this->logger->debug("payed use: ruid=%s clid=%s, rtid=%s, price=%s, unit=%s, cost=%s, src=%s, dst=%s, mov_id=%s",
													$ru_id, $clid, $rt, $price, $unit_size, $ru_cost, $src, $dst, $mov_id);
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
}

function begin(){
	$this->logger->debug("begin delayed updates pak");
	$this->updates = array();
}

function apply(){
	$this->profiler->enter('ru.apply');
	$this->logger->debug("applying delayed update pak");
	ksort($this->updates);
	foreach($this->updates as $id=>$amount){
		$this->resourceUsesDAO->update($id, $amount);
	}
	$this->profiler->leave('ru.apply');
}

function getAmount($clid, $src, $dst, $res_type, $unit_size, $price){
	if(isset($this->registry[$clid][$src][$dst][$res_type][$unit_size][$price])){
		$entry = &$this->registry[$clid][$src][$dst][$res_type][$unit_size][$price];
		return $this->amounts[$entry['last']];
	}else{
		return '0';
	}
}

function addAmount($clid, $src, $dst, $res_type, $unit_size, $price, $amount){
	if(isset($this->registry[$clid][$src][$dst][$res_type][$unit_size][$price])){
		$last = $this->registry[$clid][$src][$dst][$res_type][$unit_size][$price]['last'];
		$new_amount = $this->calculator->add($this->amounts[$last], $amount);
		$this->amounts[$last] = $new_amount;
		$this->updates[$last] = $new_amount;
		$this->logger->debug("add %s amount to existen resource use [%s]", $amount, $last);
		//  virtualMove,        
		$ru_cost = $this->cost($amount, $price, $unit_size);
		if($src && $dst){
			$this->accountsManager->virtualMove($src, $dst, $ru_cost);
		}
		return $last;
	}else{
		if($ru_id = $this->resourceUsesDAO->insert($this->calendarPeriod['id'], $clid, $res_type, $unit_size, $price, $src, $dst, $amount)){
			$this->registry[$clid][$src][$dst][$res_type][$unit_size][$price] = array(
				'last'=>$ru_id,
				'items'=>array($ru_id),
			);
			$this->amounts[$ru_id] = $amount;
			$this->logger->debug("created new resource use [%s] with %s amount", $ru_id, $amount);
			$ru_cost = $this->cost($amount, $price, $unit_size);
			if($src && $dst){
				$this->accountsManager->virtualMove($src, $dst, $ru_cost);
			}
			return $ru_id;
		}
	}
}

}
?>