<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class RulesManager {

var $logger;
var $rulesDAO;
var $rules;
var $beanFactory;
var $profiler;
var $iptNames;

function setProfiler($obj){
	$this->profiler = $obj;
}

function setIptNames($obj){
	$this->iptNames = $obj;
}

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('rules');
}

function setBeanFactory($obj){
	$this->beanFactory = $obj;
}

function setRulesDAO($obj){
	$this->rulesDAO = $obj;
}

function initBean(){
	$this->rules = array();
	$it = $this->rulesDAO->iterator();
	while($arr = $it->next()){
		$arr['driver'] = $this->beanFactory->getBean($arr['chain']);
		$this->rules[$arr['id']] = $arr;
	}
	$this->logger->debug("%s rules loaded", count($this->rules));
}

function getRule($id){
	return $this->rules[$id];
}

function readServerCounters(){
	$this->iptNames->loadNames();
	$this->iptNames->detectActive();
	$this->walk(array($this, 'readRuleCounters'));
}

function readCounters(){
	$this->logger->debug("begin read");
	$this->profiler->enter('rules.read');
	$this->iptNames->readNames();
	$this->iptNames->launchByServerRead();
	//$this->walk(array($this, 'readRuleCounters'));
	$this->profiler->leave('rules.read');
	$this->logger->debug("end read");
}

function loadCounters(){
	$this->logger->debug("begin load");
	$this->profiler->enter('rules.load');
	$this->iptNames->loadNames();
	$this->walk(array($this, 'loadRuleCounters'));
	$this->profiler->leave('rules.load');
	$this->logger->debug("end load");
}

function readRuleCounters($rule){
	$this->rules[$rule['id']]['driver']->readCounters();
}

function loadRuleCounters($rule){
	$this->rules[$rule['id']]['driver']->loadCounters();
}

function getCounter($rid, $ip){
	$this->profiler->enter('rules.access.counter');
	$counter = $this->rules[$rid]['driver']->getCounter($ip);
	$this->profiler->leave('rules.access.counter');
	return $counter;//0
}

function beginSetup(){
	$this->walk(array($this, 'beginRuleSetup'));
}

function beginRuleSetup($rule){
	$this->rules[$rule['id']]['driver']->beginSetup();
}

function setCounter($rid, $ip, $block){
	$this->rules[$rid]['driver']->setCounter($ip, $block);
}

function finishSetup(){
	$this->walk(array($this, 'finishRuleSetup'));
}

function finishRuleSetup($rule){
	$this->rules[$rule['id']]['driver']->finishSetup();
}

function walk($callback){
	foreach($this->rules as $rule){
		call_user_func($callback, $rule);
	}
}

function show(){
	print_r($this->rules);
}

}
?>