<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class SelfLimitationsManager {

var $logger;
var $selfLimitationsDAO;
var $personalPeriodsManager;
var $personalPeriodPlansManager;
var $resourceTypesManager;
var $calculator;
var $currencyFormat;
var $init;
var $limitations;
var $loaded;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('self.limitations');
}

function setSelfLimitationsDAO($obj){
	$this->selfLimitationsDAO = $obj;
}

function setPersonalPeriodsManager($obj){
	$this->personalPeriodsManager = $obj;
}

function setPersonalPeriodPlansManager($obj){
	$this->personalPeriodPlansManager = $obj;
}

function setResourceTypesManager($obj){
	$this->resourceTypesManager = $obj;
}

function setCalculator($obj){
	$this->calculator = $obj;
}

function setCurrencyFormat($obj){
	$this->currencyFormat = $obj;
}

function load(){
	if(!$this->loaded){
		$this->personalPeriodsManager->load();
		$this->personalPeriodPlansManager->load();
		$this->init = array();
		$this->limitations = array();
		$it = $this->selfLimitationsDAO->iterator();
		$counter = 0;
		while($arr = $it->next()){
			$this->limitations[$arr['client']][$arr['resource_type']] = array(
				'amount'=>$arr['amount'],
				'init'=>$this->isOver($arr['client'], $arr['resource_type'], $arr['amount']),
			);
			$counter++;
		}
		$this->logger->debug("%s self limitations loaded", $counter);
		$this->loaded = true;
	}
}

function isOver($clid, $rtid, $amount){
	if($period = $this->personalPeriodsManager->getClientPersonalPeriod($clid)){
		if($plan = $this->personalPeriodPlansManager->getPlan($period['id'], $rtid)){
			return $this->calculator->cmp($plan['use_actual'], $amount) >= 0;
		}
	}
}

function isResourceBlocked($clid, $rtid){
	if(isset($this->limitations[$clid][$rtid])){
		$limit = $this->limitations[$clid][$rtid]['amount'];
		if($this->calculator->cmp($limit, 0) > 0){
			return $this->isOver($clid, $rtid, $limit);
		}
	}
}

function notifyClient($client, &$context){
	$context['self_limitations'] = array();
	if(isset($this->limitations[$client['id']])){
		foreach($this->limitations[$client['id']] as $rtid=>$arr){
			if($this->calculator->cmp($arr['amount'], 0) > 0){
				if(!$arr['init'] && $this->isOver($client['id'], $rtid, $arr['amount'])){
					$rt = $this->resourceTypesManager->getResourceType($rtid);
					$context['self_limitations'][$rt['name']] = $this->currencyFormat->format($arr['amount']);
				}
			}
		}
	}
	return !empty($context['self_limitations']);
}

function show(){
	print_r($this->limitations);
}

}
?>