<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class ShapersManager {

var $logger;
var $shapersDAO;
var $clientShaperPatchesManager;
var $connectionTypesManager;
var $hostsManager;
var $optionsManager;
var $personalPeriodsManager;
var $personalPeriodPlansManager;
var $shapers;
var $calculator;
var $unshapeFrom = '01:00:00';
var $unshapeTo = '09:00:00';
var $connection;
var $loaded;

function setConnection($obj){
	$this->connection = $obj;
}

function setUnshapeFrom($str){
	$this->unshapeFrom = $str;
}

function setUnshapeTo($str){
	$this->unshapeTo = $str;
}

function setCalculator($obj){
	$this->calculator = $obj;
}

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('shapers');
}

function setShapersDAO($obj){
	$this->shapersDAO = $obj;
}

function setClientShaperPatchesManager($obj){
	$this->clientShaperPatchesManager = $obj;
}

function setConnectionTypesManager($obj){
	$this->connectionTypesManager = $obj;
}

function setHostsManager($obj){
	$this->hostsManager = $obj;
}

function setOptionsManager($obj){
	$this->optionsManager = $obj;
}

function setPersonalPeriodsManager($obj){
	$this->personalPeriodsManager = $obj;
}

function setPersonalPeriodPlansManager($obj){
	$this->personalPeriodPlansManager = $obj;
}

function cmpShapers($sh1, $sh2){
	$i1 = strlen($sh1['name']);
	$i2 = strlen($sh2['name']);
	return $i2 - $i1;
}

function load(){
	if(!$this->loaded){
		$this->hostsManager->load();
		$this->personalPeriodsManager->load();
		$this->personalPeriodPlansManager->load();
		$this->shapers = array();
		$it = $this->shapersDAO->iterator();
		while($arr = $it->next()){
			$this->shapers[$arr['id']] = $arr;
		}
		uasort($this->shapers, array($this, 'cmpShapers'));
		$this->logger->debug("%s shapers loaded", count($this->shapers));
		$this->loaded = true;
	}
}

function checkPlanRemake(){
	$this->checkPlanRemakeState = false;
	$this->personalPeriodsManager->walkPeriods(array($this, 'checkPlanPeriodRemake'));
	return $this->checkPlanRemakeState;
}

function checkPlanPeriodRemake($period){
	if(!$this->checkPlanRemakeState){
		if($this->connectionTypesManager->hasVariableShapers($period['connection_type'])){
			if($this->personalPeriodPlansManager->isJustComplete($period['id'])){
				$this->checkPlanRemakeState = true;
				$this->logger->info('detected just completed plans on con.types with variable shapers');
			}
		}
	}
}

function checkValueRemake(){
	$this->checkValueRemakeState = false;
	$this->personalPeriodsManager->walkPeriods(array($this, 'checkValuePeriodRemake'));
	return $this->checkValueRemakeState;
}

function checkValuePeriodRemake($period){
	$this->period = $period;
	if(!$this->checkValueRemakeState){
		$this->connectionTypesManager->walkShapers($period['connection_type'], array($this, 'checkValueShaperRemake'));
	}
}

function checkValueShaperRemake($ctShaper){
	if(!$this->checkValueRemakeState){
		if($ctShaper['use_case']==3){
			if($plan = $this->personalPeriodPlansManager->getPlan($this->period['id'], $ctShaper['use_resource_type'])){
				if(($this->calculator->cmp($plan['use_value_init'], $ctShaper['use_value']) < 0) &&
					($this->calculator->cmp($plan['use_value'], $ctShaper['use_value']) >= 0)){
					$this->checkValueRemakeState = true;
					$this->logger->info('detected just completed use values on con.types with variable shapers');
				}
			}
		}
	}
}

function loadUnshapedBySubscription(){
	$this->unshapedBySubscription = array();
	$t = date("H:i:s", time() + 60 * 5);
	if($this->unshapeFrom < $this->unshapeTo){
		$isUnshapeTime = ($t > $this->unshapeFrom) && ($t <= $this->unshapeTo);
	}else{
		$isUnshapeTime = ($t > $this->unshapeFrom) || ($t <= $this->unshapeTo);
	}
	if($isUnshapeTime){
		$sel = $this->connection->select("SELECT pp.client
			FROM personal_period_resource_uses ppru
			LEFT JOIN resource_uses ru ON ru.id=ppru.resource_use
			LEFT JOIN personal_periods pp ON pp.id=ppru.personal_period
			LEFT JOIN connection_types ct ON ct.id=pp.connection_type
			LEFT JOIN charge_types cht ON cht.id=ct.unshape_charge_type
			INNER JOIN clients c ON c.personal_period=pp.id
			WHERE pp.state=1 AND ru.resource_type=cht.resource_type");
		while($arr = $sel->next()){
			$this->unshapedBySubscription[$arr['client']] = 1;
		}
	}
}

function make(){
//	$this->logger->info("begin");
	$optTemplate = $this->optionsManager->getOption('SHAPER_TEMPLATE');
	if($optTemplate['actual']){
		$this->loadUnshapedBySubscription();
		$this->clientShaperPatchesManager->prepare(time() + 60 * 5);
		$this->shapersData = array();
		$this->shapersData2 = array(); //new
		$this->personalPeriodsManager->walkPeriods(array($this, 'shapePeriod'));
		$script = $optTemplate['value'];
		foreach($this->shapers as $shaper){
			$data = isset($this->shapersData[$shaper['id']]) ? $this->shapersData[$shaper['id']] : array();
			$data2 = isset($this->shapersData2[$shaper['id']]) ? $this->shapersData2[$shaper['id']] : array(); //new
			$rendered = $this->renderShaper($shaper['pattern'], $shaper['name'], $data, $data2); //changed
			$script = str_replace("%".$shaper['name'], $rendered, $script);
		}
		$optConfig = $this->optionsManager->getOption('SHAPER_CONFIG');
		if($optConfig['actual']){
			$filename = trim($optConfig['value']);
			$this->logger->debug("saving shaper script to '%s'", $filename);
			if($fd = @fopen ($filename, "w")){
				$writen = fwrite($fd, $script);
				fclose($fd);
				if($writen > 0){
					$optRestart = $this->optionsManager->getOption('SHAPER_RESTARTER');
					if($optRestart['actual']){
						$cmd = trim($optRestart['value']);
						if($cmd){
							$this->logger->debug("restarting: %s", $cmd);
							exec($cmd, $out, $exit_code);
							if($exit_code){
								$this->logger->warning("shaper restarting returned exit code [%d]", $exit_code);
								return $exit_code;
							}else{
								$this->logger->info("shaper restarted");
							}
						}else{
							$this->logger->warning("shaper restarter is empty");
						}
					}else{
						$this->logger->info("shaper restarting disabled by configuration, aborting");
					}
				}else{
					$this->logger->warning("no data saved in shaper script, aborting");
				}
			}else{
				$this->logger->error("can't open '%s' for writing", $filename);
			}
		}else{
			$this->logger->info("shaper saving disabled by configuration, aborting");
		}
	}else{
		$this->logger->info("shaper rebuilding disabled by configuration, aborting");
	}
}

function isUnshaped($clid){
	return $this->clientShaperPatchesManager->isPatched($clid) || isset($this->unshapedBySubscription[$clid]);
}

function shapePeriod($period){
	$this->period = $period;
	$this->clientId = $period['client'];
	$this->planState = $this->personalPeriodPlansManager->isAllPlansComplete($period['id']) ? 2 : 1;
	if(!$this->isUnshaped($this->clientId)){
		$this->connectionTypesManager->walkShapers($period['connection_type'], array($this, 'shapeCtShaper'));
	}
}

function shapeCtShaper($ctShaper){
	$this->ctShaper = $ctShaper;
	if($plan = $this->personalPeriodPlansManager->getPlan($this->period['id'], $ctShaper['use_resource_type'])){
		$use_over = $this->calculator->cmp($plan['use_value'], $ctShaper['use_value']) >= 0;
	}else{
		$use_over = false;
	}
	if($ctShaper['use_case']==0){
		$use_shaper = true;
	}elseif($ctShaper['use_case']==3){
		$use_shaper = $use_over;
	}elseif($ctShaper['use_case']==4){
		$use_shaper = !$use_over;
	}else{
		$use_shaper = $ctShaper['use_case']==$this->planState;
	}
	if($use_shaper){
		$this->hostsManager->walkClientHosts($this->clientId, array($this, 'shapeHost'));
	}
}

function shapeHost($host){
	$this->shapersData[$this->ctShaper['shaper']][] = $host['ip'];
	$this->shapersData2[$this->ctShaper['shaper']][$this->clientId][] = $host['ip'];//new
}

function renderShaper($pattern, $name, $hosts, $clients){//changed
	ob_start();
	eval('?>'.$pattern.'<?');
	$result = ob_get_contents();
	ob_end_clean();
	return $result;
}

function show(){
	print_r($this->shapers);
}

}
?>