<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class TrafficAccounter {

var $clientsPerTransaction = 1;
var $clientsCounter;

var $transactionGroup;
var $transaction;
var $logger;
var $calculator;
var $transactionManager;
var $timeService;
var $accountsManager;
var $accountTypesManager;
var $clientsManager;
var $connectionTypesManager;
var $clientShaperPatchesManager;

var $connectionPolicy;
var $bonusesManager;
var $hostsManager;
var $ipUseIntervalsDAO;
var $ipUseLogManager;
var $ipUsesManager;
var $personalFirewallManager;
var $personalPeriodsManager;
var $personalPeriodPlansManager;
var $personalPeriodResourceUsesManager;
var $resourceUsesManager;
var $rulesManager;
var $tariffsManager;
var $workDirOption;
var $fetchPeriod = 5;

var $interval;
var $personalPeriod;
var $client;
var $connectionType;
var $host;
var $ctRule;
var $ctResource;
var $rule;
var $tariff;
var $rest;
var $profiler;

function setClientsPerTransaction($int){
	$this->clientsPerTransaction = max(1, intval($int));
}

function setProfiler($obj){
	$this->profiler = $obj;
}

function setLogFactory($logFactory){
	$this->logger = $logFactory->getLogger('traffic.accounter');
}

function setCalculator($obj){
	$this->calculator = $obj;
}

function setTimeService($obj){
	$this->timeService = $obj;
}

function setAccountsManager($obj){
	$this->accountsManager = $obj;
}

function setAccountTypesManager($obj){
	$this->accountTypesManager = $obj;
}

function setClientsManager($obj){
	$this->clientsManager = $obj;
}

function setClientShaperPatchesManager($obj){
	$this->clientShaperPatchesManager = $obj;
}

function setConnectionTypesManager($obj){
	$this->connectionTypesManager = $obj;
}

function setConnectionPolicy($obj){
	$this->connectionPolicy = $obj;
}

function setBonusesManager($obj){
	$this->bonusesManager = $obj;
}

function setHostsManager($obj){
	$this->hostsManager = $obj;
}

function setIpUseIntervalsDAO($obj){
	$this->ipUseIntervalsDAO = $obj;
}

function setIpUseLogManager($obj){
	$this->ipUseLogManager = $obj;
}

function setIpUsesManager($obj){
	$this->ipUsesManager = $obj;
}

function setPersonalPeriodsManager($obj){
	$this->personalPeriodsManager = $obj;
}

function setPersonalPeriodPlansManager($obj){
	$this->personalPeriodPlansManager = $obj;
}

function setPersonalPeriodResourceUsesManager($obj){
	$this->personalPeriodResourceUsesManager = $obj;
}

function setPersonalFirewallManager($obj){
	$this->personalFirewallManager = $obj;
}

function setResourceUsesManager($obj){
	$this->resourceUsesManager = $obj;
}

function setRulesManager($obj){
	$this->rulesManager = $obj;
}

function setTariffsManager($obj){
	$this->tariffsManager = $obj;
}

function setTransactionManager($obj){
	$this->transactionManager = $obj;
}

function setWorkDirOption($workDirOption){
	$this->workDirOption = $workDirOption;
}

function setFetchPeriod($i){
	$this->fetchPeriod = $i;
}

function initBean(){
	$this->workdir = $this->workDirOption->getValue();
	if(substr($this->workdir, -1, 1)!='/'){
		$this->workdir .= '/';
	}
}

function writeTime(){
	$filename = $this->workdir.'accounting-make-date';
	if($file = @fopen($filename, "w+")){
		$this->logger->debug("Writing time to '%s'", $filename);
		fputs($file, $this->timeService->getTime());
		fclose($file);
	}
}

function readTime(){
	$filename = $this->workdir.'accounting-make-date';
	if($file = @fopen($filename, "r")){
		$this->logger->debug("Reading time from '%s'", $filename);
		$time = fgets($file);
		fclose($file);
		return $time;
	}
}

function getInterval(){
	if(!$this->interval){
		$this->logger->debug("registering interval [%s - %s]", date("Y-m-d H:i:s", $this->from_time), date("Y-m-d H:i:s", $this->to_time));
		$this->interval = $this->ipUseIntervalsDAO->insert($this->from_time, $this->to_time);
	}
	return $this->interval;
}

function calc($tariff, $amount){
	return $this->calculator->divLow($this->calculator->mul($amount, $tariff['price']), $tariff['unit_size']);
}

function getResourceUseAmount($rtid, $src, $dst, $tariff){
	return $this->resourceUsesManager->getAmount($this->client['id'], $src, $dst, $rtid, $tariff['unit_size'], $tariff['price']);
}

function registerUse($rtid, $src, $dst, $tariff, $amount){
	//   ru
	$ru = $this->resourceUsesManager->addAmount($this->client['id'], $src, $dst, $rtid, $tariff['unit_size'], $tariff['price'], $amount);
	//   ppru
	$ppru = $this->personalPeriodResourceUsesManager->addAmount($this->personalPeriod['id'], $ru, $amount);
	//   iu
	$iu = $this->ipUsesManager->addBytes($ppru, $this->host['id'], $this->ctRule['rule'], $amount);
	//   
	$this->ipUseLogManager->add($iu, $amount);
}

function payUse($rtid, $src, $dst, $tariff, $amount){
	//    ru
	$old_amount = $this->getResourceUseAmount($rtid, $src, $dst, $tariff);
	$old_cost = $this->calc($tariff, $old_amount);
	//      
	$new_amount = $this->calculator->add($old_amount, $amount);
	$new_cost = $this->calc($tariff, $new_amount);
	//  
	$diff = $this->calculator->sub($new_cost, $old_cost);
	//  
	$this->registerUse($rtid, $src, $dst, $tariff, $amount);
	return $diff;
}

function fetch(){
	$this->clientsManager->load();
	$this->hostsManager->load();
	$this->bonusesManager->load();
	$this->resourceUsesManager->load();
	$this->personalPeriodsManager->load();
	$this->personalPeriodResourceUsesManager->load();
	$this->personalPeriodPlansManager->load();
	$this->ipUsesManager->load();
	$this->logger->debug("begin");
	$this->to_time = $this->timeService->getTime();
	if(!($this->from_time = $this->readTime())){
		$this->from_time = $this->to_time - $this->fetchPeriod * 60;
	}
	$this->ipUseLogManager->setTimeInterval($this->from_time, $this->to_time);
	$this->clientShaperPatchesManager->prepare($this->from_time + $this->fetchPeriod * 30);
	$this->tariffsManager->load($this->from_time);
	
	$this->clientsCounter = 0;
	$this->transactionManager->begin();
	$this->clientsManager->walk(array($this, 'accountClient'));
	$this->transactionManager->end();
	$this->logger->debug("end");
}

function accountClient($client){
	$this->logger->debug("accounting client '%s'", $client['login']);
	$this->client = $client;
	if(($this->personalPeriod = $this->personalPeriodsManager->getClientPersonalPeriod($client['id'])) && $this->personalPeriod['state']==1){
		$this->profiler->enter('traffic.accounter.calc');
		$this->connectionType = $this->connectionTypesManager->getConnectionType($this->personalPeriod['connection_type']);
		$this->logger->debug("connection type is '%s'", $this->connectionType['name']);
		$this->clientsCounter++;
		if($this->clientsCounter > $this->clientsPerTransaction){
			$this->transactionManager->end();
			$this->transactionManager->begin();
			$this->clientsCounter = 1;
		}
		$this->personalPeriodPlansManager->begin();
		$this->resourceUsesManager->begin();
		//  ppru   iu  
		//$this->personalPeriodResourceUsesManager->begin();
		$this->ipUsesManager->begin();
		$this->ipUseLogManager->begin($this->personalPeriod['id']);
		$this->hostsManager->walkClientHosts($client['id'], array($this, 'accountHost'));
		$this->profiler->leave('traffic.accounter.calc');
		$this->profiler->enter('traffic.accounter.apply');
		$this->ipUseLogManager->apply();
		$this->ipUsesManager->apply();
		//$this->personalPeriodResourceUsesManager->apply();
		$this->resourceUsesManager->apply();
		$this->personalPeriodPlansManager->apply();
		
		$this->profiler->leave('traffic.accounter.apply');
	}else{
		$this->logger->debug("client '%s' has no active personal period, skipping", $client['login']);
	}
}

function accountHost($host){
	$this->logger->debug("accounting host '%s'", $host['ip']);
	$this->host = $host;
	$this->connectionTypesManager->walkResources($this->connectionType['id'], array($this, 'accountCtResource'));
}

function accountCtResource($ctResource){
	$this->logger->debug("accounting resource type [%s]", $ctResource['resource_type']);
	$this->ctResource = $ctResource;
	$this->connectionTypesManager->walkRules($ctResource['id'], array($this, 'accountCtRule'));
}

function accountCtRule($ctRule){
	$this->ctRule = $ctRule;
	$this->rule = $this->rulesManager->getRule($this->ctRule['rule']);
	$this->rest = $this->rulesManager->getCounter($this->ctRule['rule'], $this->host['ip']);
	if($this->hasRest()){
		$this->logger->debug("accounting %s bytes at rule [%s] '%s'", $this->rest, $this->rule['id'], $this->rule['name']);
		$src = $this->client['account'];
		$dst = $this->ctResource['use_dst'];
		$rtid = $this->ctResource['resource_type'];
		$complete = $this->personalPeriodPlansManager->isPlanComplete($this->personalPeriod['id'], $this->ctResource['resource_type']);
		$tid = $complete ? $this->ctRule['tariff_over'] : $this->ctRule['tariff'];
		$this->tariff = $this->tariffsManager->getTariff($tid);
		//  " ",    
		if($this->clientShaperPatchesManager->isPatched($this->client['id'])){
			$this->tariff['price'] = '0';
		}
		if($this->host['countable']){
			$this->personalPeriodPlansManager->addUseValue($this->personalPeriod['id'], $rtid, $this->rest);
			if($this->calculator->cmp($this->tariff['price'], '0') == 0){
				//         
				$cost = $this->payUse($rtid, $src, $dst, $this->tariff, $this->rest);
				$this->logger->debug("payed %s bytes as cost %s from deposit", $this->rest, $cost);
			}else{
				//   - 
				$this->bonusesManager->walkClientActualHighBonuses($this->client['id'], array($this, 'payRestByBonus'));
				if($this->hasRest()){
					if($complete){
						//   - 
						$this->bonusesManager->walkClientActualLowBonuses($this->client['id'], array($this, 'payRestByBonus'));
					}else{
						//    
						//         :
						$lack = $this->personalPeriodPlansManager->getPlanLack($this->personalPeriod['id'], $rtid);
						//        :
						$fit = $this->getResourceUseFit($rtid, $src, $dst, $this->tariff, $lack, $this->rest);
						// 
						$cost = $this->payUse($rtid, $src, $dst, $this->tariff, $fit);
						$this->logger->debug("payed %s bytes as cost %s from deposit", $fit, $cost);
						//   :
						$this->personalPeriodPlansManager->addPlanUse($this->personalPeriod['id'], $rtid, $cost);
						//  :
						$this->rest = $this->calculator->sub($this->rest, $fit);
						//       -    - 
						if($this->hasRest()){
							$this->bonusesManager->walkClientActualLowBonuses($this->client['id'], array($this, 'payRestByBonus'));
						}
					}
					//   ,     -   
					if($this->hasRest()){
						//   
						$cost = $this->payUse($rtid, $src, $dst, $this->tariff, $this->rest);
						$this->logger->debug("payed %s bytes as cost %s from deposit", $this->rest, $cost);
						//   :
						$this->personalPeriodPlansManager->addPlanUse($this->personalPeriod['id'], $rtid, $cost);
					}
				}
			}
		}else{
			//    ru
			$this->registerUse($rtid, '', '', $this->tariff, $this->rest); 
		}
	}
}

function payRestByBonus($bonus){
	if($this->hasRest()){
		$rtid = $this->ctResource['resource_type'];
		if($this->accountTypesManager->isApplicable($bonus['account_type'], $rtid)){
			$src = $bonus['account'];
			$dst = $this->ctResource['use_dst'];
			$state = $this->accountsManager->getVirtualState($bonus['account']); //    
			if($this->calculator->cmp($state, '0') > 0){
				//     
				$fit = $this->getResourceUseFit($rtid, $src, $dst, $this->tariff, $state, $this->rest);
				// 
				$cost = $this->payUse($rtid, $src, $dst, $this->tariff, $fit);
				//$type = $this->accountTypesManager->getAccountType($bonus['account_type']);
				$this->logger->debug("payed %s bytes as cost %s from bonus of type [%s]", $fit, $cost, $bonus['account_type']);
				//  :
				$this->rest = $this->calculator->sub($this->rest, $fit);
			}
		}
	}
}

function getResourceUseFit($rtid, $src, $dst, $tariff, $limit, $amount){
	if($this->calculator->cmp($tariff['price'], 0) > 0){
		$capacity = $this->getResourceUseCapacity($rtid, $src, $dst, $tariff, $limit);
		return $this->calculator->cmp($capacity, $amount) >= 0 ? $amount : $capacity;
	}else{
		return $amount;
	}
}

function getResourceUseCapacity($rtid, $src, $dst, $tariff, $limit){
	//    ru
	$ru_amount = $this->getResourceUseAmount($rtid, $src, $dst, $tariff);
	//   
	$ru_cost = $this->calc($tariff, $ru_amount);
	//     ru
	$max_cost = $this->calculator->add($limit, $ru_cost);
	//          ru
	$max_amount = $this->calculator->divHigh($this->calculator->mul($max_cost, $tariff['unit_size']), $tariff['price']);
	//      ru
	return $this->calculator->sub($max_amount, $ru_amount);
}

function hasRest(){
	return $this->calculator->cmp($this->rest, '0') > 0;
}

}
?>