<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

include_once dirname(__FILE__).'/Task.php';

class TaskARPRemake implements Task {

var $connection;
var $optionsManager;
var $logger;
var $scriptExecutor;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('task.arp.remake');
}

function setConnection($obj){
	$this->connection = $obj;
}

function setOptionsManager($obj){
	$this->optionsManager = $obj;
}

function setScriptExecutor($obj){
	$this->scriptExecutor = $obj;
}

function IpMatch($network, $mask, $ip) {
   $n = ip2long($network);
   $i = ip2long($ip);
   if (is_numeric($mask)) {// 10.10.10.0/24
       $m = ~((1<< (32-$mask))-1);
   }else{// 10.10.10.0/255.255.255.0
       $m = ip2long($mask); 
   }
   return !(($n & $m)-($i & $m));
}

function execute($data){
	$fileName = $this->optionsManager->getTrimOptionValue('ARP_CONFIG');
	if($fileName){
		$arp = array();
		$static = $this->optionsManager->getOptionValue('ARP_STATIC');
		if($static){
			$arp[] = $static;
		}
		$networks = array();
		$sel = $this->connection->select("SELECT ip FROM networks WHERE local");
		while($arr = $sel->next()){
			$parts = explode("/", $arr['ip']);
			$networks[] = array(
				'net'=>$parts[0],
				'mask'=>$parts[1],
			);
		}
		foreach(array("hosts", "modems") as $table){
			$res = $this->connection->select("SELECT ip, mac FROM $table WHERE mac>'' AND POSITION('/' IN ip)=0 ORDER BY ip");
			while($arr = $sel->next()){
				foreach($networks as $net){
					if($this->IpMatch($net['net'], $net['mask'], $arr['ip'])){
						$arp[] = $arr['ip']."\t".$arr['mac'];
					}
				}
			}
		}
		//  
		$config = implode("\n", $arp)."\n";
		if($fd = @fopen ($fileName, "w")){
			$writen = fwrite($fd, $config);
			fclose($fd);
			if($writen>0){
				//  ARP
				$this->logger->info('restarting arp');
				$command = $this->optionsManager->getOptionValue('ARP_RESTARTER');
				if($command){
					$err = $this->scriptExecutor->ExecuteScript($command, $this->logger);
					if($err){
						return 3; // failed
					}
				}
			}
		}else{
			$this->logger->error('cant save into %s', $fileName);
			return 3; // failed
		}
	}
	return 2; // ok
}

}
?>