<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

include_once dirname(__FILE__).'/Task.php';
include_once dirname(__FILE__).'/../tools/IpAddress.php';

class TaskDHCPRemake2 implements Task {

var $query = array(
	'subnets'=>"SELECT t.id, t.subnet, t.mask, t.gate, t.broadcast, t.notes, d.name as domain, name_server
		FROM dhcp_templates t
		LEFT JOIN domains d ON d.id=t.domain",
	'modems_width'=>"
		SELECT MAX(LENGTH(CONCAT(m.name,'.',IFNULL(d.name,'')))) as width
		FROM modems m
		LEFT JOIN domains d on d.id=m.domain
		INNER JOIN dhcp_modems dm ON dm.modem=m.id",
	'modems'=>"
		SELECT m.id, m.name, m.mac, m.ip, d.name as domain, dm.dhcp
		FROM modems m
		LEFT JOIN domains d ON m.domain = d.id
		LEFT JOIN modem_profiles mp ON m.profile=mp.id
		INNER JOIN dhcp_modems dm ON dm.modem=m.id
		WHERE  mac>'' AND mp.id is NOT NULL ORDER BY m.ip",
	'hosts_width'=>"
		SELECT MAX(LENGTH(CONCAT(h.name,'.',IFNULL(d.name,'')))) as width
		FROM hosts h
		LEFT JOIN domains d on d.id=h.domain
		INNER JOIN dhcp_hosts dh ON dh.host=h.id",
	'hosts'=>"
		SELECT h.id, h.name, h.mac, h.ip, d.name as domain, dh.dhcp
		FROM hosts h
		LEFT JOIN domains d ON h.domain = d.id
		INNER JOIN dhcp_hosts dh ON dh.host=h.id
		WHERE h.mac>'' AND h.ip not like '%/%' ORDER BY h.ip",
);

var $connection;
var $optionsManager;
var $logger;
var $scriptExecutor;
var $width = 10;
var $dhcpSubnetBuilder;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('task.dhcp.remake');
}

function setConnection($obj){
	$this->connection = $obj;
}

function setOptionsManager($obj){
	$this->optionsManager = $obj;
}

function setScriptExecutor($obj){
	$this->scriptExecutor = $obj;
}

function setDhcpSubnetBuilder($obj){
	$this->dhcpSubnetBuilder = $obj;
}

function loadSubnets(){
	$this->subnets = array();
	$sel = $this->connection->select($this->query['subnets']);
	while($arr = $sel->next()){
		$arr['addr'] = new IpAddress($arr['subnet'], $arr['mask']);
		$arr['items'] = array();
		$this->subnets[$arr['id']] = $arr;
	}
	$this->logger->debug("%s subnets loaded", count($this->subnets));
}

function loadHostsWidth(){
	$sel = $this->connection->select($this->query['hosts_width']);
	$arr = $sel->next();
	$this->width = max($this->width, $arr['width']);
}

function loadModemsWidth(){
	$sel = $this->connection->select($this->query['modems_width']);
	$arr = $sel->next();
	$this->width = max($this->width, $arr['width']);
}

function loadModems(){
	$modemsConfDir = $this->optionsManager->getTrimOptionValue('MODEMS_CONFDIR');
	if($modemsConfDir && is_dir($modemsConfDir)){
		$this->logger->debug("begin collecting modems");
		$this->loadModemsWidth();
		// DHCP   
		$sel = $this->connection->select($this->query['modems']);
		while($arr = $sel->next()){
			$filename = str_replace(array(':','-'),'',$arr['mac']).'.docsis';
			if(!$arr['name']){
				$arr['name'] = 'modem-'.$arr['id'];
			}
			if($arr['domain']){
				$arr['name'] .= '.'.$arr['domain'];
			}
			$ip = new SlashedIpAddress($arr['ip']);
			$k = $arr['dhcp'];
			if($this->subnets[$k]['addr']->contains($ip)){
				$this->subnets[$k]['items'][$arr['name']] = sprintf(
					"    host %-".$this->width."s { hardware ethernet %s; fixed-address %s; filename \"%s\";}", 
					$arr['name'], $arr['mac'], $arr['ip'], $filename
				);
			}
		}
		$this->logger->debug("end collecting modems");
	}
}

function loadHosts(){
	// DHCP   
	$this->logger->debug("begin collecting hosts");
	$sel = $this->connection->select($this->query['hosts']);
	while($arr = $sel->next()){
		if(!$arr['name']){
			$arr['name'] = 'host-'.$arr['id'];
		}
		if($arr['domain']){
			$arr['name'] .= '.'.$arr['domain'];
		}
		$ip = new SlashedIpAddress($arr['ip']);
		$k = $arr['dhcp'];
		if($this->subnets[$k]['addr']->contains($ip)){
			$this->subnets[$k]['items'][$arr['name']] = sprintf(
				"    host %-".$this->width."s { hardware ethernet %s; fixed-address %s;}",
				$arr['name'], $arr['mac'], $arr['ip']
			);
		}
	}
	$this->logger->debug("end collecting hosts");
}

function fillTemplate($config){
	// 
	$this->logger->debug("begin filling template");
	$replacement = "";
	foreach($this->subnets as $k=>$sub){
		$replacement .= "subnet ".$sub['subnet']." netmask ".$sub['mask']." {\n";
		$replacement .= sprintf('    option %-22s %s;', 'routers', $sub['gate'])."\n";
		$replacement .= sprintf('    option %-22s %s;', 'subnet-mask', $sub['mask'])."\n";
		if($sub['broadcast']){
			$replacement .= sprintf('    option %-22s %s;', 'broadcast-address', $sub['broadcast'])."\n";
		}
		if($sub['name_server']){
			$replacement .= sprintf('    option %-22s %s;', 'domain-name-servers', $sub['name_server'])."\n";
		}
		if($sub['domain']){
			$replacement .= sprintf('    option %-22s "%s";', 'domain-name', $sub['domain'])."\n";
		}
		if(strlen(trim($sub['notes'])) > 0){
			$replacement .= $sub['notes']."\n";
		}
		if(count($sub['items']) > 0){
			asort($sub['items']);
			$replacement .= "\n".implode("\n", $sub['items'])."\n";
		}
		$replacement .= "\n}\n\n";
	}
	$config = str_replace('#%subnets', $replacement, $config);
	$this->logger->debug("end filling template");
	return $config;
}


function execute($data){
	if(isset($data[0])){
		$this->dhcpSubnetBuilder->link_all();
	}
	$result = 2;
	$this->logger->debug("begin");
	$config = $this->optionsManager->getOptionValue('DHCP_TEMPLATE');
	if($config){
		$this->loadSubnets();
		$this->loadHostsWidth();
		$this->loadModems();
		$this->loadHosts();
		$config = $this->fillTemplate($config);
		// 
		$this->logger->debug("saving config");
		$fileName = $this->optionsManager->getTrimOptionValue('DHCP_CONFIG');
		$command = $this->optionsManager->getOptionValue('DHCP_RESTARTER');
		if($fd = @fopen ($fileName, "w")){
			$writen = fwrite($fd, $config);
			fclose($fd);
			if($writen>0){
				// DHCPD
				$this->logger->info('restarting DHCP');
				if($command){
					$err = $this->scriptExecutor->ExecuteScript($command, $this->logger);
					if($err){
						$result = 3;//failed
					}
				}
			}
		}else{
			$this->logger->error('cant save into %s', $fileName);
			$result = 3;//failed
		}
	}
	$this->logger->debug("end");
	return $result;//ok
}

}
?>