<?
class CLISocketServerDelegate {

var $command;
var $logger;
var $messageTerminator = '';
var $cmd_in;
var $cmd_out;

function CLISocketServerDelegate($command, $logger, $messageTerminator='', $cmd_in=0, $cmd_out=1){
	$this->command = $command;
	$this->logger = $logger;
	$this->messageTerminator = $messageTerminator;
	$this->cmd_in = $cmd_in;
	$this->cmd_out = $cmd_out;
}

function process($client){
	$buffer = @socket_read($client->socket, 32);
	if($buffer == ''){ // error or disconnected
		$code = socket_last_error($client->socket);
		if($code){
			$msg = socket_strerror($code);
			$this->logger->warning("%s:%s: %s, %s", $client->address, $client->port, $msg, $code);
		}else{
			$this->logger->info("%s:%s closed by remote", $client->address, $client->port);
			$client->close();
		}
	}else{
		if($this->messageTerminator){
			$pos = strpos($buffer, $this->messageTerminator);
			if($pos===false){
				$client->buffer .= $buffer;
			}else{
				$pos += strlen($this->messageTerminator);
				$client->buffer .= substr($buffer, 0, $pos);
				$this->processBuffer($client);
				$client->buffer = substr($buffer, $pos);
			}
		}else{
			$client->buffer = $buffer;
			$this->processBuffer($client);
		}
	}
}

function escape($str){
	return str_replace(array("\t", "\n", "\r", "\0"), array('\t', '\n','\r','\0'), $str);
}

function processBuffer($client){
	$this->logger->info("%s:%s request: [%s]", $client->address, $client->port, $this->escape($client->buffer));
	$cmd = trim($client->buffer);
	if($cmd=='quit'){
		$client->close();
		return;
	}
	if($cmd=='serverquit'){
		$client->server->stop();
		return;
	}
	$descriptorspec = array(
		0 => array("pipe", "r"), // stdin
		1 => array("pipe", "w"), // stdout
		2 => array("pipe", "w"), // stderr
	);
	if($this->cmd_in > 2){
		$descriptorspec[$this->cmd_in] = array("pipe", "r");
	}
	if($this->cmd_out > 2){
		$descriptorspec[$this->cmd_out] = array("pipe", "w");
	}
	if($process = proc_open($this->command, $descriptorspec, $pipes)){
		fwrite($pipes[$this->cmd_in], $client->buffer);
		fclose($pipes[$this->cmd_in]);
		$reply = stream_get_contents($pipes[$this->cmd_out]);
		fclose($pipes[$this->cmd_out]);
		if($this->cmd_in!=0){
			fclose($pipes[0]);
		}
		if($this->cmd_out!=1){
			$stdout = trim(stream_get_contents($pipes[1]));
			fclose($pipes[1]);
		}
		if($this->cmd_out!=2){
			$stderr = trim(stream_get_contents($pipes[2]));
			fclose($pipes[2]);
		}
		$exit_code = proc_close($process);
		if($exit_code){
			if($stdout){
				$this->logger->warning('stdout: '.$stdout);
			}
			if($stderr){
				$this->logger->warning('stderr: '.$stderr);
			}
			$this->logger->warning("%s:%s   reply: [%s], exit-code: %s", $client->address, $client->port, $this->escape($reply), $exit_code);
		}else{
			$this->logger->info("%s:%s   reply: [%s], exit-code: %s", $client->address, $client->port, $this->escape($reply), $exit_code);
		}
		socket_write($client->socket, $reply);
	}
}

}
?>