<?
interface Writer {

function open();

function write($str);

function close();

}

interface Reader {

function open();

function read();

function close();

}


class LineWriter implements Writer {

var $terminators;
var $buffer;
var $writer;


function LineWriter($writer, $terminators = array("\n")){
	$this->writer = $writer;
	$this->terminators = $terminators;
}

function open(){
	$this->writer->open();
}

function write($str){
	$this->buffer .= $str;
	$prev = 0;
	while(true){
		$pos = false;
		foreach($this->terminators as $terminator){
			$pos = strpos($this->buffer, $terminator, $prev);
			if($pos!==false){
				$this->writer->write(substr($this->buffer, $prev, $pos - $prev + strlen($terminator)));
				$prev = $pos + strlen($terminator);
				break;
			}
		}
		if($pos===false){
			break;
		}
	}
	$this->buffer = substr($this->buffer, $prev);
}

function close(){
	if(strlen($this->buffer) > 0){
		$this->writer->write($this->buffer);
	}
	$this->writer->close();
}

}

class CallbackWriter implements Writer {

var $write;
var $open;
var $close;

function CallbackWriter($write, $open = '', $close=''){
	$this->write = $write;
	$this->open = $open;
	$this->close = $close;
}

function write($str){
	call_user_func($this->write, $str);
}

function close(){
	if($this->close){
		call_user_func($this->close);
	}
}

function open(){
	if($this->open){
		call_user_func($this->open);
	}
}

}

class CallbackReader implements Reader {

var $read;
var $open;
var $close;

function CallbackReader($read, $open = '', $close=''){
	$this->read = $read;
	$this->open = $open;
	$this->close = $close;
}

function read(){
	return call_user_func($this->read);
}

function close(){
	if($this->close){
		call_user_func($this->close);
	}
}

function open(){
	if($this->open){
		call_user_func($this->open);
	}
}

}

class NoneReader implements Reader {

function open(){}

function read(){}

function close(){}

}

class NoneWriter implements Writer {

function open(){}

function write($str){}

function close(){}

}


?>