<?
class SocketServerException extends Exception {}

class SocketClient {

var $socket;
var $index;
var $address;
var $port;
var $server;

function SocketClient($server, $socket, $index){
	socket_getpeername($socket, $this->address, $this->port);
	$this->server = $server;
	$this->socket = $socket;
	$this->index = $index;
	$this->server->clients[$index] = $this;
}

function close(){
	$this->server->closeClient($this);
}

}

class SocketServer {

var $delegate;
var $logger;
var $localAddress;
var $localPort;
var $maxClients;
var $clients;
var $socket;
var $terminated = false;

function SocketServer($delegate, $logger, $localAddress, $localPort, $maxClients = 10){
	$this->delegate = $delegate;
	$this->logger = $logger;
	$this->localAddress = $localAddress;
	$this->localPort = $localPort;
	$this->maxClients = $maxClients;
}

function start($is_check = false){
	$this->socket = @socket_create(AF_INET, SOCK_STREAM, 0);
	if(!$this->socket){
		$code = socket_last_error($this->socket);
		$msg = socket_strerror($code);
		$this->logger->error('cant create socket: %s', $msg);
		throw new SocketServerException($msg, $code);
	}
	if(!@socket_bind($this->socket, $this->localAddress, $this->localPort)){
		if($is_check){
			return;
		}else{
			$code = socket_last_error($this->socket);
			$msg = socket_strerror($code);
			$this->logger->error('cant bind socket to %s:%s: %s', $this->localAddress, $this->localPort, $msg);
			throw new SocketServerException($msg, $code);
		}
	}
	if(!@socket_listen($this->socket)){
		$code = socket_last_error($this->socket);
		$msg = socket_strerror($code);
		$this->logger->error('cant listen socket at %s:%s: %s', $this->localAddress, $this->localPort, $msg);
		throw new SocketServerException($msg, $code);
	}
	$this->logger->info('server started at %s:%s, max clients: %s', $this->localAddress, $this->localPort, $this->maxClients);
	$this->process();
}

function stop(){
	$this->terminated = true;
}

function closeClient($client){
	@socket_close($client->socket);
	unset($this->clients[$client->index]);
	$this->logger->info("%s:%s disconnected, count: %s/%s", $client->address, $client->port, count($this->clients), $this->maxClients);
}

function process(){
	while(!$this->terminated){
		$read = array();
		$read[0] = $this->socket;
		for ($i = 0; $i < $this->maxClients; $i++){
			if (isset($this->clients[$i])){
				$read[$i + 1] = $this->clients[$i]->socket;
			}
		}
		$ready = socket_select($read, $write=null, $except=null, $timeout = null);
		if($ready === false){
			$code = socket_last_error($this->socket);
			$msg = socket_strerror($code);
			$this->logger->warning('cant do select at %s:%s: %s', $this->localAddress, $this->localPort, $msg);
		}
		if($ready > 0){
			if (in_array($this->socket, $read)) {
				$accepted = false;
				for ($i = 0; $i < $this->maxClients; $i++){
					if (!isset($this->clients[$i])){
						$socket = socket_accept($this->socket);
						$client = new SocketClient($this, $socket, $i);
						$this->logger->info("new connection accepted: %s:%s, count: %s/%s", $client->address, $client->port, count($this->clients), $this->maxClients);
						$accepted = true;
						break;
					}
				}
				if(!$accepted){
					$socket = socket_accept($this->socket);
					socket_getpeername($socket, $address, $port);
					socket_write($socket, "too many connections\n");
					socket_close($socket);
					$this->logger->warning('cant accept new connection from %s:%s: too many clients', $address, $port);
				}
			}
			foreach($this->clients as $client){
				if (in_array($client->socket, $read)) {
					$this->delegate->process($client);
				}
			}
		}
	}
	foreach($this->clients as $client){
		$this->closeClient($client);
	}
	socket_close($this->socket);
	$this->logger->info('server quit');
}

}
?>