<?
include_once "classes/log/StdoutAppender.php";
include_once "classes/log/FileAppender.php";
include_once "classes/log/StreamAppender.php";
include_once "classes/log/DefaultLogger.php";
include_once "classes/tools/IpAddress.php";
include("includes/environment.php");
include("functions/accounts.php");

class OSMPPayer {

var $account;
var $logger;

function OSMPPayer($logger){
	global $db;
	$this->logger = $logger;
	if($arr = $db->Next($db->Execute("SELECT account FROM system_accounts WHERE code='PAY_OSMP'"))){
		$this->account = $arr['account'];
	}
}

function getClientByContract($contract){
	return loadRecord(sprintf("SELECT c.id, c.login, c.contract, c.account, c.connection_type, c.first_name, c.last_name, 
		CONCAT(s.name, ' ', h.house, ', ',c.apartment) as address
		FROM clients c
		LEFT JOIN houses h ON h.id=c.house_id
		LEFT JOIN streets s ON s.id=h.street
		WHERE c.contract='%s'", addslashes($contract)));
}

function wrap($str){
	return iconv('windows-1251', 'utf-8', htmlspecialchars($str));
}

function escape($str){
	return str_replace(array("\t", "\n", "\r", "\0"), array('\t', '\n','\r','\0'), $str);
}

function response($trx_id, $code, $message, $body = ''){
	$date = str_replace(" ", "T", date("Y-m-d H:i:s"));
	$body = sprintf("<?xml version=\"1.0\" encoding=\"UTF-8\"?><response><osmp_txn_id>%s</osmp_txn_id>%s<result>%s</result><comment>%s</comment></response>", $trx_id, $body, $code, $this->wrap($message));
	if($code > 0){
		$this->logger->warning($message);
	}
	$this->logger->debug('response: '.$this->escape($body));
	header('Content-Type: text/xml; charset=utf-8');
	header('Content-Length: '.strlen($body));
	echo $body;
}

function process(){
	global $db;
	switch($_GET['command']){
		case 'check': return $this->check();
		case 'pay': return $this->pay();
		case 'cancel':  return $this->cancel();
	}
}

function cancel(){
	$this->logger->info("cancel request: %s", $_SERVER['QUERY_STRING']);
	$txn_id = $_GET['txn_id'];
	return $this->response($txn_id, 7, 'operation is unsupported');
}

function check(){
	$this->logger->info("check request: %s", $_SERVER['QUERY_STRING']);
	$contract = $_GET['account'];
	$txn_id = $_GET['txn_id'];
	$amount = $_GET['sum'];
	$iamount = intval(strval($amount * 100));
	if(!$this->account){
		return $this->response($txn_id, 8, 'service unavailable');
	}
	if(!$txn_id){
		return $this->response($txn_id, 300, 'transaction number is not specified');
	}
	if(!$contract){
		return $this->response($txn_id, 4, 'account is not specified');
	}
	if($iamount <= 0){
		return $this->response($txn_id, 241, 'sum is too low');
	}
	if($iamount > 1000000){
		return $this->response($txn_id, 242, 'sum is too high');
	}
	if($client = $this->getClientByContract($contract)){
		$this->logger->debug("check contract %s found: %s", $contract, $client['login']);
		$name = trim($client['first_name'].' '.$client['last_name']);
		$this->response($txn_id, 0, '', sprintf('<fields><field name="name">%s</field></fields>', $this->wrap($name)));
	}else{
		$this->logger->debug("check contract %s not found", $contract);
		$this->response($txn_id, 5, 'account not found');
	}
}

function pay(){
	global $db;
	$this->logger->info("pay request: %s", $_SERVER['QUERY_STRING']);
	$txn_id = $_GET['txn_id'];
	$contract = $_GET['account'];
	$amount = $_GET['sum'];
	$iamount = intval(strval($amount * 100));
	$date = $_GET['txn_date'];
	$pay_type = isset($_GET['pay_type']) ? intval($_GET['pay_type']) : 0;
	if(!$this->account){
		return $this->response($txn_id, 8, 'service unavailable');
	}
	if(!$txn_id){
		return $this->response($txn_id, 300, 'transaction number is not specified');
	}
	if(!$contract){
		return $this->response($txn_id, 4, 'account is not specified');
	}
	if(!$date){
		return $this->response($txn_id, 300, 'date is not specified');
	}
	if($iamount <= 0){
		return $this->response($txn_id, 241, 'sum is too low');
	}
	if($iamount > 1000000){
		return $this->response($txn_id, 242, 'sum is too high');
	}
	if($client = $this->getClientByContract($contract)){
		if($pay = loadRecord(sprintf("SELECT * FROM osmp_pays WHERE order_id='%s'", addslashes($txn_id)))){
			if($iamount==$pay['amount'] && $client['id']==$pay['client']){
				$this->logger->warning("pay %s already exists, ok", $txn_id);
				$this->response($txn_id, 0, 'already exists', sprintf("<prv_txn>%s</prv_txn><sum>%s</sum>", $pay['id'], $amount));
			}else{
				$this->logger->warning("pay %s already exists, diff: contract=%s, amount=%s", $txn_id, $client['contract'], $pay['amount']/100);
				$this->response($txn_id, 7, 'already exists but not match');
			}
		}else{
			$success = false;
			for($i=0; $i<3; $i++){
				$db->Begin();
				$mid = move($this->account, $client['account'], $iamount);
				$db->Execute(sprintf("INSERT INTO osmp_pays (client, movement, order_id, amount, date, pay_type)
						VALUES ('%s', '%s', '%s', '%s', '%s', '%s')",
						$client['id'], $mid,
						addslashes($txn_id),
						addslashes($iamount),
						addslashes($date),
						addslashes($pay_type)
				));
				if($db->hasErrors()){
					$this->logger->warning(implode("\n", $db->errors));
					$db->Rollback();
					sleep(3);
				}else{
					$db->Commit();
					$success = true;
					break;
				}
			}
			if($success){
				$pay = loadRecord(sprintf("SELECT * FROM osmp_pays WHERE order_id='%s'", addslashes($txn_id)));
				$this->logger->info('pay %s is ok', $txn_id);
				$this->response($txn_id, 0, 'ok', sprintf("<prv_txn>%s</prv_txn><sum>%s</sum>", $pay['id'], $amount));
			}else{
				$this->logger->error('pay %s cant be moved', $txn_id);
				$this->response($txn_id, 1, '');
			}
		}
	}else{
		$this->logger->debug("pay contract %s: not found", $contract);
		$this->response($txn_id, 5, 'account not found');
	}
}

}

if($access_zone = getTrimOption('OSMP_ACCESS_ZONE')){
	$valid = false;
	$ip = new SlashedIpAddress($_SERVER["REMOTE_ADDR"]);
	foreach(preg_split("|[\s,;]+|", $access_zone, -1, PREG_SPLIT_NO_EMPTY) as $ip_str){
		$zone = new SlashedIpAddress($ip_str);
		if($zone->contains($ip)){
			$valid = true;
			break;
		}
	}
	if(!$valid){
		exit;
	}
}



$appender = new FileAppender('/var/log/osmp.log');
$logger = new DefaultLogger();
$logger->setAppender($appender);
$logger->setName('osmp.payer');
$payer = new OSMPPayer($logger);
$payer->process();


?>