<?
include_once "includes/environment.php";
include_once "includes/core.php";
include_once "functions/accounts.php";
include_once "classes/xml/StringXmlParser.php";
include_once "classes/xml/XmlNodeDelegate.php";
include_once "classes/tools/PhpTemplate.php";
include_once "classes/log/FileAppender.php";
include_once "classes/log/DefaultLogger.php";

class WrongXmlException extends Exception {}

class WrongSignException extends Exception {}

class Privat24Handler {

var $pub_key;
var $priv_key;
var $token;
var $datetime;
var $account = '';
var $results = array(
	0=>'ok',
	1=>'  ',
	2=>'  ',
	3=>'    ',
	4=>'  ',
	5=>'   IP ',
	6=>'   .   ',
	7=>' ',
	99=>' ',
);
var $logger;

function Privat24Handler($logger){
	$this->logger = $logger;
	if($arr = loadRecord("SELECT account FROM system_accounts WHERE code='PAY_PRIVAT24'")){
		$this->account = $arr['account'];
	}
}

function uniName($name){
	$name = strtolower($name);
	$arr = explode(':', $name);
	return count($arr)==2 ? $arr[1] : $arr[0];
}

function isTag($node, $name){
	return $this->uniName($node->name) == $this->uniName($name);
}

function getInnerSafe($node, $name){
	foreach($node->nodes as $inner){
		if($this->isTag($inner, $name)){
			return $inner;
		}
	}
	throw new WrongXmlException($name.' expected');
}

function getInnerSafeValue($node, $name){
	$node = $this->getInnerSafe($node, $name);
	return $node->body;
}


function getInnerValue($node, $name, $def = ''){
	try{
		return $this->getInnerSafeValue($node, $name);
	}catch(WrongXmlException $ex){
		return $def;
	}
}

function mapNodes($nodes){
	$map = array();
	foreach($nodes as $n){
		$map[$this->uniName($n->name)] = $n;
	}
	return $map;
}

function mapValues($nodes){
	$map = array();
	foreach($nodes as $n){
		$map[$this->uniName($n->name)] = $n->body;
	}
	return $map;
}

function handle($xml){
	global $beans;
	if(!$this->account){
		$this->logger->warning('unconfigured (account: PAY_PRIVAT24)');
		return;
	}
	try{
		$this->logger->debug("request: ".$this->decode($xml));
		$parser = new StringXmlParser();
		$parser->setEncoding("UTF-8");
		$parser->setOptions(array(
			XML_OPTION_CASE_FOLDING => 0,
			XML_OPTION_SKIP_WHITE => 1,
		));
		$parser->setLogFactory($beans->getBean('loggerFactory'));
		$delegate = new XmlNodeDelegate();
		if(!$parser->parse($delegate, $xml)){
			throw new WrongXmlException('cant parse xml');
		}
		$operation = $delegate->node->nodes[0];
		switch($this->uniName($operation->name)){
			case 'requests': $this->handleRequests($operation); break;
			case 'bills': $this->handleBills($operation); break;
			case 'pay_orders': $this->handleOrders($operation); break;
			default:
				throw new WrongXmlException('operation not supported: '.$operation->name);
		}
	}catch(WrongXmlException $ex){
		$this->logger->error('wrong xml: '. $ex->getMessage());
		$this->error(11);
	}
}

function handleRequests($operation){
	global $db;
	$this->logger->info('REQUESTS');
	$payee = $this->getInnerSafeValue($operation, 'payee');
	$bills = array();
	$rejects = array();
	$date = time();
	$period = date("my", $date + 60 * 60 * 24 * 15);
	$err_code = 1;
	$err_msg = $this->wrap("  ");
	foreach($operation->nodes as $node){
		if($this->uniName($node->name)=='payer'){
			$payer_node = $node;
			$contract = $this->getInnerSafeValue($payer_node, 'contract_number');
			$this->logger->info('bill request: sid=%s, acc=%s', $payee, $contract);
			$client = loadRecord(sprintf("SELECT id, account, block, first_name, last_name, connection_type FROM clients WHERE contract='%s'", addslashes($contract)));
			if($client){
				$amount = loadValue(sprintf("SELECT threshold + incomplete FROM clients_states WHERE client='%s'", $client['id']), 0) + 
					$client['block'] +
					loadValue(sprintf("SELECT sum(use_plan) as plan FROM connection_type_resources WHERE connection_type='%s'", $client['connection_type']), 0) -
					loadValue(sprintf("SELECT state_end FROM calendar_account_states WHERE account='%s' AND calendar_period=(SELECT id FROM calendar_periods WHERE marker=1)", $client['account']), 0);
				if($amount < 0){
					$amount = 0;
				}
				$db->Execute(sprintf("INSERT INTO portmone_bills (client, contract, payee, period, date, amount) VALUES ('%s', '%s', '%s', '%s', '%s', '%s')",
					$client['id'], addslashes($contract), addslashes($payee), $period, date("Y-m-d H:i:s", $date), $amount));
				$bid = $db->getJustInserted('portmone_bills', 'id');
				$this->logger->info('bill created: '.$bid);
				$db->Execute(sprintf("UPDATE portmone_bills SET code=id WHERE id='%s'", $bid));
				$bills[] = "<BILL><PAYER><CONTRACT_NUMBER>$contract</CONTRACT_NUMBER></PAYER>".
					"<BILL_DATE>".date("Y-m-d", $date)."</BILL_DATE><BILL_NUMBER>$bid</BILL_NUMBER>".
					"<AMOUNT>".number_format($amount/100, 2, '.', '')."</AMOUNT><DEBT>0</DEBT><REMARK></REMARK></BILL>";
			}else{
				$this->logger->warning('acc not found: '.$contract);
				$rejects[] = "<REJECT><PAYER><CONTRACT_NUMBER>$contract</CONTRACT_NUMBER></PAYER>".
					"<ERROR_CODE>$err_code</ERROR_CODE><ERROR_MESSAGE>$err_msg</ERROR_MESSAGE></REJECT>";
			}
		}
	}
	$result = '';
	if(!empty($bills)){
		$result .= "<BILLS><PAYEE>$payee</PAYEE><BILL_PERIOD>$period</BILL_PERIOD>".implode("", $bills)."</BILLS>";
	}
	if(!empty($rejects)){
		$result .= "<REJECTS><PAYEE>$payee</PAYEE>".implode("", $rejects)."</REJECTS>";
	}
	$this->response("<RESPONSE>".$result."</RESPONSE>");
}

function handleBills($operation){
	global $db;
	$this->logger->info('BILLS');
	$bill_node = $this->getInnerSafe($operation, 'bill');
	$bill_id = $this->getInnerSafeValue($bill_node, 'bill_id');
	$bill_number = $this->getInnerSafeValue($bill_node, 'bill_number');
	$bill_period = $this->getInnerSafeValue($bill_node, 'bill_period');
	$bill_date = $this->getInnerSafeValue($bill_node, 'bill_date');
	$pay_date = $this->getInnerSafeValue($bill_node, 'pay_date');
	$payed_amount = $this->getInnerSafeValue($bill_node, 'payed_amount');
	$payed_commission = $this->getInnerSafeValue($bill_node, 'payed_commission');
	$amount = intval(strval($payed_amount * 100));
	$comis =  intval(strval($payed_commission * 100));
	$auth_code = $this->getInnerValue($bill_node, 'auth_code');
	$payer_node = $this->getInnerSafe($bill_node, 'payer');
	$contract = $this->getInnerSafeValue($payer_node, 'contract_number');
	$bank_node = $this->getInnerSafe($bill_node, 'bank');
	$bank_name = $this->decode($this->getInnerSafeValue($bank_node, 'name'));
	$bank_code = $this->getInnerSafeValue($bank_node, 'code');
	$bank_account = $this->getInnerSafeValue($bank_node, 'account');
	$payee_node = $this->getInnerSafe($bill_node, 'payee');
	$payee_name = $this->decode($this->getInnerSafeValue($payee_node, 'name'));
	$payee_code = $this->getInnerSafeValue($payee_node, 'code');
	$this->logger->info('pay request for bill [%s], contract [%s]', $bill_number, $contract);
	$db->Begin();
	try{
		$client = loadRecord(sprintf("SELECT id, account, block, first_name, last_name, connection_type FROM clients WHERE contract='%s'", addslashes($contract)));
		if(!$client){
			$this->logger->error('contract not found: [%s]', $contract);
			$this->result(1);
			throw new Exception();
		}
		$bill = loadRecord(sprintf("SELECT id FROM portmone_bills WHERE code='%s'", addslashes($bill_number)));
		if($bill){
			$bid = $bill['id'];
		}else{ //   
			$db->Execute(sprintf("INSERT INTO portmone_bills (code, client, contract, payee, period, date, amount) VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s')",
				addslashes($bill_number), $client['id'], addslashes($contract), addslashes($payee_code), addslashes($bill_period), addslashes($bill_date), $amount));
			$bid = $db->getJustInserted('portmone_bills', 'id');
			$this->logger->info('free bill created: [%s]', $bid);
		}
		$pay = loadRecord(sprintf("SELECT * FROM portmone_bills_pays WHERE portmone_id='%s'", addslashes($bill_id)));
		if($pay){
			$this->logger->error('pay already exists: [%s]', $bill_id);
			$this->result(3);
			throw new Exception();
		}
		$mid = move($this->account, $client['account'], $amount);
		$db->Execute(sprintf("INSERT INTO portmone_bills_pays (bill, portmone_id, date, amount, commission, auth_code, movement, bank_name, bank_code, bank_account) 
			VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s')", 
			$bid, addslashes($bill_id), addslashes($pay_date), $amount, $comis, addslashes($auth_code), $mid,
			addslashes($bank_name), addslashes($bank_code), addslashes($bank_account)));
		$pid = $db->getJustInserted('portmone_bills_pays', 'id');
		$this->logger->info('pay created: [%s]', $pid);
		$success = $db->End();
		if($success){
			$this->result(0);
		}else{
			$this->logger->warning('db error: '.implode(",", $db->errors));
			$this->error(10);
		}
	}catch(Exception $ex){
		$db->Rollback();
	}
}

function handleOrders($operation){
	global $db;
	$this->logger->info('ORDERS');
	$order_node = $this->getInnerSafe($operation, 'pay_order');
	$order_id = $this->getInnerSafeValue($order_node, 'pay_order_id');
	$order_number = $this->getInnerSafeValue($order_node, 'pay_order_number');
	$order_date = $this->getInnerSafeValue($order_node, 'pay_order_date');
	$order_amount = $this->getInnerSafeValue($order_node, 'pay_order_amount');
	$payee_node = $this->getInnerSafe($order_node, 'payee');
	$payee_name = $this->decode($this->getInnerSafeValue($payee_node, 'name'));
	$payee_code = $this->getInnerSafeValue($payee_node, 'code');
	$bank_node = $this->getInnerSafe($order_node, 'bank');
	$bank_name = $this->decode($this->getInnerSafeValue($bank_node, 'name'));
	$bank_code = $this->getInnerSafeValue($bank_node, 'code');
	$bank_account = $this->getInnerSafeValue($bank_node, 'account');
	$db->Begin();
	try{
		$order = loadRecord(sprintf("SELECT id FROM portmone_bills_orders WHERE order_id='%s'", addslashes($order_id)));
		if($order){
			$this->logger->warning('bank pay already exists: '.$bill_id);
			$this->result(4);
			throw new Exception();
		}
		$db->Execute(sprintf("INSERT INTO portmone_bills_orders (order_id, order_number, date, amount, payee_name, payee_code, bank_name, bank_code, bank_account, created)
			VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', now())",
			addslashes($order_id), addslashes($order_number), addslashes($order_date), intval(strval($order_amount * 100)), addslashes($payee_name), addslashes($payee_code),
			addslashes($bank_name), addslashes($bank_code), addslashes($bank_account)));
		$oid = $db->getJustInserted('portmone_bills_orders', 'id');
		$this->logger->info('order created: [%s], order_id: [%s]', $oid, $order_id);
		$bills_node = $this->getInnerSafe($order_node, 'bills');
		foreach($bills_node->nodes as $bill_node){
			$bill_id = $this->getInnerSafeValue($bill_node, 'bill_id');
			$auth_code = $this->getInnerValue($bill_node, 'auth_code');
			$pay_date = $this->getInnerSafeValue($bill_node, 'pay_date');
			$payed_amount = $this->getInnerSafeValue($bill_node, 'payed_amount');
			$payed_commission = $this->getInnerSafeValue($bill_node, 'payed_commission');
			$payer_node = $this->getInnerSafe($bill_node, 'payer');
			$contract = $this->getInnerSafeValue($payer_node, 'contract_number');
			$pay = loadRecord(sprintf("SELECT id FROM portmone_bills_pays WHERE portmone_id='%s'", addslashes($bill_id)));
			if($pay){
				$db->Execute(sprintf("INSERT INTO portmone_bills_orders_details (portmone_order, pay, amount, commission, portmone_id, auth_code, contract)
					VALUES (%s, %s, '%s', '%s', '%s', '%s', '%s')",
					$oid, $pay['id'], intval(strval($payed_amount * 100)), intval(strval($payed_commission * 100)), addslashes($bill_id), addslashes($auth_code), addslashes($contract)));
				$did = $db->getJustInserted('portmone_bills_orders_details', 'id');
				$this->logger->info('order detail created: [%s], bill_id: [%s], contract: [%s]', $did, $bill_id, $contract);
			}else{
				$this->logger->error('pay not found: [%s]', $bill_id);
				$this->result(5);
				throw new Exception();
			}
		}
		$success = $db->End();
		if($success){
			$this->logger->info('order saved: '.$oid);
			$this->result(0);
		}else{
			$this->logger->warning('db error: '.implode(", ", $db->errors));
			$this->error(10);
		}
	}catch(Exception $ex){
		$db->Rollback();
	}
}

function wrap($str){
	return iconv('windows-1251', 'utf-8', htmlspecialchars($str));
}

function decode($str){
	return iconv('utf-8', 'windows-1251', $str);
}

function error($code, $message = ''){
	if(!$message){
		$message = $this->wrap($this->results[$code]);
	}
	$this->response("<SYSTEM_ERROR><ERROR_CODE>$code</ERROR_CODE><REASON>$message</REASON></SYSTEM_ERROR>");
}

function result($code, $message = ''){
	if(!$message){
		$message = $this->wrap($this->results[$code]);
	}
	$this->response("<RESULT><ERROR_CODE>$code</ERROR_CODE><REASON>$message</REASON></RESULT>");
}

function response($xml){
	$this->logger->debug("response: ".$this->decode($xml));
	$xml = '<?xml version="1.0" encoding="UTF-8"?>'.$xml;
	header('Content-Type: text/xml; charset=utf-8');
	header('Content-Length: '.strlen($xml));
	echo $xml;
}

}

if($access_zone = getTrimOption('PORTMONE_ACCESS_ZONE')){
	$valid = false;
	$ip = new SlashedIpAddress($_SERVER["REMOTE_ADDR"]);
	foreach(preg_split("|[\s,;]+|", $access_zone, -1, PREG_SPLIT_NO_EMPTY) as $ip_str){
		$zone = new SlashedIpAddress($ip_str);
		if($zone->contains($ip)){
			$valid = true;
			break;
		}
	}
	if(!$valid){
		exit;
	}
}

$appender = new FileAppender('/var/log/portmone.log');
$logger = new DefaultLogger();
$logger->setAppender($appender);
$logger->setName('portmone');
//$logger->setLevel(2);
$handler = new PortmoneHandler($logger);

$logger->info('request from: '.$_SERVER['REMOTE_ADDR']);
$xml = $_POST['data'];
$handler->handle($xml);
?>
