function ClastererMapController(map, markerSource){
	var clusterer = new MarkerClusterer(map, [], {gridSize:25, zoomOnClick:false});
	var wrapper = this;
	this.markers = [];
	this.visibility = 0;
	
	this.setVisibility = function(v){
		v = v ? 1 : 0;
		if(v != this.visibility){
			this.visibility = v;
			$.cookie("map_all_houses", v);
			if(this.visibility){
				clusterer.addMarkers(this.markers);
			}else{
				clusterer.clearMarkers();
			}
		}
	}
		
	this.setVisibility($.cookie("map_all_houses")=='1' ? 1 : 0);
	
	this.addMarkers = function(arr){
		$.each(arr, function(i, v){
			wrapper.markers.push(v);
		})
		if(this.visibility){
			clusterer.addMarkers(arr);
		}
	}
	
	google.maps.event.addListener(map, 'bounds_changed', function(){
		var bounds = map.getBounds();
		markerSource.bounds_changed(bounds, function(newMarkers){
			wrapper.addMarkers(newMarkers);
		})
	});
	
	var control = document.createElement("div");
	control.style.margin = "4px";
	control.style.padding = "0px";
	control.style.backgroundColor = 'white';
	control.style.borderStyle = 'solid';
	control.style.borderWidth = '1px';
	control.style.cursor = 'pointer';
	control.style.textAlign = 'center';

	var flag = document.createElement("input");
	flag.style.verticalAlign = "middle";
//	flag.style.margin = "2px";
	flag.type = "checkbox";
	flag.margin = '0px';
	flag.id = "showAll";
	flag.checked = this.visibility;
	control.appendChild(flag);
	var lab = document.createElement("label");
	lab.htmlFor = "showAll";
	lab.style.fontFamily = 'Arial,sans-serif';
	lab.style.fontSize = '12px';

	lab.style.paddingLeft = '4px';
	lab.style.paddingRight = '4px';
	lab.innerHTML = " ";
	control.appendChild(lab);
	google.maps.event.addDomListener(flag, 'click', function() {
		wrapper.setVisibility(flag.checked);
	});
	map.controls[google.maps.ControlPosition.TOP_RIGHT].push(control);
}

function ModesClastererMapController(map, markerSource, modes, modeIndex, cookie){
	var clusterer = new MarkerClusterer(map, [], {gridSize:25, zoomOnClick:false});
	var wrapper = this;
	this.markers = [];
	this.mode = 0;
	this.matcher = modes[0].matcher;
	
	var sel = document.createElement("select");
	sel.style.margin = '6px';
	sel.style.borderWidth = '2px';
	var options = [];
	$.each(modes, function(i, value){
		var o = new Option(value.title, i);
		sel.options.add(o);
	});
	
	$(sel).change(function(ev){
		wrapper.setMode(this.value);
	})
	map.controls[google.maps.ControlPosition.TOP_RIGHT].push(sel);
	
	this.setMode = function(v){
		if(v != this.mode){
			wrapper.mode = v;
			wrapper.matcher = modes[v].matcher;
			sel.value = v;
			$.cookie(cookie, v);
			clusterer.clearMarkers();
			if(this.mode > 0){
				var matched = [];
				$.each(this.markers, function(i, m){
					if(wrapper.match(m)){
						matched.push(m);
					}
				});
				clusterer.addMarkers(matched);
			}
		}
	}
		
	this.match = function(marker){
		return marker.info[modeIndex].match(wrapper.matcher);
	}
	
	this.setMode($.cookie(cookie));
	
	this.addMarkers = function(arr){
		var matched = [];
		$.each(arr, function(i, v){
			wrapper.markers.push(v);
			if(wrapper.match(v)){
				matched.push(v);
			}
		})
		clusterer.addMarkers(matched);
	}
	
	google.maps.event.addListener(map, 'bounds_changed', function(){
		var bounds = map.getBounds();
		markerSource.bounds_changed(bounds, function(newMarkers){
			wrapper.addMarkers(newMarkers);
		})
	});
	
	
}


function AjaxMarkerSource(addr, data, onclick){
	var registry = {};
	var image = new google.maps.MarkerImage(
	  'images/marker/plus/image.png',
	  new google.maps.Size(16,16),
	  new google.maps.Point(0,0),
	  new google.maps.Point(8,16)
	);
	
	var shadow = new google.maps.MarkerImage(
	  'images/marker/plus/shadow.png',
	  new google.maps.Size(28,16),
	  new google.maps.Point(0,0),
	  new google.maps.Point(8,16)
	);

	this.bounds_changed = function(bounds, callback){
		var newMarkers = [];
		var ne = bounds.getNorthEast();
		var sw = bounds.getSouthWest();
		var b = {
			ne:{lat: ne.lat(), lng:ne.lng()},
			sw:{lat: sw.lat(), lng:sw.lng()},
			data: data
		};
		$.post(addr, JSON.stringify(b), function(response){
			$.each(response.split("\n"), function(index, line){
				var info = line.split("~");
				if(info.length < 4) return;
				if(!registry[info[0]]){
					registry[info[0]] = 1;
					var pos = new google.maps.LatLng(0.000090 + parseFloat(info[1]), parseFloat(info[2]));
					var marker = new google.maps.Marker({
						position: pos,
						icon: image,
						shadow: shadow,
						title: info[3]
					});
					marker.info = info;
					if(onclick){
						google.maps.event.addListener(marker, 'click', function(){
							onclick(this.info[0]);
						});
					}
					newMarkers.push(marker);
				}
			});
			callback(newMarkers);
		}, "text");
	}
}

function limitMapZoom(map, zoom){
	google.maps.event.addListener(map, 'zoom_changed', function(){
		if (map.getZoom() > zoom) {
			map.setZoom(zoom);
		};
	});
}
