#!/usr/bin/php -q
<?php
set_time_limit (0);
ini_set('include_path', DIRNAME(__FILE__). '/../');
include_once "classes/log/StdoutAppender.php";
include_once "classes/log/FileAppender.php";
include_once "classes/log/StreamAppender.php";
include_once "classes/log/DefaultLogger.php";

$opt = getopt('fhb:i:l:v:n:o:c:z:');

function unslash($str){
	return str_replace(array('\n', '\r', '\t', '\0'), array("\n", "\r", "\t", "\0"), $str);
}

if(isset($opt['h'])){
?>

-b <dir>       make backup to <dir>
-i <dir>       used with -b to specify base dir for incremental backup
-f             cleanup target dir before backup
-o <options>   options to pass to innobackupex script
-c <dir>       compress (tar.gz) backups dirs and remove any other dirs inside <dir>
-z <level>     compress level, default 5 (for tar.gz)
-l <stream>    file descriptor for log, if not specified then log to file /var/log/backup
-v <level>     logging level 1-5 (DEBUG - NONE)
-n <name>      name in log, default is 'backup'
-h             this help

<?
exit;
}

$log_to = isset($opt['l']) ? $opt['l'] : 2;
$log_level = isset($opt['v']) ? intval($opt['v']) : 2;
$log_name = isset($opt['n']) ? $opt['n'] : 'backup';
$compress_level = isset($opt['z']) ? intval($opt['z']) : 5;
$date_format = isset($opt['d']) ? $opt['d'] : 'Y/m/d';

$logger = new DefaultLogger();
$logger->setName($log_name);
$logger->setLevel($log_level);
if(!isset($opt['l'])){	
	$logger->setAppender(new FileAppender('/var/log/backup'));
}else{
	if($log_stream = fopen('/proc/self/fd/'.$log_to, "a+")){
		$logger->setAppender(new StreamAppender($log_stream));
	}else{
		$logger->setAppender(new StdoutAppender());
		$logger->error('cant open %s for logging', $log_to);
	}
}

class StreamLogger {

var $terminator;
var $buffer;
var $stream;

function StreamLogger($stream, $logger, $terminator = "\n"){
	$this->logger = $logger;
	$this->stream = $stream;
	$this->terminator = $terminator;
}

function log(){
	$buffer = fread($this->stream, 8192);
	$this->buffer .= $buffer;
	$prev = 0;
	while(true){
		$pos = strpos($this->buffer, $this->terminator, $prev);
		if($pos===false){
			break;
		}
		$this->logger->info(substr($this->buffer, $prev, $pos - $prev));
		$prev = $pos + strlen($this->terminator);
	}
	$this->buffer = substr($this->buffer, $prev);
}

}

function my_exec($cmd){
	$descriptorspec = array(
		0 => array("pipe", "r"), // stdin
		1 => array("pipe", "w"), // stdout
		2 => array("pipe", "w"), // stderr
	);
	if($process = proc_open($cmd, $descriptorspec, $pipes)){
		$loggers = array(
			new StreamLogger($pipes[1], $GLOBALS['logger']),
			new StreamLogger($pipes[2], $GLOBALS['logger']),
		);
		while(!feof($pipes[1]) || !feof($pipes[2])){
			$read = array($pipes[1], $pipes[2]);
			$ready = stream_select($read, $write=null, $except=null, $timeout = null);
			foreach($loggers as $logger){
				if(in_array($logger->stream, $read)){
					$logger->log();
				}
			}
		}
		fclose($pipes[1]);
		fclose($pipes[2]);
		return proc_close($process);
	}else{
		$logger->error('cant execute %s', $cmd);
		return -1;
	}
}

function backup($target_dir, $incremental_base=''){
	global $logger, $opt;
}

if(isset($opt['b'])){ // backup
	if(is_dir($opt['b'])){
		if(isset($opt['f'])){
			$logger->info('cleanup %s', $opt['b']);
			my_exec('rm -r '.$opt['b'].'/*');
		}
	}else{
		$logger->info('creating backup dir [%s]', $opt['b']);
		if(!mkdir($opt['b'], 0777, true)){
			$logger->error('cant create backup dir [%s]', $opt['b']);
			exit;
		}
	}
	$logger->info('%s backup to %s', $opt['i'] ? 'incremental' : 'base', $opt['b']);
	$cmd = 'innobackupex-1.5.1 '.(isset($opt['o']) ? $opt['o'] : '' ).($opt['i'] ? ' --incremental --incremental-basedir='.$opt['i'] : '').' --no-timestamp '.$opt['b'];
	$logger->info($cmd);
	my_exec($cmd);
}

if(isset($opt['c'])){ // compress && cleanup
	$logger->info('cleanup %s', $opt['c']);
	if(is_dir($opt['c'])){
		$d = dir($opt['c']);
		while ($entry = $d->read()){
			if($entry=='.' || $entry=='..'){
				continue;
			}
			$entry = $opt['c'].'/'.basename($entry);
			$checkpoints = $entry.'/xtrabackup_checkpoints';
			if(is_dir($entry) && is_file($checkpoints)){ // && strstr(file_get_contents($checkpoints),'full-backuped')
				$logger->info('compressing %s', $entry);
				$compressed = $entry.'.tar.gz';
				$cmd = 'tar -c '.$entry.' | pigz -f'.$compress_level.' > '.$compressed;
				$logger->info($cmd);
				my_exec($cmd);
				if(is_file($compressed)){
					$logger->info('removing %s', $entry);
					my_exec('rm -r '.$entry);
				}
			}else{
				$logger->info('removing %s', $entry);
				my_exec('rm -r '.$entry);
			}
		}
		$d->close();
	}else{
		$logger->info('dir not exists');
	}
}

$logger->info('quit');
?>