CREATE TABLE access_tracker (
	client INT UNSIGNED NOT NULL PRIMARY KEY,
	state TINYINT NOT NULL
) ENGINE=InnoDB;

CREATE TABLE access_tracker_queue (
	modem INT UNSIGNED NOT NULL PRIMARY KEY,
	added DATETIME NOT NULL
) ENGINE=InnoDB;

CREATE TABLE accounts (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	type char(1) NOT NULL,
	name VARCHAR(255) NOT NULL,
	KEY (name)
) ENGINE=InnoDB;

CREATE TABLE action_log (
	id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	operator INT UNSIGNED NOT NULL,
	type TINYINT UNSIGNED NOT NULL,
	object INT UNSIGNED,
	action TEXT NOT NULL,
	date DATETIME NOT NULL,
	KEY (operator),
	KEY (date),
	KEY (object)
) ENGINE=InnoDB;

CREATE TABLE action_log_objects (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) UNIQUE
) ENGINE=InnoDB;

CREATE TABLE ads (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	period_begin DATETIME,
	period_end DATETIME,
	number INT NOT NULL,
	body TEXT,
	query INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE,
	key(period_begin),
	key(period_end),
	key(query)
);

CREATE TABLE blocks (
	id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED NOT NULL,
	resource_type INT UNSIGNED,
	actual_from DATETIME,
	actual_to DATETIME,
	note VARCHAR(255),
	client_note varchar(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (client),
	KEY (resource_type),
	KEY (actual_from),
	KEY (actual_to)
) ENGINE=InnoDB;

CREATE TABLE bonus_accounts (
	account INT UNSIGNED NOT NULL PRIMARY KEY,
	client INT UNSIGNED NOT NULL,
	resource_type INT UNSIGNED,
	name VARCHAR(255) NOT NULL,
	priority TINYINT NOT NULL,
	actual_from DATETIME,
	actual_to DATETIME,
	lost_dst INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE KEY (client, name),
	KEY (client),
	KEY (resource_type),
	KEY (actual_from),
	KEY (actual_to),
	KEY (lost_dst)
) ENGINE=InnoDB;

CREATE TABLE browser_properties (
	browser INT UNSIGNED NOT NULL,
	property INT UNSIGNED NOT NULL,
	value VARCHAR(255) NOT NULL,
	PRIMARY KEY (browser, property),
	KEY (browser),
	KEY (property)
) ENGINE=InnoDB;

CREATE TABLE browser_property_names (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE
) ENGINE=InnoDB;

CREATE TABLE browsers (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	bid VARCHAR(255) NOT NULL UNIQUE,
	last_visit DATETIME NOT NULL,
	last_ip VARCHAR(64) NOT NULL,
	visit_counter BIGINT UNSIGNED NOT NULL
) ENGINE=InnoDB;

CREATE TABLE calendar_account_states (
	account INT UNSIGNED NOT NULL,
	calendar_period INT UNSIGNED NOT NULL,
	state_begin BIGINT NOT NULL,
	state_end BIGINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	PRIMARY KEY (account, calendar_period),
	KEY (calendar_period),
	KEY (account)
) ENGINE=InnoDB;

CREATE TABLE calendar_periods (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	period_begin DATETIME NOT NULL,
	period_end DATETIME NOT NULL,
	next INT UNSIGNED UNIQUE,
	state TINYINT NOT NULL,
	marker tinyint(4) UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (period_begin),
	KEY (period_end)
) ENGINE=InnoDB;

CREATE TABLE jobs (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	street INT UNSIGNED,
	house VARCHAR(64),
	entrance TINYINT,
	floor TINYINT,
	apartment INT,
	client VARCHAR(255) NOT NULL,
	phone VARCHAR(64),
	type TINYINT NOT NULL,
	alarm TINYINT NOT NULL,
	worker INT UNSIGNED,
	date DATETIME,
	notes VARCHAR(255),
	state TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE cards (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	code_public VARCHAR(255) UNIQUE,
	code_private VARCHAR(255) UNIQUE,
	amount BIGINT UNSIGNED NOT NULL,
	account INT UNSIGNED NOT NULL,
	expire DATETIME NOT NULL,
	movement BIGINT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (account),
	KEY (movement)
) ENGINE=InnoDB;

CREATE TABLE charge_subscriptions (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED NOT NULL,
	type INT UNSIGNED NOT NULL,
	amount BIGINT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (client),
	KEY (type)
) ENGINE=InnoDB;

CREATE TABLE charge_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	resource_type INT UNSIGNED NOT NULL,
	unit_size BIGINT UNSIGNED NOT NULL,
	price BIGINT UNSIGNED NOT NULL,
	account INT UNSIGNED NOT NULL,
	driver VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (resource_type),
	KEY (account)
) ENGINE=InnoDB;

CREATE TABLE client_mailboxes (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED NOT NULL,
	type INT UNSIGNED NOT NULL,
	login VARCHAR(250) NOT NULL,
	password VARCHAR(250) NOT NULL,
	active TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (client),
	KEY (type)
) ENGINE=InnoDB;

CREATE TABLE client_notes (
	id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED NOT NULL,
	type tinyint NOT NULL,
	active tinyint NOT NULL,
	note TEXT,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY client (client)
) ENGINE=InnoDB;

CREATE TABLE client_shaper_patches (
	id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED NOT NULL,
	from_time datetime,
	to_time datetime,
	resource_use INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE clients (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	account INT UNSIGNED NOT NULL UNIQUE,
	department INT UNSIGNED,
	connection_type INT UNSIGNED NOT NULL,
	first_name VARCHAR(255) NOT NULL,
	last_name VARCHAR(255) NOT NULL,
	patronymic VARCHAR(255) NOT NULL,
	street INT UNSIGNED NOT NULL,
	house VARCHAR(64) NOT NULL,
	building INT NOT NULL,
	letter char(1) NOT NULL,
	entrance INT NOT NULL,
	floor INT NOT NULL,
	apartment INT NOT NULL,
	surveyor VARCHAR(64) NOT NULL,
	phone VARCHAR(64) NOT NULL,
	mobile VARCHAR(64),
	login VARCHAR(64) NOT NULL UNIQUE,
	pass VARCHAR(64) NOT NULL,
	email VARCHAR(128) NOT NULL,
	bank_account VARCHAR(32),
	contract VARCHAR(32) NOT NULL,
	passport VARCHAR(32),
	passport_info VARCHAR(255),
	notice INT NOT NULL,
	alarm INT NOT NULL,
	block INT NOT NULL,
	reduction BIGINT UNSIGNED NOT NULL,
	blockable TINYINT NOT NULL,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (department),
	KEY (connection_type),
	KEY (street)
) ENGINE=InnoDB;

CREATE TABLE credits (
  id int unsigned NOT NULL auto_increment primary key,
  client int unsigned NOT NULL,
  amount bigint(20) unsigned NOT NULL,
  actual_from datetime,
  actual_to datetime,
  note varchar(255),
  stamp bigint(20) unsigned unique,
  KEY (client),
  KEY (actual_from),
  KEY (actual_to)
) ENGINE=InnoDB;

CREATE TABLE config (
	id VARCHAR(64) NOT NULL PRIMARY KEY,
	value TEXT NOT NULL,
	control TINYINT NOT NULL,
	actual TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE connection_type_resources (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	connection_type INT UNSIGNED NOT NULL,
	resource_type INT UNSIGNED NOT NULL,
	use_dst INT UNSIGNED NOT NULL,
	use_plan BIGINT UNSIGNED NOT NULL,
	use_limit BIGINT UNSIGNED NOT NULL,
	penalty_dst INT UNSIGNED,
	penalty_type TINYINT NOT NULL,
	bonus_src INT UNSIGNED,
	bonus_amount BIGINT UNSIGNED NOT NULL,
	bonus_name VARCHAR(255),
	bonus_priority TINYINT NOT NULL,
	bonus_lost_dst INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE (connection_type, resource_type),
	KEY (resource_type),
	KEY (use_dst),
	KEY (penalty_dst),
	KEY (bonus_src),
	KEY (bonus_lost_dst)
) ENGINE=InnoDB;

CREATE TABLE connection_type_rules (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	connection_type_resource INT UNSIGNED NOT NULL,
	rule INT UNSIGNED NOT NULL,
	tariff INT UNSIGNED NOT NULL,
	tariff_over INT UNSIGNED NOT NULL,
	blockable TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE (connection_type_resource, rule),
	KEY (rule),
	KEY (tariff),
	KEY (tariff_over)
) ENGINE=InnoDB;

CREATE TABLE connection_type_shapers (
	connection_type INT UNSIGNED NOT NULL,
	shaper INT UNSIGNED NOT NULL,
	use_case TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	PRIMARY KEY  (connection_type, shaper),
	KEY shaper (shaper)
) ENGINE=InnoDB;

CREATE TABLE connection_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	period_months SMALLINT NOT NULL,
	period_days SMALLINT NOT NULL,
	abon_dst INT UNSIGNED,
	abon_amount BIGINT UNSIGNED NOT NULL,
	abon_resource_type int unsigned,
	unshape_resource_type int unsigned,
	unshape_dst INT UNSIGNED,
	unshape_amount BIGINT UNSIGNED NOT NULL,
	allow_enter TINYINT NOT NULL,
	allow_leave TINYINT NOT NULL,
	max_hosts int not null,
	start_always TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY abon_dst (abon_dst),
	KEY abon_resource_type (abon_resource_type)
) ENGINE=InnoDB;

CREATE TABLE departments (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	description varchar(255) not null,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE districts (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE domains (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(64) NOT NULL UNIQUE,
	template TEXT NOT NULL,
	host_template TEXT NOT NULL,
	version BIGINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE homedir_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(250) NOT NULL UNIQUE,
	shell VARCHAR(250) NOT NULL,
	home VARCHAR(250) NOT NULL,
	gid BIGINT NOT NULL,
	base_uid INT NOT NULL,
	quota BIGINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE homedirs (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED NOT NULL,
	type INT UNSIGNED NOT NULL,
	uid INT NOT NULL,
	login VARCHAR(255) NOT NULL UNIQUE,
	password VARCHAR(255) NOT NULL,
	description VARCHAR(255) NOT NULL,
	active TINYINT NOT NULL default '1',
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE hosts (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED NOT NULL,
	ip VARCHAR(32) NOT NULL UNIQUE,
	mac VARCHAR(64) UNIQUE,
	name VARCHAR(64) UNIQUE,
	domain INT UNSIGNED,
	login VARCHAR(255) UNIQUE,
	password VARCHAR(255),
	modem INT UNSIGNED NOT NULL,
	port INT NOT NULL,
	vlan INT NOT NULL,
	notes VARCHAR(255) NOT NULL,
	countable TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (client),
	KEY (domain),
	KEY (modem)
) ENGINE=InnoDB;

CREATE TABLE hosts_firewall (
	host INT UNSIGNED NOT NULL,
	rule INT UNSIGNED NOT NULL,
	PRIMARY KEY  (host,rule),
	KEY (rule)
) ENGINE=InnoDB;

CREATE TABLE houses (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	street INT UNSIGNED,
	house VARCHAR(16) NOT NULL,
	district INT UNSIGNED,
	state TINYINT NOT NULL,
	queries INT NOT NULL,
	floors TINYINT,
	entrances TINYINT,
	appartments INT,
	served_by TINYINT NOT NULL,
	kpreo INT UNSIGNED,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	key(street),
	key(district),
	key(kpreo)
) ENGINE=InnoDB;

CREATE TABLE ibox_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED,
	terminal BIGINT UNSIGNED NOT NULL,
	receipt BIGINT UNSIGNED NOT NULL,
	amount BIGINT UNSIGNED NOT NULL,
	date datetime not null,
	movement BIGINT UNSIGNED,
	KEY movement (movement),
	UNIQUE(terminal, receipt)
) ENGINE=InnoDB;

CREATE TABLE ipcad_log (
	id BIGINT UNSIGNED NOT NULL auto_increment PRIMARY KEY,
	src INT UNSIGNED NOT NULL,
	dst INT UNSIGNED NOT NULL,
	port SMALLINT UNSIGNED NOT NULL,
	dir TINYINT UNSIGNED NOT NULL,
	proto TINYINT UNSIGNED NOT NULL,
	bytes INT UNSIGNED NOT NULL,
	date DATETIME NOT NULL,
	KEY src (src),
	KEY date (date)
) ENGINE=InnoDB;

CREATE TABLE ip_use_intervals (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	from_time DATETIME NOT NULL,
	to_time DATETIME NOT NULL,
	KEY from_time (from_time),
	KEY to_time (to_time)
) ENGINE=InnoDB;

CREATE TABLE ip_use_log (
	ip_use INT UNSIGNED NOT NULL,
	ip_use_interval INT UNSIGNED NOT NULL,
	bytes BIGINT UNSIGNED,
	PRIMARY KEY  (ip_use, ip_use_interval),
	KEY (ip_use_interval)
) ENGINE=InnoDB;

CREATE TABLE ip_uses (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	personal_period_resource_use INT UNSIGNED NOT NULL,
	host INT UNSIGNED,
	rule INT UNSIGNED,
	bytes BIGINT UNSIGNED NOT NULL,
	label VARCHAR(32) UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY personal_period_resource_use (personal_period_resource_use),
	KEY host (host),
	KEY rule (rule)
) ENGINE=InnoDB;

CREATE TABLE koops (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	street INT UNSIGNED,
	house VARCHAR(16) NOT NULL,
	contract VARCHAR(32),
	type TINYINT NOT NULL,
	commander VARCHAR(255),
	commander_phone VARCHAR(64),
	commander_address VARCHAR(255),
	accounter VARCHAR(255),
	accounter_phone VARCHAR(64),
	props VARCHAR(255),
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	key(street)
) ENGINE=InnoDB;

CREATE TABLE kpreos (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	number VARCHAR(64) NOT NULL UNIQUE,
	street INT UNSIGNED,
	house VARCHAR(16) NOT NULL,
	type TINYINT NOT NULL,
	dispatcher_phone VARCHAR(64),
	commander VARCHAR(255),
	commander_phone VARCHAR(64),
	chief_engineer VARCHAR(255),
	chief_engineer_phone VARCHAR(64),
	xman VARCHAR(255),
	xman_phone VARCHAR(64),
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	key(street)
) ENGINE=InnoDB;

CREATE TABLE mail_lists (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	file VARCHAR(255) NOT NULL UNIQUE,
	list_pattern TEXT NOT NULL,
	element_pattern TEXT NOT NULL,
	query INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (query)
) ENGINE=InnoDB;

CREATE TABLE mail_lists_clients (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	mail_list INT UNSIGNED NOT NULL,
	client INT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE (mail_list, client),
	KEY (client)
) ENGINE=InnoDB;

CREATE TABLE mailbox_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(250) NOT NULL UNIQUE,
	domain VARCHAR(250) NOT NULL,
	boxsize BIGINT UNSIGNED NOT NULL,
	mailsize BIGINT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE mailboxes (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	login VARCHAR(64) NOT NULL,
	pass VARCHAR(64) NOT NULL,
	domain VARCHAR(64) NOT NULL,
	type INT UNSIGNED NOT NULL,
	boxsize INT UNSIGNED NOT NULL,
	mailsize INT UNSIGNED NOT NULL,
	KEY (type)
) ENGINE=InnoDB;

CREATE TABLE modem_profiles (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	template TEXT,
	executor TEXT,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE modems (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	department INT UNSIGNED,
	name VARCHAR(32) UNIQUE ,
	domain INT UNSIGNED,
	ip VARCHAR(32) NOT NULL UNIQUE,
	mac VARCHAR(64) UNIQUE,
	profile INT UNSIGNED NOT NULL,
	modem INT UNSIGNED,
	port INT NOT NULL,
	uplink INT NOT NULL, 
	notes VARCHAR(255),
	street INT UNSIGNED,
	house VARCHAR(64) NOT NULL,
	entrance INT NOT NULL,
	floor INT NOT NULL,
	check_address tinyint not null default 1,
	address_space varchar(255),
	up_since DATETIME,
	model VARCHAR(16) NOT NULL,
	firmware VARCHAR(16) NOT NULL,
	state_id BIGINT,
	audit tinyint NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (department),
	KEY (domain),
	KEY (profile),
	KEY (street)
) ENGINE=InnoDB;

CREATE TABLE modems_states (
	id BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	modem INT UNSIGNED NOT NULL,
	state TINYINT NOT NULL,
	loss tinyint,
	date DATETIME NOT NULL,
	key (modem),
	key (date)
) ENGINE=InnoDB;

CREATE TABLE modems_actions (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(64) NOT NULL UNIQUE,
	action_pattern TEXT,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE movements (
	id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	calendar_period INT UNSIGNED NOT NULL,
	src INT UNSIGNED NOT NULL,
	dst INT UNSIGNED NOT NULL,
	amount BIGINT UNSIGNED NOT NULL,
	date DATETIME NOT NULL,
	note VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY calendar_period (calendar_period),
	KEY src (src),
	KEY dst (dst),
	KEY date (date)
) ENGINE=InnoDB;

CREATE TABLE networks (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	ip VARCHAR(64) NOT NULL UNIQUE,
	local TINYINT NOT NULL,
	filename VARCHAR(255) NOT NULL UNIQUE,
	template TEXT NOT NULL,
	host_template TEXT NOT NULL,
	version INT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE notified_personal_periods (
	client INT UNSIGNED NOT NULL PRIMARY KEY,
	personal_period INT UNSIGNED NOT NULL,
	KEY personal_period (personal_period)
) ENGINE=InnoDB;

CREATE TABLE operators (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	department INT UNSIGNED,
	name VARCHAR(64) NOT NULL,
	login VARCHAR(64) NOT NULL UNIQUE,
	pass VARCHAR(64) NOT NULL,
	role INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY department (department)
) ENGINE=InnoDB;

CREATE TABLE operators_access (
	operator INT UNSIGNED NOT NULL,
	entity INT UNSIGNED NOT NULL,
	access INT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	PRIMARY KEY  (operator, entity)
) ENGINE=InnoDB;

CREATE TABLE operators_logins (
	operator INT UNSIGNED NOT NULL,
	loged DATETIME NOT NULL,
	ip VARCHAR(32) NOT NULL,
	PRIMARY KEY  (operator, loged)
) ENGINE=InnoDB;

create table pays_import_head (
	id int not null primary key auto_increment,
	head text not null,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE pays_import (
	id bigint unsigned NOT NULL auto_increment,
	head INT NOT NULL,
	pos int unsigned NOT NULL,
	src int unsigned NOT NULL,
	dst int unsigned NOT NULL,
	amount bigint unsigned NOT NULL,
	info text,
	valid tinyint NOT NULL,
	note varchar(255),
	calendar_period int unsigned NOT NULL,
	date datetime NOT NULL,
	PRIMARY KEY (id),
	KEY pos (pos),
	KEY src (src),
	KEY dst (dst),
	KEY calendar_period (calendar_period)
) ENGINE=InnoDB;

CREATE TABLE personal_period_plans (
	personal_period INT UNSIGNED NOT NULL,
	resource_type INT UNSIGNED NOT NULL,
	use_plan BIGINT UNSIGNED NOT NULL,
	use_limit BIGINT UNSIGNED NOT NULL,
	use_actual BIGINT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	PRIMARY KEY (personal_period, resource_type),
	KEY resource_type (resource_type)
) ENGINE=InnoDB;

CREATE TABLE personal_periods (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	next INT UNSIGNED UNIQUE,
	client INT UNSIGNED NOT NULL,
	connection_type INT UNSIGNED NOT NULL,
	period_begin DATETIME NOT NULL,
	period_end DATETIME NOT NULL,
	state TINYINT NOT NULL,
	marker tinyint(4),
	note varchar(255),
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE (client, marker),
	KEY (connection_type),
	KEY (period_begin),
	KEY (period_end)
) ENGINE=InnoDB;

CREATE TABLE personal_period_resource_uses (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	personal_period INT UNSIGNED NOT NULL,
	resource_use INT UNSIGNED NOT NULL,
	amount BIGINT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE (personal_period, resource_use),
	KEY (resource_use)
) ENGINE=InnoDB;

CREATE TABLE port_services (
	port INT UNSIGNED NOT NULL PRIMARY KEY,
	service VARCHAR(64) NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE portmone_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED,
	code VARCHAR(64) NOT NULL UNIQUE,
	amount BIGINT UNSIGNED NOT NULL,
	date datetime not null,
	movement BIGINT UNSIGNED,
	KEY movement (movement)
) ENGINE=InnoDB;

CREATE TABLE protocols (
	id INT UNSIGNED NOT NULL PRIMARY KEY,
	name VARCHAR(32) NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE queries (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	body TEXT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE resource_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE resource_uses (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	calendar_period INT UNSIGNED NOT NULL,
	client INT UNSIGNED NOT NULL,
	resource_type INT UNSIGNED,
	unit_size BIGINT UNSIGNED NOT NULL,
	price BIGINT UNSIGNED NOT NULL,
	amount BIGINT UNSIGNED NOT NULL,
	src INT UNSIGNED,
	dst INT UNSIGNED,
	movement BIGINT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (calendar_period),
	KEY (client),
	KEY (src),
	KEY (dst),
	KEY (resource_type),
	KEY (movement)
) ENGINE=InnoDB;

CREATE TABLE rules (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	chain VARCHAR(255) NOT NULL UNIQUE,
	for_client VARCHAR(255) NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE self_limitations (
	client INT UNSIGNED NOT NULL,
	resource_type INT UNSIGNED NOT NULL,
	amount BIGINT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	PRIMARY KEY (client, resource_type),
	KEY (resource_type)
) ENGINE=InnoDB;

CREATE TABLE shapers (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	pattern TEXT,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE staff (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	duties VARCHAR(255) NOT NULL,
	phone VARCHAR(64) NOT NULL,
	mobile VARCHAR(64) NOT NULL,
	notes TEXT,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE stamps (
	id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	created DATETIME,
	created_by INT UNSIGNED,
	modified DATETIME,
	modified_by INT UNSIGNED,
	KEY created (created),
	KEY created_by (created_by),
	KEY modified (modified),
	KEY modified_by (modified_by)
) ENGINE=InnoDB;

CREATE TABLE streets (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE system_accounts (
	account INT UNSIGNED NOT NULL PRIMARY KEY,
	code VARCHAR(255) UNIQUE,
	name VARCHAR(255) NOT NULL UNIQUE,
	type TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE system_entities (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	key_name VARCHAR(32) NOT NULL UNIQUE,
	name VARCHAR(64) NOT NULL
) ENGINE=InnoDB;

CREATE TABLE tariffs (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	description TEXT NOT NULL,
	unit_size BIGINT UNSIGNED NOT NULL,
	function TEXT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE mac_vendors (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	code VARCHAR(32) NOT NULL UNIQUE,
	name VARCHAR(32) NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE ukr_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	movement BIGINT UNSIGNED,
	id_ups VARCHAR(64) NOT NULL UNIQUE,
	date datetime not null,
	KEY movement (movement)
) ENGINE=InnoDB;

CREATE TABLE wm_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	movement BIGINT UNSIGNED,
	wmid BIGINT NOT NULL,
	wmtid BIGINT NOT NULL UNIQUE,
	source VARCHAR(32),
	destination VARCHAR(32),
	KEY movement (movement)
) ENGINE=InnoDB;

CREATE TABLE workers (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

ALTER TABLE access_tracker ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE access_tracker_queue ADD FOREIGN KEY (modem) REFERENCES modems (id) ON DELETE CASCADE;
ALTER TABLE action_log ADD FOREIGN KEY (operator) REFERENCES operators (id) ON DELETE CASCADE;
ALTER TABLE action_log ADD FOREIGN KEY (object) REFERENCES action_log_objects (id) ON DELETE CASCADE;
ALTER TABLE ads ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE ads ADD FOREIGN KEY (query) REFERENCES queries (id) ON DELETE SET NULL;
ALTER TABLE blocks ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE blocks ADD FOREIGN KEY (resource_type) REFERENCES resource_types (id) ON DELETE CASCADE;
ALTER TABLE blocks ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE bonus_accounts ADD FOREIGN KEY (account) REFERENCES accounts (id);
ALTER TABLE bonus_accounts ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE bonus_accounts ADD FOREIGN KEY (resource_type) REFERENCES resource_types (id);
ALTER TABLE bonus_accounts ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE bonus_accounts ADD FOREIGN KEY (lost_dst) REFERENCES accounts (id);
ALTER TABLE browser_properties ADD FOREIGN KEY (browser) REFERENCES browsers (id) ON DELETE CASCADE;
ALTER TABLE browser_properties ADD FOREIGN KEY (property) REFERENCES browser_property_names (id) ON DELETE CASCADE;
ALTER TABLE calendar_account_states ADD FOREIGN KEY (account) REFERENCES accounts (id) ON DELETE CASCADE;
ALTER TABLE calendar_account_states ADD FOREIGN KEY (calendar_period) REFERENCES calendar_periods (id) ON DELETE CASCADE;
ALTER TABLE calendar_account_states ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE calendar_periods ADD FOREIGN KEY (next) REFERENCES calendar_periods (id);
ALTER TABLE calendar_periods ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE cards ADD FOREIGN KEY (account) REFERENCES accounts (id);
ALTER TABLE cards ADD FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL;
ALTER TABLE cards ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE charge_subscriptions ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE charge_subscriptions ADD FOREIGN KEY (type) REFERENCES charge_types (id);
ALTER TABLE charge_subscriptions ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE charge_types ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE client_mailboxes ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE client_mailboxes ADD FOREIGN KEY (type) REFERENCES mailbox_types (id);
ALTER TABLE client_mailboxes ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE client_notes ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE client_notes ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE client_shaper_patches ADD FOREIGN KEY (resource_use) REFERENCES resource_uses (id) ON DELETE SET NULL;
ALTER TABLE clients ADD FOREIGN KEY (account) REFERENCES accounts (id);
ALTER TABLE clients ADD FOREIGN KEY (department) REFERENCES departments (id);
ALTER TABLE clients ADD FOREIGN KEY (connection_type) REFERENCES connection_types (id);
ALTER TABLE clients ADD FOREIGN KEY (street) REFERENCES streets (id);
ALTER TABLE clients ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE credits ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE credits ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE config ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE connection_type_resources ADD FOREIGN KEY (connection_type) REFERENCES connection_types (id) ON DELETE CASCADE;
ALTER TABLE connection_type_resources ADD FOREIGN KEY (resource_type) REFERENCES resource_types (id);
ALTER TABLE connection_type_resources ADD FOREIGN KEY (use_dst) REFERENCES accounts (id);
ALTER TABLE connection_type_resources ADD FOREIGN KEY (penalty_dst) REFERENCES accounts (id);
ALTER TABLE connection_type_resources ADD FOREIGN KEY (bonus_src) REFERENCES accounts (id);
ALTER TABLE connection_type_resources ADD FOREIGN KEY (bonus_lost_dst) REFERENCES accounts (id);
ALTER TABLE connection_type_resources ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE connection_type_rules ADD FOREIGN KEY (connection_type_resource) REFERENCES connection_type_resources (id) ON DELETE CASCADE;
ALTER TABLE connection_type_rules ADD FOREIGN KEY (rule) REFERENCES rules (id) ON DELETE CASCADE;
ALTER TABLE connection_type_rules ADD FOREIGN KEY (tariff) REFERENCES tariffs (id);
ALTER TABLE connection_type_rules ADD FOREIGN KEY (tariff_over) REFERENCES tariffs (id);
ALTER TABLE connection_type_rules ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE connection_type_shapers ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE connection_type_shapers ADD FOREIGN KEY (connection_type) REFERENCES connection_types (id) ON DELETE CASCADE;
ALTER TABLE connection_type_shapers ADD FOREIGN KEY (shaper) REFERENCES shapers (id) ON DELETE CASCADE;
ALTER TABLE connection_types ADD FOREIGN KEY (abon_dst) REFERENCES accounts (id);
alter table connection_types ADD FOREIGN KEY (abon_resource_type) REFERENCES resource_types(id);
ALTER TABLE connection_types ADD FOREIGN KEY (unshape_resource_type) REFERENCES resource_types(id);
ALTER TABLE connection_types ADD FOREIGN KEY (unshape_dst) REFERENCES accounts(id);
ALTER TABLE connection_types ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE departments ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE districts ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE domains ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE homedir_types ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE homedirs ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE homedirs ADD FOREIGN KEY (type) REFERENCES homedir_types (id);
ALTER TABLE homedirs ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE hosts ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE hosts ADD FOREIGN KEY (domain) REFERENCES domains (id);
ALTER TABLE hosts ADD FOREIGN KEY (modem) REFERENCES modems (id);
ALTER TABLE hosts ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE hosts_firewall ADD FOREIGN KEY (host) REFERENCES hosts (id) ON DELETE CASCADE;
ALTER TABLE hosts_firewall ADD FOREIGN KEY (rule) REFERENCES rules (id) ON DELETE CASCADE;
ALTER TABLE houses ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE houses ADD FOREIGN KEY (street) REFERENCES streets (id);
ALTER TABLE houses ADD FOREIGN KEY (district) REFERENCES districts (id);
ALTER TABLE houses ADD FOREIGN KEY (kpreo) REFERENCES kpreos (id) ON DELETE SET NULL;
ALTER TABLE ibox_pays ADD FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL;
ALTER TABLE ibox_pays ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL;
ALTER TABLE ip_use_log ADD FOREIGN KEY (ip_use) REFERENCES ip_uses (id) ON DELETE CASCADE;
ALTER TABLE ip_use_log ADD FOREIGN KEY (ip_use_interval) REFERENCES ip_use_intervals (id);
ALTER TABLE ip_uses ADD FOREIGN KEY (personal_period_resource_use) REFERENCES personal_period_resource_uses (id) ON DELETE CASCADE;
ALTER TABLE ip_uses ADD FOREIGN KEY (host) REFERENCES hosts (id) ON DELETE SET NULL;
ALTER TABLE ip_uses ADD FOREIGN KEY (rule) REFERENCES rules (id) ON DELETE SET NULL;
ALTER TABLE ip_uses ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE jobs ADD FOREIGN KEY (street) REFERENCES streets (id);
ALTER TABLE jobs ADD FOREIGN KEY (worker) REFERENCES workers (id);
ALTER TABLE jobs ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE koops ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE koops ADD FOREIGN KEY (street) REFERENCES streets (id);
ALTER TABLE kpreos ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE kpreos ADD FOREIGN KEY (street) REFERENCES streets (id);
ALTER TABLE mail_lists ADD FOREIGN KEY (query) REFERENCES queries (id);
ALTER TABLE mail_lists ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE mail_lists_clients ADD FOREIGN KEY (mail_list) REFERENCES mail_lists (id) ON DELETE CASCADE;
ALTER TABLE mail_lists_clients ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE mail_lists_clients ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE mailbox_types ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE mailboxes ADD FOREIGN KEY (type) REFERENCES mailbox_types (id);
ALTER TABLE modem_profiles ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE modems ADD FOREIGN KEY (department) REFERENCES departments (id) ON DELETE SET NULL;
ALTER TABLE modems ADD FOREIGN KEY (domain) REFERENCES domains (id);
ALTER TABLE modems ADD FOREIGN KEY (profile) REFERENCES modem_profiles (id);
ALTER TABLE modems ADD FOREIGN KEY (modem) REFERENCES modems (id) ON DELETE SET NULL;
ALTER TABLE modems ADD FOREIGN KEY (street) REFERENCES streets (id) ON DELETE SET NULL;
ALTER TABLE modems ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE modems_states ADD FOREIGN KEY (modem) REFERENCES modems (id) ON DELETE CASCADE;
ALTER TABLE modems_actions ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE movements ADD FOREIGN KEY (calendar_period) REFERENCES calendar_periods (id) ON DELETE CASCADE;
ALTER TABLE movements ADD FOREIGN KEY (src) REFERENCES accounts (id);
ALTER TABLE movements ADD FOREIGN KEY (dst) REFERENCES accounts (id);
ALTER TABLE movements ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE networks ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE notified_personal_periods ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE notified_personal_periods ADD FOREIGN KEY (personal_period) REFERENCES personal_periods (id) ON DELETE CASCADE;
ALTER TABLE operators ADD FOREIGN KEY (department) REFERENCES departments (id) ON DELETE SET NULL;
ALTER TABLE operators ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE operators_access ADD FOREIGN KEY (operator) REFERENCES operators (id) ON DELETE CASCADE;
ALTER TABLE operators_access ADD FOREIGN KEY (entity) REFERENCES system_entities (id) ON DELETE CASCADE;
ALTER TABLE operators_access ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE operators_logins ADD FOREIGN KEY (operator) REFERENCES operators (id) ON DELETE CASCADE;
ALTER TABLE pays_import ADD FOREIGN KEY (head) REFERENCES pays_import_head (id) ON DELETE CASCADE;
ALTER TABLE pays_import_head ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE personal_period_plans ADD FOREIGN KEY (personal_period) REFERENCES personal_periods (id) ON DELETE CASCADE;
ALTER TABLE personal_period_plans ADD FOREIGN KEY (resource_type) REFERENCES resource_types (id);
ALTER TABLE personal_period_plans ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE personal_periods ADD FOREIGN KEY (next) REFERENCES personal_periods (id) ON DELETE SET NULL;
ALTER TABLE personal_periods ADD FOREIGN KEY (client) REFERENCES clients (id);
ALTER TABLE personal_periods ADD FOREIGN KEY (connection_type) REFERENCES connection_types (id);
ALTER TABLE personal_periods ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE personal_period_resource_uses ADD FOREIGN KEY (personal_period) REFERENCES personal_periods (id);
ALTER TABLE personal_period_resource_uses ADD FOREIGN KEY (resource_use) REFERENCES resource_uses (id) ON DELETE CASCADE;
ALTER TABLE personal_period_resource_uses ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE port_services ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE portmone_pays ADD FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL;
ALTER TABLE portmone_pays ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL;
ALTER TABLE protocols ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE queries ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE resource_types ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE resource_uses ADD FOREIGN KEY (calendar_period) REFERENCES calendar_periods (id);
ALTER TABLE resource_uses ADD FOREIGN KEY (client) REFERENCES clients (id);
ALTER TABLE resource_uses ADD FOREIGN KEY (resource_type) REFERENCES resource_types (id);
ALTER TABLE resource_uses ADD FOREIGN KEY (src) REFERENCES accounts (id);
ALTER TABLE resource_uses ADD FOREIGN KEY (dst) REFERENCES accounts (id);
ALTER TABLE resource_uses ADD FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL;
ALTER TABLE resource_uses ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE rules ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE self_limitations ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE self_limitations ADD FOREIGN KEY (resource_type) REFERENCES resource_types (id);
ALTER TABLE self_limitations ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE shapers ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
-- ALTER TABLE stamps ADD FOREIGN KEY (created_by) REFERENCES operators (id);
-- ALTER TABLE stamps ADD FOREIGN KEY (modified_by) REFERENCES operators (id);
ALTER TABLE staff ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE streets ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE system_accounts ADD FOREIGN KEY (account) REFERENCES accounts (id);
ALTER TABLE system_accounts ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE tariffs ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE mac_vendors ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE wm_pays ADD FOREIGN KEY (movement) REFERENCES movements (id);
ALTER TABLE ukr_pays ADD FOREIGN KEY (movement) REFERENCES movements (id);
ALTER TABLE workers ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;