#!/usr/bin/php -q
<?php
/*
author: Ponomarev Denis <ponomarev@gmail.com>
$Date: 2004/04/27 08:37:36 $
$Revision: 1.19 $
 */
!isset($_SERVER['SERVER_NAME']) || exit;
error_reporting(E_ALL);

ini_set('include_path', DIRNAME(__FILE__). '/../');
include("includes/options.php");
include("classes/context/include.php");

$beanFactory = new CompositBeanFactory(array(
	new ArrayBeanFactory('boot/caban/log.php'),
	new ArrayBeanFactory('boot/caban/dao.php'),
	new ArrayBeanFactory('boot/caban/tools.php'),
));
$loggerFactory = $beanFactory->getBean('loggerFactory');
$logger = $loggerFactory->getLogger('switch.acl2');

$logger->info("BEGIN");

$tftp_server = '194.50.85.2'; //ip2long(implode('.', array_reverse(explode('.', '194.50.85.1'))));

$edgecore_models = array(
	'.1.3.6.1.4.1.259.6.10.57'=>'ES4612',
	'.1.3.6.1.4.1.259.6.10.74'=>'ES3526XA',
	'.1.3.6.1.4.1.259.6.10.94'=>'ES3552M',
	'.1.3.6.1.4.1.259.8.1.5'=>'ES3510',
	'.1.3.6.1.4.1.259.8.1.6'=>'ES3510',
);

$options = array(
	'destination' => '/var/lib/tftpboot/',
	'snmp-timeout' => 2000000,
	'snmp-retries' => 2,
	'snmp-reader' => 'public',
	'snmp-writer' => 'security',
	'http-timeout' => '10',
	'http-user' => 'security',
	'http-password' => 'security',
);

snmp_set_oid_numeric_print(1);
snmp_set_quick_print(1);
snmp_set_valueretrieval(SNMP_VALUE_PLAIN);

function _snmpset($oid, $type, $value){
	global $options, $modem;
	$result = snmpset($modem['ip'], $options['snmp-writer'], $oid, $type, $value, $options['snmp-timeout'], $options['snmp-retries']);
	if(!$result){
		echo "failed snmpset: ".$modem['ip']." ".$oid." ".$type." ".$value."\n";
	}
	return $result;
}

function countLevel($id){
	global $modems;
	$modem = $modems[$id];
	if($parent = $modem['modem']){
		return countLevel($parent) + 1;
	}else{
		return 0;
	}
}

function compareByLevelDesc($m1, $m2){
	return $m2['level'] - $m1['level'];
}

// rich+

/*************************
* Used MIB variables
* **********************/

$base_oid = ".1.3.6.1.4.1.259.6.10.94";
$acl_oid = $base_oid.".1.16.4.15.1"; // ES3528MO-MIB::diffServAclEntry
$ipace_oid = $base_oid.".1.16.4.17.1"; // ES3528MO-MIB::diffServIpAceEntry
$macace_oid = $base_oid.".1.16.4.18.1"; // ES3528MO-MIB::DiffServMacAceEntry
$classmap_oid = $base_oid.".1.16.4.13.1"; // ES3528MO-MIB::diffServClassMapEntry
$policymap_oid = $base_oid.".1.16.4.10.1"; // ES3528MO-MIB::diffServPolicyMapEntry
$dsrv_action_oid = $base_oid.".1.16.4.19.1"; // ES3528MO-MIB::diffServActionEntry
$dsrv_meter_oid = $base_oid.".1.16.4.20.1"; // ES3528MO-MIB::diffServMeterEntry
$policy_element_oid = $base_oid.".1.16.4.12.1"; // diffServPolicyMapElementEntry

/************************
* CONFIGURATION
************************/

$acl_block_all = 1;
$acl_block_msnet = 2;
$acl_clear_dscp = 3;

// !THE ORDER IS IMPORTRANT!
$acl_block_all_aces = array(1, 2, 3, 4, 11, 10);
$acl_block_msnet_aces = array(5, 6, 7, 8, 9);

// Convert arrays of ACEs to switch-using form
$acl_block_all_aces_hex=''; foreach ($acl_block_all_aces as $i){$acl_block_all_aces_hex.=sprintf("%04s",dechex($i));}
$acl_block_msnet_aces_hex=''; foreach ($acl_block_msnet_aces as $i){$acl_block_msnet_aces_hex.=sprintf("%04s",dechex($i));}

$debug = false;
$verbose = true;


function ClassMapControl($classmap_id, $ace_id, $type, $operation){
	/*
	 * Attach or detach one ACE to/from classmap
	 */

	extract($GLOBALS);

	$success = true;

	$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.1.0', 'i', $classmap_id);
	$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.2.0', 'i', $type); // ACE type: ipAce(2)
	$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.3.0', 'i', $ace_id);
	$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.4.0', 'i', $operation);

	switch ($type){
	case 1:
		$ace_type_oid='ES3528MO-MIB::diffServMacAceEntry.16.';
		break;
	case 2:
		$ace_type_oid='ES3528MO-MIB::diffServIpAceEntry.22.';
		break;
//	case 3:
//		$ace_type_oid='ES3528MO-MIB::diffServAclEntry.5.';
//		break;
	}

	if ($operation == 2){
		$success &= _snmpset($ace_type_oid.$ace_id, 'i', 1);
	}
	else{
		$success &= _snmpset($ace_type_oid.$ace_id, 'i', 6);
	}

	return $success;
}

function aceControl($acl_id, $ace_id, $type, $operation){
	/*
	 * Attach or detach one ACE to/from ACL
	 */

	extract($GLOBALS);

	$success = true;

	$success &= _snmpset('ES3528MO-MIB::diffServAclAttachCtl.1.0', 'i', $acl_id);
	$success &= _snmpset('ES3528MO-MIB::diffServAclAttachCtl.2.0', 'i', $type); // ACE type: ipAce(2)
	$success &= _snmpset('ES3528MO-MIB::diffServAclAttachCtl.3.0', 'i', $ace_id);
	$success &= _snmpset('ES3528MO-MIB::diffServAclAttachCtl.4.0', 'i', $operation);

	switch ($type){
	case 1:
		$ace_type_oid='ES3528MO-MIB::diffServMacAceEntry.16.';
		break;
	case 2:
		$ace_type_oid='ES3528MO-MIB::diffServIpAceEntry.22.';
		break;
	}

	if ($operation == 2){
		$success &= _snmpset($ace_type_oid.$ace_id, 'i', 1);
	}
	else{
		$success &= _snmpset($ace_type_oid.$ace_id, 'i', 6);
	}

	return $success;
}

function ClearClassMap($classmap_id){

	extract($GLOBALS);

	$success = true;

	// Exit if ClassMap dont exists
	$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $classmap_oid.".7.".$classmap_id, $options['snmp-timeout'], $options['snmp-retries']);
	if ($oidvalue == '')
	{
		return $success;
	}

	// Exit if $operation == detach and ACEs list is empty
	$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $classmap_oid.".6.".$classmap_id, $options['snmp-timeout'], $options['snmp-retries']);
	if ($oidvalue == ''){
		return $success;
	}

	// Get current ACE list attached to $classmap_id
	$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $classmap_oid.".6.".$classmap_id, $options['snmp-timeout'], $options['snmp-retries']);
	$aces_modem=unpack("H*",$oidvalue); $aces_modem=$aces_modem[1];

	// Get types of current ACEs attached to $classmap_id
	$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $classmap_oid.".5.".$classmap_id, $options['snmp-timeout'], $options['snmp-retries']);
	$ace_types_modem=unpack("H*",$oidvalue); $ace_types_modem=$ace_types_modem[1];
	$ace_types_modem = str_split($ace_types_modem,4);
	
	$success &= _snmpset($classmap_oid.'.7.'.$classmap_id, 'i', 2);

	$aces = $aces_modem;

	$j=0;
	foreach(str_split($aces,4) as $i){
		$curr_ace=hexdec($i);
		$curr_ace_type=hexdec($ace_types_modem[$j]);
		if ($curr_ace_type != 3){
			$success &= ClassMapControl($classmap_id, $curr_ace, $curr_ace_type, 3);
		}
		$j+=1;
	}

	$success &= _snmpset($classmap_oid.'.7.'.$classmap_id, 'i', 6);

	return $success;
}

function aclControl($acl_id, $acesListHex, $operation){
	/*
	 * Make $acl_id ACEs list correspond to $acesListHex
	 *
	 * If ACL $acl_id is not exists, return true.
	 */

	extract($GLOBALS);
	echo "OPETARION: ".$operation."\n";

	$success = true;

	// Exit if ACL dont exists
	$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $acl_oid.".5.".$acl_id, $options['snmp-timeout'], $options['snmp-retries']);
	if ($oidvalue == '')
	{
		return $success;
	}

	// Exit if $operation == detach and ACEs list is empty
	$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $acl_oid.".4.".$acl_id, $options['snmp-timeout'], $options['snmp-retries']);
	if ($operation == 3 && $oidvalue == ''){
		return $success;
	}

	// Get current ACE list attached to $acl_id
	$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $acl_oid.".4.".$acl_id, $options['snmp-timeout'], $options['snmp-retries']);
	$aces_modem=unpack("H*",$oidvalue); $aces_modem=$aces_modem[1];

	// Exit if $aces_modem equal to $acesListHex
	if ($acesListHex == $aces_modem){
		return $success;
	}

	$success &= _snmpset($acl_oid.'.5.'.$acl_id, 'i', 2);

	if ($operation == 2){
		$aces = $acesListHex;
	}
	else{
		$aces = $aces_modem;
	}
	
	foreach(str_split($aces,4) as $i){
		$curr_ace=hexdec($i);
		$success &= aceControl($acl_id, $curr_ace, 2, $operation);
	}

	if ($operation == 2){
		$success &= _snmpset($acl_oid.'.5.'.$acl_id, 'i', 1);
	}
	else{
		$success &= _snmpset($acl_oid.'.5.'.$acl_id, 'i', 6);
	}

	return $success;
}

function createAcl($curr_id, $name, $type){
	extract($GLOBALS);

	$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $acl_oid.".2.".$curr_id, $options['snmp-timeout'], $options['snmp-retries']);
	if ( $oidvalue != $name) {
		$success &= _snmpset($acl_oid.'.5.'.$curr_id, 'i', 5);
		$success &= _snmpset($acl_oid.'.2.'.$curr_id, 's', $name);
		$success &= _snmpset($acl_oid.'.3.'.$curr_id, 'i', $type);
		$success &= _snmpset($acl_oid.'.5.'.$curr_id, 'i', 1);
	}
}

function createClassmap($curr_id, $name, $desc){
	extract($GLOBALS);

	$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $classmap_oid.".2.".$curr_id, $options['snmp-timeout'], $options['snmp-retries']);
	if ( $oidvalue != $name) {
		$success &= _snmpset($classmap_oid.'.7.'.$curr_id, 'i', 5);
		$success &= _snmpset($classmap_oid.'.2.'.$curr_id, 's', $name);
		$success &= _snmpset($classmap_oid.'.3.'.$curr_id, 's', $desc);
		$success &= _snmpset($classmap_oid.'.4.'.$curr_id, 'i', 1); // matchAny
		$success &= _snmpset($classmap_oid.'.7.'.$curr_id, 'i', 1);
	}
}

function createPolicy($curr_id, $name, $desc){
	extract($GLOBALS);

	$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $policymap_oid.".2.".$curr_id, $options['snmp-timeout'], $options['snmp-retries']);
	if ( $oidvalue != $name) {
		$success &= _snmpset($policymap_oid.'.5.'.$curr_id, 'i', 5);
		$success &= _snmpset($policymap_oid.'.2.'.$curr_id, 's', $name);
		$success &= _snmpset($policymap_oid.'.3.'.$curr_id, 's', $desc);
		$success &= _snmpset($policymap_oid.'.5.'.$curr_id, 'i', 1);
	}
}

function createPolicyMeter($curr_id, $rate, $burst){
	extract($GLOBALS);

	$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $dsrv_meter_oid.".3.".$curr_id, $options['snmp-timeout'], $options['snmp-retries']);
	if ( $oidvalue != $rate) {
		$success &= _snmpset($dsrv_meter_oid.'.6.'.$curr_id, 'i', 5);
		$success &= _snmpset($dsrv_meter_oid.'.3.'.$curr_id, 'i', $rate);
		$success &= _snmpset($dsrv_meter_oid.'.4.'.$curr_id, 'i', $burst); // The maximum number of bytes in a single transmission burst
		$success &= _snmpset($dsrv_meter_oid.'.6.'.$curr_id, 'i', 1);
	}
}

function createPolicyAction($curr_id, $action, $PktNewDscp, $RedPktNewDscp){
	extract($GLOBALS);

	$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], "ES3528MO-MIB::diffServActionEntry.8.".$curr_id, $options['snmp-timeout'], $options['snmp-retries']);
	if ( $oidvalue != 1) {
		$success &= _snmpset('ES3528MO-MIB::diffServActionEntry.8.'.$curr_id, 'i', 5);
		$success &= _snmpset('ES3528MO-MIB::diffServActionEntry.2.'.$curr_id, 'b', $action);
		if ($PktNewDscp >= 0){
			$success &= _snmpset('ES3528MO-MIB::diffServActionEntry.5.'.$curr_id, 'i', $PktNewDscp);
		}
		if ($RedPktNewDscp >= 0){
			$success &= _snmpset('ES3528MO-MIB::diffServActionEntry.6.'.$curr_id, 'i', $RedPktNewDscp);
		}
		$success &= _snmpset('ES3528MO-MIB::diffServActionEntry.8.'.$curr_id, 'i', 1);
	}
}

function createPolicyElement($curr_id, $classmap, $meter, $action){
	extract($GLOBALS);

	$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], "ES3528MO-MIB::diffServPolicyMapElementEntry.5.".$curr_id, $options['snmp-timeout'], $options['snmp-retries']);
	if ( $oidvalue != 1) {
		$success &= _snmpset('ES3528MO-MIB::diffServPolicyMapElementEntry.5.'.$curr_id, 'i', 5);
		$success &= _snmpset('ES3528MO-MIB::diffServPolicyMapElementEntry.2.'.$curr_id, 'i', $classmap);
		if ($meter > 0){
			$success &= _snmpset('ES3528MO-MIB::diffServPolicyMapElementEntry.3.'.$curr_id, 'i', $meter); // meter ID
		}
		$success &= _snmpset('ES3528MO-MIB::diffServPolicyMapElementEntry.4.'.$curr_id, 'i', $action); // action id
		$success &= _snmpset('ES3528MO-MIB::diffServPolicyMapElementEntry.5.'.$curr_id, 'i', 1);
	}
}

function AttachPolicyElement($curr_id, $element_id){
	extract($GLOBALS);

	$success &= _snmpset('ES3528MO-MIB::diffServPolicyMapAttachCtl.1.0', 'i', $curr_id); // policy map id
	$success &= _snmpset('ES3528MO-MIB::diffServPolicyMapAttachCtl.2.0', 'i', $element_id); // policy element id
	$success &= _snmpset('ES3528MO-MIB::diffServPolicyMapAttachCtl.3.0', 'i', 2);
}

function convertAceId($acl_aces){
	/*
	 * Return string of hex ACE IDs converted from an array
	 * Input array: array(1, 2, 3, 4, 11, 10)
	 * Output string: 000200030004000b000a
	 */
	$s=''; 
	foreach ($acl_aces as $i){
		$s.=sprintf("%04s",dechex($i));
	}

	return $s;
}

// rich-

$dao = $beanFactory->getBean('modemsDAO');
$db = $beanFactory->getBean('connection');

$it = $dao->iterator();

$modems = array();
while($modem = $it->next()){
	$modems[$modem['id']] = $modem;
}
foreach($modems as $id=>$modem){
	$modems[$id]['level'] = countLevel($id);
}
usort($modems, 'compareByLevelDesc');

foreach($modems as $modem){
	//if($modem['model']=='ES3528M' || $modem['model']=='ES3552M'){
	if($modem['ip']=='10.0.0.4'){
	//if($modem['ip']=='10.0.9.74'){
		$model_oid = @snmpget($modem['ip'], $options['snmp-reader'], ".1.3.6.1.2.1.1.2.0", $options['snmp-timeout'], $options['snmp-retries']);
		if($model_oid){
			if(isset($edgecore_models[$model_oid])){
				$oid = array(
					'set'=>'SNMPv2-SMI::enterprises.259.6.10.94.1.1.7.0',
					'config_src'=>$model_oid.".1.24.1.1.0",
					'config_dst'=>$model_oid.".1.24.1.3.0",
					'config_dst_name'=>$model_oid.".1.24.1.4.0",
					'config_action'=>$model_oid.".1.24.1.8.0"
				);
				$cfg_name = $modem['name'].'-confg';

				$success = true;

				/*************************
				 * CLASSMAP CLEARING
				 * ( desrtoy CLASSMAPs 1-2 if they have ACLs attached )
				 *************************/

				$ids = array(1, 2, 3, 4, 5);
				foreach ($ids as $i) {
					ClearClassMap($i);
				}

				/*************************
				 * CLEARING ACLs
				 *************************/

				aclControl($acl_block_all, convertAceId($acl_block_all_aces), 3);
				aclControl($acl_block_msnet, convertAceId($acl_block_msnet_aces), 3);

				/*************************
				 * ACL CREATING or REPLACING
				 * ( don't raise error if ID is already used )
				 *************************/
				createAcl($acl_block_all, 'ACL_BLOCK_ALL', 3);
				createAcl($acl_block_msnet, 'ACL_BLOCK_MSNET', 3);

				/*************************
				 * ACE CREATING or REPLACING
				 * ( don't raise error if ID is already used )
				 *************************/

				// IpACE 1: permit dst tcp port 80
				$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $ipace_oid.".17.1", $options['snmp-timeout'], $options['snmp-retries']);
				if ( $oidvalue != 80) {
					if ( $debug ) { print "ACE 1\n";}

					$success &= _snmpset($ipace_oid.'.22.1', 'i', 5);
					$success &= _snmpset($ipace_oid.'.2.1', 'i', 2);
					$success &= _snmpset($ipace_oid.'.3.1', 'i', 1);
					$success &= _snmpset($ipace_oid.'.8.1', 'i', 6);
					$success &= _snmpset($ipace_oid.'.16.1', 'i', 2);
					$success &= _snmpset($ipace_oid.'.17.1', 'i', 80);
					$success &= _snmpset($ipace_oid.'.19.1', 'i', 65535);
					$success &= _snmpset($ipace_oid.'.22.1', 'i', 1);					

					if ( $verbose ) { $logger->info("edge %s: ACE 1 created", $modem['name'].'.lanet');}
				}

				// IpACE 2: permit dst tcp port 443
				$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $ipace_oid.".17.2", $options['snmp-timeout'], $options['snmp-retries']);
				if ( $oidvalue != 443) {
					if ( $debug ) { print "ACE 2\n";}

					$success &= _snmpset($ipace_oid.'.22.2', 'i', 5);
					$success &= _snmpset($ipace_oid.'.2.2', 'i', 2);
					$success &= _snmpset($ipace_oid.'.3.2', 'i', 1);
					$success &= _snmpset($ipace_oid.'.8.2', 'i', 6);
					$success &= _snmpset($ipace_oid.'.16.2', 'i', 2);
					$success &= _snmpset($ipace_oid.'.17.2', 'i', 443);
					$success &= _snmpset($ipace_oid.'.19.2', 'i', 65535);
					$success &= _snmpset($ipace_oid.'.22.2', 'i', 1);					

					if ( $verbose ) { $logger->info("edge %s: ACE 2 created", $modem['name'].'.lanet');}
				}

				// IpACE 3: permit DNS
				$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $ipace_oid.".17.3", $options['snmp-timeout'], $options['snmp-retries']);
				if ( $oidvalue != 53) {
					if ( $debug ) { print "ACE 3\n";}

					$success &= _snmpset($ipace_oid.'.22.3', 'i', 5);
					$success &= _snmpset($ipace_oid.'.2.3', 'i', 2);
					$success &= _snmpset($ipace_oid.'.3.3', 'i', 1);
					$success &= _snmpset($ipace_oid.'.6.3', 'a', '194.50.85.0');
					$success &= _snmpset($ipace_oid.'.7.3', 'a', '255.255.255.0');
					$success &= _snmpset($ipace_oid.'.8.3', 'i', 17);
					$success &= _snmpset($ipace_oid.'.16.3', 'i', 2);
					$success &= _snmpset($ipace_oid.'.17.3', 'i', 53);
					$success &= _snmpset($ipace_oid.'.19.3', 'i', 65535);
					$success &= _snmpset($ipace_oid.'.22.3', 'i', 1);

					if ( $verbose ) { $logger->info("edge %s: ACE 3 created", $modem['name'].'.lanet');}
				}

				// IpACE 4: permit DHCP
				$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $ipace_oid.".17.4", $options['snmp-timeout'], $options['snmp-retries']);
				if ( $oidvalue != 67) {
					if ( $debug ) { print "ACE 4\n";}

					$success &= _snmpset($ipace_oid.'.22.4', 'i', 5);
					$success &= _snmpset($ipace_oid.'.2.4', 'i', 2);
					$success &= _snmpset($ipace_oid.'.3.4', 'i', 1);
					$success &= _snmpset($ipace_oid.'.8.4', 'i', 17);
					$success &= _snmpset($ipace_oid.'.16.4', 'i', 2);
					$success &= _snmpset($ipace_oid.'.17.4', 'i', 67);
					$success &= _snmpset($ipace_oid.'.19.4', 'i', 65535);
					$success &= _snmpset($ipace_oid.'.22.4', 'i', 1);

					if ( $verbose ) { $logger->info("edge %s: ACE 4 created", $modem['name'].'.lanet');}
				}

				// IpACE 5: deny 135
				$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $ipace_oid.".17.5", $options['snmp-timeout'], $options['snmp-retries']);
				if ( $oidvalue != 135) {
					if ( $debug ) { print "ACE 5\n";}

					$success &= _snmpset($ipace_oid.'.22.5', 'i', 5);
					$success &= _snmpset($ipace_oid.'.2.5', 'i', 2);
					$success &= _snmpset($ipace_oid.'.3.5', 'i', 2);
					$success &= _snmpset($ipace_oid.'.8.5', 'i', 6);
					$success &= _snmpset($ipace_oid.'.16.5', 'i', 2);
					$success &= _snmpset($ipace_oid.'.17.5', 'i', 135);
					$success &= _snmpset($ipace_oid.'.19.5', 'i', 65535);
					$success &= _snmpset($ipace_oid.'.22.5', 'i', 1);

					if ( $verbose ) { $logger->info("edge %s: ACE 5 created", $modem['name'].'.lanet');}
				}

				// IpACE 6: deny 139
				$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $ipace_oid.".17.6", $options['snmp-timeout'], $options['snmp-retries']);
				if ( $oidvalue != 139) {
					if ( $debug ) { print "ACE 6\n";}

					$success &= _snmpset($ipace_oid.'.22.6', 'i', 5);
					$success &= _snmpset($ipace_oid.'.2.6', 'i', 2);
					$success &= _snmpset($ipace_oid.'.3.6', 'i', 2);
					$success &= _snmpset($ipace_oid.'.8.6', 'i', 6);
					$success &= _snmpset($ipace_oid.'.16.6', 'i', 2);
					$success &= _snmpset($ipace_oid.'.17.6', 'i', 139);
					$success &= _snmpset($ipace_oid.'.19.6', 'i', 65535);
					$success &= _snmpset($ipace_oid.'.22.6', 'i', 1);

					if ( $verbose ) { $logger->info("edge %s: ACE 6 created", $modem['name'].'.lanet');}
				}

				// IpACE 7: deny 445
				$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $ipace_oid.".17.7", $options['snmp-timeout'], $options['snmp-retries']);
				if ( $oidvalue != 445) {
					if ( $debug ) { print "ACE 7\n";}

					$success &= _snmpset($ipace_oid.'.22.7', 'i', 5);
					$success &= _snmpset($ipace_oid.'.2.7', 'i', 2);
					$success &= _snmpset($ipace_oid.'.3.7', 'i', 2);
					$success &= _snmpset($ipace_oid.'.8.7', 'i', 6);
					$success &= _snmpset($ipace_oid.'.16.7', 'i', 2);
					$success &= _snmpset($ipace_oid.'.17.7', 'i', 445);
					$success &= _snmpset($ipace_oid.'.19.7', 'i', 65535);
					$success &= _snmpset($ipace_oid.'.22.7', 'i', 1);

					if ( $verbose ) { $logger->info("edge %s: ACE 7 created", $modem['name'].'.lanet');}
				}

				// IpACE 8: deny 2869
				$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $ipace_oid.".17.8", $options['snmp-timeout'], $options['snmp-retries']);
				if ( $oidvalue != 2869) {
					if ( $debug ) { print "ACE 8\n";}

					$success &= _snmpset($ipace_oid.'.22.8', 'i', 5);
					$success &= _snmpset($ipace_oid.'.2.8', 'i', 2);
					$success &= _snmpset($ipace_oid.'.3.8', 'i', 2);
					$success &= _snmpset($ipace_oid.'.8.8', 'i', 6);
					$success &= _snmpset($ipace_oid.'.16.8', 'i', 2);
					$success &= _snmpset($ipace_oid.'.17.8', 'i', 2869);
					$success &= _snmpset($ipace_oid.'.19.8', 'i', 65535);
					$success &= _snmpset($ipace_oid.'.22.8', 'i', 1);

					if ( $verbose ) { $logger->info("edge %s: ACE 8 created", $modem['name'].'.lanet');}
				}

				// IpACE 9: deny udp 1900
				$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $ipace_oid.".17.9", $options['snmp-timeout'], $options['snmp-retries']);
				if ( $oidvalue != 1900) {
					if ( $debug ) { print "ACE 9\n";}

					$success &= _snmpset($ipace_oid.'.22.9', 'i', 5);
					$success &= _snmpset($ipace_oid.'.2.9', 'i', 2);
					$success &= _snmpset($ipace_oid.'.3.9', 'i', 2);
					$success &= _snmpset($ipace_oid.'.8.9', 'i', 17);
					$success &= _snmpset($ipace_oid.'.16.9', 'i', 2);
					$success &= _snmpset($ipace_oid.'.17.9', 'i', 1900);
					$success &= _snmpset($ipace_oid.'.19.9', 'i', 65535);
					$success &= _snmpset($ipace_oid.'.22.9', 'i', 1);

					if ( $verbose ) { $logger->info("edge %s: ACE 9 created", $modem['name'].'.lanet');}
				}

				// IpACE 10 (last for ACL BLOCKALL): deny all
				$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $ipace_oid.".22.10", $options['snmp-timeout'], $options['snmp-retries']);
				if ( $oidvalue != 1) {
					if ( $debug ) { print "ACE 10\n";}

					$success &= _snmpset($ipace_oid.'.22.10', 'i', 5);
					$success &= _snmpset($ipace_oid.'.2.10', 'i', 2);
					$success &= _snmpset($ipace_oid.'.3.10', 'i', 2);
					$success &= _snmpset($ipace_oid.'.22.10', 'i', 1);

					if ( $verbose ) { $logger->info("edge %s: ACE 10 created", $modem['name'].'.lanet');}
				}

				// IpACE 11: permit PING (for testing purposes)
				$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $ipace_oid.".8.11", $options['snmp-timeout'], $options['snmp-retries']);
				if ( $oidvalue != 1) {
					if ( $debug ) { print "ACE 11\n";}

					$success &= _snmpset($ipace_oid.'.22.11', 'i', 5);
					$success &= _snmpset($ipace_oid.'.2.11', 'i', 2);
					$success &= _snmpset($ipace_oid.'.3.11', 'i', 1);
					$success &= _snmpset($ipace_oid.'.6.11', 'a', '194.50.85.0');
					$success &= _snmpset($ipace_oid.'.7.11', 'a', '255.255.255.0');
					$success &= _snmpset($ipace_oid.'.8.11', 'i', 1);
					$success &= _snmpset($ipace_oid.'.22.11', 'i', 1);

					if ( $verbose ) { $logger->info("edge %s: ACE 11 created", $modem['name'].'.lanet');}
				}

				/*************************
				 * ATTACHING ACE TO ACL
				 * ( raise error only when used ACE is already 
				 * attached to ANOTHER ACL or object not exists )
				 *************************/

				aclControl($acl_block_all, convertAceId($acl_block_all_aces), 2);
				aclControl($acl_block_msnet, convertAceId($acl_block_msnet_aces), 2);

				/*************************
				 * CLASSMAP CREATING or REPLACING
				 * ( don't raise error if IF is already used )
				 *************************/

				createClassmap(1, 'DROP_ALL','');
				createClassmap(2, 'DROP_MSNET','');
				createClassmap(3, 'IPTV_VLAN', ''); // vlan 33, used in UPLINK_POLICY
				createClassmap(4, 'SW_CONTROL_VLAN', ''); // vlan 2, used in UPLINK_POLICY
				createClassmap(5, 'CLIENT_VLANS',''); // used in DEFAULT_POLICY, should be LAST because of unknown count of used ACE IDs

				/*************************
				 * ATTACHING ACL TO CLASSMAP
				 * ( don't raise errors if IDs are correct )
				 *************************/

				// ATTACH ACL 2 to CLASSMAP 1
				if ( $debug ) { print "ACL acl_block_all -> CLASSMAP 1\n";}

				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.1.0', 'i', 1);
				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.2.0', 'i', 3);
				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.3.0', 'i', $acl_block_all);
				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.4.0', 'i', 2);
				if ( $verbose ) { $logger->info("edge %s: ACL 2 attached to CLASSMAP 1", $modem['name'].'.lanet');}

				// ATTACH ACL 1 TO CLASSMAP 2
				if ( $debug ) { print "ACL acl_block-msnet -> CLASSMAP 2\n";}

				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.1.0', 'i', 2);
				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.2.0', 'i', 3);
				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.3.0', 'i', $acl_block_msnet);
				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.4.0', 'i', 2);
				if ( $verbose ) { $logger->info("edge %s: ACL 1 attached to CLASSMAP 2", $modem['name'].'.lanet');}

				// ATTACH IPTV Vlan (33) to CLASSMAP IPTV_VLAN (4)
				$iptv_vlan_id = 33;
				if ( $debug ) { print "ACE 12\n";};

				$success &= _snmpset($macace_oid.'.16.12', 'i', 5);
				$success &= _snmpset($macace_oid.'.2.12', 'i', 1);
				$success &= _snmpset($macace_oid.'.3.12', 'i', 1);
				$success &= _snmpset($macace_oid.'.8.12', 'i', 2);
				$success &= _snmpset($macace_oid.'.9.12', 'i', $iptv_vlan_id); //Vlan id
				$success &= _snmpset($macace_oid.'.16.12', 'i', 1);

				if ( $verbose ) { $logger->info("edge %s: ACE 12 created", $modem['name'].'.lanet');}

				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.1.0', 'i', 3);
				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.2.0', 'i', 1);
				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.3.0', 'i', 12);
				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.4.0', 'i', 2);

				// ATTACH switch control VLAN to CLASSMAP SW_CONTROL_VLAN (5)
				$sw_control_vlan_id = 2;
				if ( $debug ) { print "ACE 13\n";};

				$success &= _snmpset($macace_oid.'.16.13', 'i', 5);
				$success &= _snmpset($macace_oid.'.2.13', 'i', 1);
				$success &= _snmpset($macace_oid.'.3.13', 'i', 1);
				$success &= _snmpset($macace_oid.'.8.13', 'i', 2);
				$success &= _snmpset($macace_oid.'.9.13', 'i', $sw_control_vlan_id); //Vlan id
				$success &= _snmpset($macace_oid.'.16.13', 'i', 1);

				if ( $verbose ) { $logger->info("edge %s: ACE 13 created", $modem['name'].'.lanet');}

				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.1.0', 'i', 4);
				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.2.0', 'i', 1);
				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.3.0', 'i', 13);
				$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.4.0', 'i', 2);

				// ATTACH VLANS FROM $modem['address_space'] to VLAN CLASSMAP CLIENTS_VLAN (3)	
				$last_ace_id = 14;
				foreach (split("\r",$modem['address_space']) as $i){
					$vlan_id = substr(strstr($i,':'),1);
					// MacACE for "CLEAR_DSCP" CLASS-MAP
					//$oidvalue = @snmpget($modem['ip'], $options['snmp-reader'], $macace_oid.".16.".$last_ace_id, $options['snmp-timeout'], $options['snmp-retries']);
					//if ( $oidvalue != 1 ) 
					// Rewrite rules in any case
					{
						if ( $debug ) { print "ACE $last_ace_id\n";}

						$success &= _snmpset($macace_oid.'.16.'.$last_ace_id, 'i', 5);
						$success &= _snmpset($macace_oid.'.2.'.$last_ace_id, 'i', 1);
						$success &= _snmpset($macace_oid.'.3.'.$last_ace_id, 'i', 1);
						$success &= _snmpset($macace_oid.'.8.'.$last_ace_id, 'i', 2);
						$success &= _snmpset($macace_oid.'.9.'.$last_ace_id, 'i', $vlan_id); //Vlan id
						$success &= _snmpset($macace_oid.'.16.'.$last_ace_id, 'i', 1);

						if ( $verbose ) { $logger->info("edge %s: ACE $last_ace_id created", $modem['name'].'.lanet');}

						$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.1.0', 'i', 5);
						$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.2.0', 'i', 1);
						$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.3.0', 'i', $last_ace_id);
						$success &= _snmpset('ES3528MO-MIB::diffServClassMapAttachCtl.4.0', 'i', 2);
						
						$last_ace_id+=1;
					}
				}
				
				/*************************
				 * POLICYMAP CREATIGN OR REPLACING
				 * ( don't raise errors if ID is already used )
				 *************************/

				createPolicy(1, 'DENY_ALL', '');
				createPolicy(2, 'DENY_MSNET', '');
				createPolicy(3, 'UPLINK_POLICY', '');
				createPolicy(4, 'DEFAULT_POLICY', '');

				/*************************
				 * POLICYMAP METERs CREATING
				 *************************/

				createPolicyMeter(1, 1000, 64);
				createPolicyMeter(2, 1000, 64);
				createPolicyMeter(3, 1, 64);						
				
				/*************************
				 * POLICYMAP ACTIONS CREATING
				 *************************/

				createPolicyAction(1, '2,4', 0, -1); // Action for DENY_ALL
				createPolicyAction(2, '2,4', 0, -1); // Action for DENY_MSNET
				createPolicyAction(3, '2', 56, -1); // Action for UPLINK_POLICY, SW_CONTROL_VLAN
				createPolicyAction(4, '2,3', 0, 42); // Action for UPLINK_POLICY, IPTV_VLAN
				createPolicyAction(5, '2', 0, -1); // Action for DEFAULT_POLICY
				

				/*************************
				 * POLICYMAP ELEMENTS CREATING
				 *************************/

				createPolicyElement(1, 1, 1, 1);
				createPolicyElement(2, 2, 2, 2);
				createPolicyElement(3, 4, -1, 3); // Policy UPLINK_POLICY, Class SW_CONTROL_VLAN
				createPolicyElement(4, 3, 3, 4); // Policy UPLINK_POLICY, Class IPTV_VLAN
				createPolicyElement(5, 5, -1, 5); //Policy DEFAULT_POLICY

				/*************************
				 * ATTACH ELEMENTS TO POLICY MAPS
				 *************************/

				AttachPolicyElement(1,1);
				AttachPolicyElement(2,2);
				AttachPolicyElement(3,3);
				AttachPolicyElement(3,4);
				AttachPolicyElement(4,5);

				//break;
			}
		}else {
			$logger->error("edge %s: no answer", $modem['name']);
		}
	}
}
$logger->info("END");

?>
