#!/usr/bin/php -q
<?php
/*
author: Ponomarev Denis <ponomarev@gmail.com>
$Date: 2004/04/27 08:37:36 $
$Revision: 1.19 $
*/
!isset($_SERVER['SERVER_NAME']) || exit;
error_reporting(E_ALL);

ini_set('include_path', DIRNAME(__FILE__). '/../');
include("includes/options.php");
include("classes/context/include.php");

$beanFactory = new CompositBeanFactory(array(
	new ArrayBeanFactory('boot/caban/log.php'),
	new ArrayBeanFactory('boot/caban/dao.php'),
	new ArrayBeanFactory('boot/caban/tools.php'),
));
$loggerFactory = $beanFactory->getBean('loggerFactory');
$logger = $loggerFactory->getLogger('switch.save.conf');

$logger->info("BEGIN");

$tftp_server = '194.50.85.2'; //ip2long(implode('.', array_reverse(explode('.', '194.50.85.1'))));

$celan_models = array(
	'.1.3.6.1.4.1.655.5.3'=>'2402Fi-EA',
	'.1.3.6.1.4.1.655.5.11'=>'8F00i-EA',
	'.1.3.6.1.4.1.655.5.15'=>'0800i-EA',
	'.1.3.6.1.4.1.655.5.16'=>'1601Fi-EA',
	'.1.3.6.1.4.1.655.5.21'=>'0701Fi-EA', //14
	'.1.3.6.1.4.1.655.9.3'=>'2402FGi-EA', //15
	'.1.3.6.1.4.1.655.9.8'=>'8F02Gi-EA',
);
$edgecore_models = array(
	'.1.3.6.1.4.1.259.6.10.57'=>'ES4612',
	'.1.3.6.1.4.1.259.6.10.74'=>'ES3526XA',
	'.1.3.6.1.4.1.259.6.10.94'=>'ES3552M',
	'.1.3.6.1.4.1.259.8.1.5'=>'ES3510',
	'.1.3.6.1.4.1.259.8.1.6'=>'ES3510',
);

$options = array(
	'destination' => '/var/lib/tftpboot/',
	'snmp-timeout' => 2000000,
	'snmp-retries' => 2,
	'snmp-reader' => 'public',
	'snmp-writer' => 'security',
	'http-timeout' => '10',
	'http-user' => 'security',
	'http-password' => 'security',
);

snmp_set_oid_numeric_print(1);
snmp_set_quick_print(1);
snmp_set_valueretrieval(SNMP_VALUE_PLAIN);

$ch = curl_init();

curl_setopt($ch, CURLOPT_TIMEOUT, $options['http-timeout']);
curl_setopt($ch, CURLOPT_HTTPHEADER, array("Authorization: Basic ".base64_encode(sprintf("%s:%s", $options['http-user'], $options['http-password']))));

$dao = $beanFactory->getBean('modemsDAO');
$it = $dao->iterator();
while($modem = $it->next()){
	$model_oid = @snmpget($modem['ip'], $options['snmp-reader'], ".1.3.6.1.2.1.1.2.0", $options['snmp-timeout'], $options['snmp-retries']);
	if($model_oid){
		if(isset($celan_models[$model_oid])){
			curl_setopt($ch, CURLOPT_URL, sprintf("http://%s/config_txt.log", $modem['ip']));
			ob_start();
			curl_exec($ch);
			$err = curl_errno($ch);
			$result = ob_get_contents();
			ob_end_clean();
			if($err){
				$logger->error("celan %s failed: %s", $modem['name'], curl_error($ch));
			}else{
				if(strlen($result) > 1024){
					if($file = fopen($options['destination'].$modem['name'].'-confg', "w+")){
						fputs($file, $result);
						fclose($file);
						$logger->info("celan %s ok", $modem['name']);
					}else{
						$logger->error("celan %s: can't save", $modem['name']);
					}
				}else{
					$logger->error("celan %s: too short", $modem['name']);
				}
			}
		}
		if(isset($edgecore_models[$model_oid])){
			if($model_oid == '.1.3.6.1.4.1.259.8.1.6'){
				$model_oid = '.1.3.6.1.4.1.259.8.1.5';
			}
			$logger->info("EdgeCore %s", $modem['name']);
			$cfg_name = $modem['name'].'-confg';
			$filename = "/var/lib/tftpboot/".$cfg_name;
			if(!is_file($filename)){
				touch($filename);
				chmod($filename, 0666);
			}
			$oid = array(
				'copy_src' => $model_oid.".1.24.1.1.0",
				'copy_src_name' => $model_oid.".1.24.1.2.0",
				'copy_dst' => $model_oid.".1.24.1.3.0",
				'copy_filename' => $model_oid.".1.24.1.4.0",
				'tftp_server' => $model_oid.".1.24.1.6.0",
				'copy_action' => $model_oid.".1.24.1.8.0",
			);
			if(
				@snmpset($modem['ip'], $options['snmp-writer'], $oid['copy_src'], 'i', 2, $options['snmp-timeout'], $options['snmp-retries']) &&
				@snmpset($modem['ip'], $options['snmp-writer'], $oid['copy_src_name'], 's', 'running-config', $options['snmp-timeout'], $options['snmp-retries']) &&
				@snmpset($modem['ip'], $options['snmp-writer'], $oid['copy_dst'], 'i', 4, $options['snmp-timeout'], $options['snmp-retries']) &&
				@snmpset($modem['ip'], $options['snmp-writer'], $oid['copy_filename'], 's', $cfg_name, $options['snmp-timeout'], $options['snmp-retries']) &&
				@snmpset($modem['ip'], $options['snmp-writer'], $oid['tftp_server'], 'a', $tftp_server, $options['snmp-timeout'], $options['snmp-retries']) &&
				@snmpset($modem['ip'], $options['snmp-writer'], $oid['copy_action'], 'i', 2, $options['snmp-timeout'], $options['snmp-retries'])
			){
				$logger->info("edgecore %s: ok", $modem['name']);
			}else{
				$logger->error("edgecore %s: failed", $modem['name']);
			}
		}
	}
}
$logger->info("END");

curl_close($ch);
?>