#!/usr/bin/php -q
<?php
set_time_limit (0);

ini_set('include_path', DIRNAME(__FILE__). '/../');

include_once "classes/log/StdoutAppender.php";
include_once "classes/log/StreamAppender.php";
include_once "classes/log/FileAppender.php";
include_once "classes/log/DefaultLogger.php";
include_once "classes/tools/SocketServer.php";
include_once "classes/tools/CLISocketServerDelegate.php";

$opt = getopt('rhc:a:p:m:t:l:i:o:v:n:');

function unslash($str){
	return str_replace(array('\n', '\r', '\t', '\0'), array("\n", "\r", "\t", "\0"), $str);
}

if(isset($opt['h'])){
?>

-a <address>     address to bind 
-p <port>        port to bind, default 9000
-m <max_clients> max clients, default 100
-t <terminator>  terminator string, default \n
-c <command>     command to process messages, if none given 'cat' will be used
-i <stream>      file descriptor where command will access data, default 0 - stdin
-o <stream>      file descriptor where command will return data, default 1 - stdout
-l <stream>      file descriptor for log, default 2 - stderr
-v <level>       logging level 1-5 (DEBUG - NONE)
-n               name in log, default 'socket.server'
-r               retry mode, dont blame if port address already in use, silent quit
-h               this help

<?
exit;
}

$host = isset($opt['a']) ? $opt['a'] : $_SERVER['HOSTNAME'];
$port = isset($opt['p']) ? $opt['p'] : 9000;
$cmd = isset($opt['c']) ? $opt['c'] : 'cat';
$maxClients = isset($opt['m']) ? $opt['m'] : 100;
$terminator = isset($opt['t']) ? unslash($opt['t']) : "\n";
$in_from = isset($opt['i']) ? $opt['i'] : 0;
$out_to = isset($opt['o']) ? $opt['o'] : 1;
$log_to = isset($opt['l']) ? $opt['l'] : 2;
$log_level = isset($opt['v']) ? intval($opt['v']) : 2;
$log_name = isset($opt['n']) ? $opt['n'] : 'socket.server';

$logger = new DefaultLogger();
$logger->setName($log_name);
$logger->setLevel($log_level);
if($log_stream = fopen('/proc/self/fd/'.$log_to, "a+")){
	$logger->setAppender(new StreamAppender($log_stream));
}else{
	$logger->setAppender(new StdoutAppender());
	$logger->error('cant open %s for logging', $log_to);
}

$delegate = new CLISocketServerDelegate($cmd, $logger, $terminator, $in_from, $out_to);
$server = new SocketServer($delegate, $logger, $host, $port, $maxClients);
$server->start(isset($opt['r']));
?>