#!/usr/bin/php -q
<?php
if(count($_SERVER['argv']) < 3){
	die("Request incomplete");
}
$ip = $_SERVER['argv'][1];
$cfg = $_SERVER['argv'][2];
$timeout = 3;

ini_set('include_path', DIRNAME(__FILE__). '/../');
include("includes/options.php");
include("classes/context/include.php");
$beanFactory = new CompositBeanFactory(array(
	new ArrayBeanFactory('boot/caban/log.php'),
));
$loggerFactory = $beanFactory->getBean('loggerFactory');
$logger = $loggerFactory->getLogger('switch.celan');

$logger->info("configuring: %s", $ip);

$positives = array('on','true','1','yes','enable');
$option_pattern = "|([\w\-]+)\s*=\s*(\S.*)|ix";
//$item_pattern = "|(\w\w\:\w\w\:\w\w\:\w\w\:\w\w\:\w\w)\s+(\d+)\s+(\w+)\s+([\w\-_\.]+)|ix";
$item_pattern = "|(\w\w\:\w\w\:\w\w\:\w\w\:\w\w\:\w\w)\s+(\d+)\s+(\w+)\s+([\w\-_\.]+)\s+(\d+\.\d+\.\d+\.\d+)\s+(\d+)|ix";
$pagename_pattern = "|<input type=\"hidden\" name=\"page\" value=\"([^>]+)\">|";

$options = array();
$cfg_macs = array();
$cfg_ports = array();
$cfg_names = array();
$cfg_bindings = array();
$cfg_vlans = array();
if($file = fopen ($cfg, "r")){
	while (!feof($file)) {
		$line = trim(fgets($file, 1024));
		$pos = strpos($line, '#');
		if(!($pos===false)){
			$line = trim(substr($line,0,$pos));
		}
		if($line){
			if(preg_match($option_pattern, $line, $m)){
				$options[strtolower($m[1])] = trim($m[2]);
			}elseif(preg_match($item_pattern, $line, $m)){
				list($all, $mac, $port, $state, $login, $host_ip, $host_vlan) = $m;
				//list($all, $mac, $port, $state, $login) = $m;
				if($port > 0){
					$mac = strtolower(str_replace(":","",$mac));
					$state = in_array(strtolower($state), $positives) ? 1 : 0;
					$cfg_names[$port] = $login;
					if($mac!='000000000000'){
						$cfg_macs[$mac.":".$port] = $state;
						$cfg_bindings[$port][$mac][$host_ip] = 1;
					}
					$cfg_macs[$mac.":".$port] = $state;
					if(isset($cfg_ports[$port])){
						if($cfg_ports[$port]!=$state){
							$cfg_ports[$port] = -1;
						}
					}else{
						$cfg_ports[$port] = $state;
					}
					if($host_vlan > 0){
						$cfg_vlans[$port] = $host_vlan;
					}
				}
			}
		}
	}
	fclose($file);
}else{
	$logger->error("Can't open %s for reading", $cfg);
	exit(1);
}

$def_vlan = $options['vlan'];

$ch = curl_init();
if(isset($options['timeout'])){
	curl_setopt($ch, CURLOPT_TIMEOUT, $options['timeout']);
}
if(isset($options['http-user'])){
	curl_setopt($ch, CURLOPT_HTTPHEADER, array("Authorization: Basic ".base64_encode(sprintf("%s:%s", $options['http-user'], $options['http-password']))));
}

$exit = 0;

$control_state_key = 'control-port-state';
$control_state = isOn('control-port-state');
if($control_state){
	$state_url = sprintf("http://%s/port_configuration.htm", $ip);
	$state_page = get_cmd($ch, $state_url);
	if($state_page){
		$page_name = extractPageName($state_page);
		$port_base = extractPortBase($state_page, "port_id");
		$state_pattern = "|
		<tr>\s*
		<td[^>]+>\s*<font[^>]+>(\d+)</font>\s*</td>\s*
		<td[^>]+>\s*<font[^>]+>([^<]+)</font>\s*</td>\s*
		<td[^>]+>\s*<font[^>]+>([^<]+)</font>\s*</td>\s*
		<td[^>]+>\s*<font[^>]+>([^<]+)</font>\s*</td>\s*
		<td[^>]+>\s*<font[^>]+>([^<]+)</font>\s*</td>\s*
		<td[^>]+>\s*<font[^>]+>([^<]+)</font>\s*</td>\s*
		<td[^>]+>\s*<font[^>]+>([^<]+)</font>\s*</td>\s*
		<td[^>]+>\s*<font[^>]+>([^<]+)</font>\s*</td>\s*
		</tr>
		|imx";
		$enabled_disabled = array('Disabled'=>2, 'Enabled'=>1);
		$speed = array('10M'=>1, '100M'=>2, '1000M'=>3);
		$duplex = array('Half'=>1, 'Full'=>2);
		$on_off = array('On'=>1, 'Off'=>2);
		if(preg_match_all($state_pattern, $state_page, $m, PREG_SET_ORDER)){
			//print_r($m);
			foreach($m as $match){
				$port = $match[1];
				$co_key = $control_state_key.'-'.$port;
				$co_port_state = isset($options[$co_key]) ? isPositive($co_key) : $control_state;
				if($co_port_state){
					$state = array(
						'port_id'=>$port - 1 + $port_base,
						'PortName'=>trim($match[2]),
						'Admin'=>$enabled_disabled[trim($match[4])],
						'AutoNegotiation'=>$enabled_disabled[trim($match[5])],
						'Speed'=>$speed[trim($match[6])],
						'Duplex'=>$duplex[trim($match[7])],
						'FlowControl'=>$on_off[trim($match[8])],
						'page'=>$page_name,
						'RELOAD'=>'Apply',
					);
					//print_r($state);
					if(isset($cfg_ports[$port])){
						$cfg_state = $cfg_ports[$port]!=0 ? 1 : 2;
					}else{
						$cfg_state = $state['Admin'];
					}
					$post = false;
					if($state['Admin'] != $cfg_state){
//						echo $state['Admin']."!=".$cfg_state;
						$post = true;
						$state['Admin'] = $cfg_state;
					}
					if(isset($cfg_names[$port])){
						if($state['PortName'] != $cfg_names[$port]){
//							echo $state['PortName']."!=".$cfg_names[$port];
							$post = true;
							$state['PortName'] = $cfg_names[$port];
						}
					}
					$pn_key = 'port-name-'.$port;
					if(isset($options[$pn_key])){
						if($state['PortName'] != $options[$pn_key]){
						$post = true;
						$state['PortName'] = $options[$pn_key];
						}
					}
					if($post){
						$logger->info("change port [%d] state to [%s]", $port, $cfg_state==1 ? 'On' : 'Off');
						post_cmd($ch, $state_url, $state);
					}
				}
			}
		}else{
			$logger->error("Can't parse port controls page");
			$exit+=8;
		}
	}else{
		$logger->error("Can't read port controls");
		$exit+=8;
		$logger->info("Aborting due to errors"); 
		exit($exit);
	}
}

if(isOn('mac-table')){
	$mac_url = sprintf("http://%s/static_address.htm", $ip);
	$mac_page = get_cmd($ch, $mac_url);
	if($mac_page){
		$page_name = extractPageName($mac_page);
		$port_base = extractPortBase($mac_page, "DesPort");
		$mac1_pattern = "|
		<td[^>]*><font[^>]*>\s*(\d+)\s*[^<]*</font></td>\s*
		<td[^>]*><font[^>]*>\s*(\w\w\-\w\w\-\w\w\-\w\w\-\w\w\-\w\w)\s*[^<]*</font></td>\s*
		<td[^>]*><font[^>]*>\s*(\d+)\s*[^<]*</font></td>\s*
		<td[^>]*><font[^>]*>\s*<input[^>]*name=\"DeleteMAC(\d+)\"[^>]*>\s*[^<]*</font></td>\s*
		|imx";
		$mac2_pattern = "|
		<td[^>]*>\s*(\d+)\s*[^<]*</td>\s*
		<td[^>]*>\s*(\w\w\-\w\w\-\w\w\-\w\w\-\w\w\-\w\w)\s*[^<]*</td>\s*
		<td[^>]*>\s*(\d+)\s*[^<]*</td>\s*
		<td[^>]*>\s*<input[^>]*name=\"DeleteMAC(\d+)\"[^>]*>\s*[^<]*</td>\s*
		|imx";
		$mac_table = array();
		$delete_id = array();
		if(preg_match_all($mac1_pattern, $mac_page, $m, PREG_SET_ORDER)
		|| preg_match_all($mac2_pattern, $mac_page, $m, PREG_SET_ORDER)){
			foreach($m as $match){
				$mac = strtolower(str_replace("-","",$match[2]));
				$key = $mac.":".$match[3];
				$mac_table[$key] = $match[1];
				$delete_id[$key] = $match[4];
			}
		}
		$new_macs = array();
		$mac_out = isOn('mac-out');
		foreach($cfg_macs as $pair=>$state){
			list($mac, $port) = explode(":", $pair);
			$mo_key = 'mac-out-'.$port;
			$port_mo = isset($options[$mo_key]) ? isPositive($mo_key) : $mac_out;
			if($state==1 || !$port_mo){
				if(isset($mac_table[$pair])){
					unset($mac_table[$pair]);
				}else{
					$new_macs[$mac] = $port;
				}
			}
		}
		arsort($mac_table);
		foreach($mac_table as $key=>$index){
			list($mac,$port) = explode(":", $key);
			$id = $delete_id[$key];
			$cmd = sprintf("page=%s&deleteindex=&MACAddr=&DesPort=0&DeleteMAC%s=Delete", $page_name, $id);
			$logger->info("delete MAC %s => PORT %s", $mac, $port);
			post_cmd($ch, $mac_url, $cmd);
		}
		foreach($new_macs as $mac=>$port){
			$logger->info("add MAC %s => PORT %s", $mac, $port);
			$cmd = sprintf("page=%s&deleteindex=&MACAddr=%s&DesPort=%d&AddMacAddress=Add", $page_name, implode("-",sscanf($mac, "%2s%2s%2s%2s%2s%2s")),$port - 1 + $port_base);
			post_cmd($ch, $mac_url, $cmd);
		}
	}else{
		$logger->error("Can't read mac table");
		$exit+=2;
	}
}

$control_storm_key = 'control-storm';
$control_storm = isOn($control_storm_key);
if($control_storm){
	$storm_url = sprintf("http://%s/suppression.htm", $ip);
	$storm_page = get_cmd($ch, $storm_url);
	if($storm_page){
		$page_name = extractPageName($storm_page);
		if($page_name=='iMpacct-Corp'){
			$control_options = array('DLF');//'BC', 'MC',
		}else{
			$control_options = array('DLF');//'BC',
		}
		foreach($control_options as $opt){
			$opt_value = 0;
			if(preg_match("|INPUT\sTYPE=\"RADIO\"\sNAME=\"".$opt."Pkt\"\sVALUE=\"(\d)\"\schecked|", $storm_page, $m)){
				$opt_value = $m[1];
			}
			if($opt_value!=3){
				$cmd = sprintf("page=%s&%sPkt=3&set%sPktOP=Apply", $page_name, $opt, $opt);
				$logger->info("set '%s' option to 'by port'", $opt);
				post_cmd($ch, $storm_url, $cmd);
			}
		}
		$storm_state_patterns = array(
//			'bc'=>"|BCSet=\[([01,]+)\]|",
//			'mc'=>"|MCSet=\[([01,]+)\]|",
			'dlf'=>"|DLFSet=\[([01,]+)\]|",
		);
		$storm_state = array();
		foreach($storm_state_patterns as $k=>$pattern){
			preg_match($pattern, $storm_page, $m);
			$storm_state[$k] = explode(",", $m[1]);
		}
		foreach($cfg_ports as $port=>$state){
			$co_key = $control_storm_key.'-'.$port;
			if(!isset($options[$co_key]) || isPositive($co_key)){
				$cmd = '';
				if($page_name=='iMpacct-Corp'){
					//if(($storm_state['bc'][$port] * $storm_state['mc'][$port] * $storm_state['dlf'][$port]) == 0){
					if(($storm_state['dlf'][$port]) == 0){
						//$cmd = sprintf("page=%s&RateSelecting=0&port_id=%s&BCcheck=on&MCcheck=on&DLFcheck=on&Apply=Apply",
						$cmd = sprintf("page=%s&RateSelecting=0&port_id=%s&DLFcheck=on&Apply=Apply",
							$page_name, $port-1);
					}
				}else{
					//if(($storm_state['bc'][$port] * $storm_state['dlf'][$port]) == 0){
					if(($storm_state['dlf'][$port]) == 0){
						//$cmd = sprintf("page=%s&inSize=1&port_id=%s&BCcheck=on&DLFcheck=on&Apply=Apply",
						$cmd = sprintf("page=%s&inSize=1&port_id=%s&DLFcheck=on&Apply=Apply",
							$page_name, $port);
					}
				}
				if($cmd){
					$logger->info("modifing storm control, port %s", $port);
					post_cmd($ch, $storm_url, $cmd);
				}
			}
		}
	}else{
		$logger->error("Can't read storm control");
		$exit+=16;
	}
}

$control_security_key = 'control-port-security';
$control_security = isOn($control_security_key);
$security_url = sprintf("http://%s/mac_security_configuration.htm", $ip);
$security_page = get_cmd($ch, $security_url);
if($security_page){
	$page_name = extractPageName($security_page);
	$port_security = array();
	$port_pattern = "|
	<select[^>]*name=\"Mode(\d+)\"[^>]*>\s*
	((<option\svalue=\d[^>]*><font[^>]*>[^<]*</font></option>\s*)+)
	</select>
	|imxs";
	$option_pattern = "|<option\svalue=(\d)([^>]*)>|imxs";
	if(preg_match_all($port_pattern, $security_page, $m, PREG_SET_ORDER)){
		foreach($m as $match){
			preg_match_all($option_pattern, $match[2], $match_opt, PREG_SET_ORDER);
			$port_security[$match[1]] = 0;
			foreach($match_opt as $opt){
				if(strstr($opt[2], 'selected')){
					$port_security[$match[1]] = $opt[1];
					break;
				}
			}
		}
		$set_ports = false;
		$cmd = "page=".$page_name;
		foreach($port_security as $port=>$security){
			$co_key = $control_security_key.'-'.$port;
			$co_port_security = isset($options[$co_key]) ? isPositive($co_key) : $control_security;
			if($co_port_security){
				$secure = isset($cfg_ports[$port]) ? 1 : 0;
				if($security!=$secure){
					$set_ports = true;
				}
			}else{
				$secure = $security;
			}
			$cmd .= sprintf("&MacNo%d=0&Mode%d=%d", $port, $port, $secure);
		}
		if($set_ports){
			$logger->info("modifing port security");
			post_cmd($ch, $security_url, $cmd.'&apply=Apply');
		}
	}else{
		//echo $security_page;
		$logger->error("Can't parse port security page");
		$exit+=2;
	}
}else{
	$logger->error("Can't read port security");
	$exit+=4;
}

/*
$pvid_url = sprintf("http://%s/tagged_vlan_configuration.htm", $ip);
$pvid_page = get_cmd($ch, $pvid_url);
if($pvid_page){
	$page_name = extractPageName($security_page);
	$shift = $page_name=='CeLAN-Tech' ? 0 : 1;
	$pvid_pattern = "|PortPvidSet=\[([\d,]+)\]|";
	if(preg_match($pvid_pattern, $pvid_page, $m)){
		$pvids = explode(",", $m[1]);
		foreach($cfg_ports as $port=>$state){
			$cfg_vlan = isset($cfg_vlans[$port]) ? $cfg_vlans[$port] : $def_vlan;
			$pvid = $pvids[$port];
			if($pvid != $cfg_vlan){
				$cmd = sprintf('page=%s&port_id=%s&PVID=%s&PortPriority=0&ApplyPVID=Apply', $page_name, $port-$shift, $cfg_vlan);
				post_cmd($ch, $pvid_url, $cmd);
			}
		}
	}else{
		$logger->error("Can't parse PVID configuration");
		$exit+=32;
	}
}else{
	$logger->error("Can't read PVID configuration");
	$exit+=32;
}

$vlan_url = sprintf("http://%s/static_tagged_vlan.htm?selectedStaticVLAN=", $ip);
$vlan_page = get_cmd($ch, $vlan_url.'0');
if($vlan_page){
	$page_name = extractPageName($vlan_page);
	$shift = $page_name=='CeLAN-Tech' ? 1 : 0;
	$vlan_pattern = '|<option\s+VALUE="([^"]+)"[^>]*>\s*(\d+)\([^<]+</option>|imxs';
	$cfg_pattern = "|staticPortType\[(\d)-1\] = \[([^\]]+)\]|";
	if(preg_match_all($vlan_pattern, $vlan_page, $m, PREG_SET_ORDER)){
		$count = count($m);
		if($count > 1){
			$vlans = array();
			for($i=1; $i<$count; $i++){
				$vlans[$m[$i][1]] = $m[$i][2];
			}
			foreach($vlans as $v=>$vlan){
				if($vlan_page = get_cmd($ch, $vlan_url.$v)){
					if(preg_match_all($cfg_pattern, $vlan_page, $m, PREG_SET_ORDER)){
						$marks = array();
						for($i=0; $i<3; $i++){
							foreach(explode(",", $m[$i][2]) as $port=>$mark){
								if($mark=='"checked"' || $mark=='"1"'){
									$marks[$port + 1] = 3 - $i;
								}
							}
						}
						print_r($marks);
						$change = false;
						foreach($cfg_ports as $port=>$state){
							$cfg_vlan = isset($cfg_vlans[$port]) ? $cfg_vlans[$port] : $def_vlan;
							$mark = $cfg_vlan==$vlan ? 3 : 1;
							if($mark != $marks[$port]){
								$marks[$port] = $mark;
								$change = true;
							}
						}
						if($change){
							$cmd = sprintf('page=%s&selectedStaticVLAN=%s', $page_name, $v);
							foreach($marks as $port=>$mark){
								if($port <= 10){
									$c='A';
								}else if($port <= 20){
									$c='B';
								}else{
									$c='C';
								}
								$cmd .= sprintf('&P%s%s=%s', $c, ($port - $shift)%10, $mark);
							}
							$cmd .= '&applyModify=Apply';
							post_cmd($ch, $vlan_url.$v, $cmd);
						}
					}else{
						$logger->error("Can't parse Static Vlan [".$vlan."] configuration");
						$exit+=64;
						break;
					}
				}else{
					$logger->error("Can't read Static Vlan [".$vlan."] configuration");
					$exit+=64;
					break;
				}
			}
		}else{
			$logger->info("No Vlan configurated");
		}
	}else{
		$logger->error("Can't parse Static Vlan configuration");
		$exit+=64;
	}
}else{
	$logger->error("Can't read Static Vlan configuration");
	$exit+=64;
}
*/

curl_close($ch);
if($exit){
	$logger->info("Complete with errors");
	exit($exit);
}else{
	$logger->info("Complete");
}

function extractPageName($page){
	global $pagename_pattern;
	if(preg_match($pagename_pattern, $page, $mpage)){
		return $mpage[1];
	}else{
		return 'iMpacct-Corp';
	}
}

function extractPortBase($page, $field){
	$base_pattern = "|<select\s+name=\"$field\"[^>]*>\s*<option\s+VALUE=\"(\d)\"[^>]*>\s*1\s*</option>|imx";
	if(preg_match($base_pattern, $page, $mbase)){
		return $mbase[1];
	}else{
		return 0;
	}
}

function isPositive($str){
	global $positives;
	return in_array(strtolower($str), $positives);
}

function isOn($str){
	global $options;
	return isset($options[$str]) && isPositive($options[$str]);
}

function cmd($ch){
	global $logger;
	ob_start();
	curl_exec($ch);
	$err = curl_errno($ch);
	if($err){
		$logger->error("curl error #%d: %s", $err, curl_error($ch));
	}
	$result = ob_get_contents();
	ob_end_clean();
	return $result;
}

function get_cmd($ch, $address){
	curl_setopt($ch, CURLOPT_URL, $address);
	curl_setopt($ch, CURLOPT_POST, 0);
	curl_setopt($ch, CURLOPT_HTTPGET, 1);
	return cmd($ch);
}

function post_cmd($ch, $address, $cmd){
	global $logger;
	if(is_array($cmd)){
		$items = array();
		foreach($cmd as $k=>$v){
			$items[] = sprintf("%s=%s", $k, $v);
		}
		$cmd = implode("&", $items);
	}
	curl_setopt($ch, CURLOPT_URL, $address);
	curl_setopt($ch, CURLOPT_POST, 1);
	curl_setopt($ch, CURLOPT_POSTFIELDS, $cmd);
	$logger->info("curl: %s?%s", $address, $cmd);
	return cmd($ch);
}
?>