#!/usr/bin/php -q
<?php
if(count($_SERVER['argv']) < 3){
	die("Request incomplete");
}
$ip = $_SERVER['argv'][1];
$cfg = $_SERVER['argv'][2];
$secure = true;

openlog("isp.switch.celan.snmp", LOG_PID + LOG_CONS, LOG_USER);
syslog(LOG_INFO, sprintf("Profiling %s", $ip));

$positives = array('on','true','1','yes','enable');
$option_pattern = "|([\w\-]+)\s*=\s*(\S.*)|ix";
$item_pattern = "|(\w\w\:\w\w\:\w\w\:\w\w\:\w\w\:\w\w)\s+(\d+)\s+(\w+)|ix";

$options = array();
$cfg_macs = array();
$cfg_ports = array();
if($file = fopen ($cfg, "r")){
	while (!feof($file)) {
	    $line = trim(fgets($file, 1024));
		$pos = strpos($line, '#');
		if(!($pos===false)){
			$line = trim(substr($line,0,$pos));
		}
		if($line){
			if(preg_match($option_pattern, $line, $m)){
				$options[strtolower($m[1])] = trim($m[2]);
			}elseif(preg_match($item_pattern, $line, $m)){
				list($all, $mac, $port, $state) = $m;
				$mac = strtolower(str_replace(":","",$mac));
				$state = in_array(strtolower($state), $positives) ? 1 : 0;
				$cfg_macs[$mac.":".$port] = $state;
				if(isset($cfg_ports[$port])){
					if($cfg_ports[$port]!=$state){
						$cfg_ports[$port] = -1;
					}
				}else{
					$cfg_ports[$port] = $state;
				}
			}
		}
	}
	fclose($file);
}else{
	syslog(LOG_ERR, sprintf("Can't open %s for reading", $cfg));
	exit(1);
}

$reader = $options['public-community'];
$writer = $options['private-community'];

$timeout = isset($options['timeout']) ? $options['timeout'] * 1000 : 1000000;
$retries = isset($options['retries']) ? $options['retries'] + 0 : 2;
if(!@snmpget($ip, $reader, ".1.3.6.1.2.1.1.1.0", $timeout, $retries)){
	syslog(LOG_ERR, sprintf("No response from %s", $ip));
	exit(2);
}

$exit = 0;
if(isset($options['basic-oid'])){
	$basic_oid = $options['basic-oid'];
}else{
	$basic_oid_location = isset($options['basic-oid-location']) ? $options['basic-oid-location'] : ".1.3.6.1.2.1.1.2.0";
	$basic_oid = substr(@snmpget($ip, $reader, $basic_oid_location, $timeout, $retries), 5);
	if(!$basic_oid){
		syslog(LOG_ERR, sprintf("No response from %s", $ip));
		exit(2);
	}
}
$mapping_oid  = isset($options['port-mac-table-oid']) ? $options['port-mac-table-oid'] : $basic_oid.".1.4.2.8.1.2";
$security_oid = isset($options['port-security-oid']) ? $options['port-security-oid'] : $basic_oid.".1.4.3.1.3";
$state_oid = isset($options['port-state-oid']) ? $options['port-state-oid'] : $basic_oid.".1.2.1.1.5";

if(isOn('mac-table')){
	$mac_out = isOn('mac-out');
	$write = array();
	$mac_table = array();
	if($res = @snmprealwalk($ip, $reader, $mapping_oid, $timeout*3, $retries)){
		foreach($res as $key=>$value){
			$index = substr(strrchr($key, "."), 1);
			$link = strtolower(str_replace(" ", "", substr($value, 12)));
			$mac = substr($link,0,12);
			$empty = hexdec($mac) == 0;
			if(!$empty){
				$port_code = substr($link,12);
				$port = strpos(strrev(decbin(hexdec($port_code))), '1') + 1;
				$k = $mac.":".$port;
				$mo_key = 'mac-out-'.$port;
				$port_mo = isset($options[$mo_key]) ? isPositive($mo_key) : $mac_out;
				if(isset($cfg_macs[$k]) && ($cfg_macs[$k]==1 || !$port_mo)){
					unset($cfg_macs[$k]);
					$mac_table[$index] = $link;
				}else{
					$write[$index]['link'] = sprintf("%020d", 1); // pending delete (may be overriden later)
					$write[$index]['msg'] = sprintf("delete %s ", $k);
					$mac_table[$index] = ''; //mark place as empty
				}
			}else{
				$mac_table[$index] = '';
			}
		}
		$i = 1;
		foreach($cfg_macs as $pair=>$state){
			list($mac, $port) = explode(":", $pair);
			$mo_key = 'mac-out-'.$port;
			$port_mo = isset($options[$mo_key]) ? isPositive($mo_key) : $mac_out;
			if($state==1 || !$port_mo){
				$new_link = sprintf("%s%08d", $mac, dechex(pow(2, $port-1)));
				for(;$i < count($mac_table); $i++){
					if(!$mac_table[$i]){
						$mac_table[$i] = $new_link;
						$write[$i]['link'] = $new_link;
						$write[$i]['msg'] .= sprintf("add %s", $pair);
						break;
					}
				}
				if($i==count($mac_table)){
					syslog(LOG_ERR, "No places for new macs");
					break;
				}
			}
		}
		$success = true;
		foreach($write as $index=>$value){
			syslog(LOG_INFO, $value['msg']);
			$success = $success && snmp_set($mapping_oid.'.'.$index, 'x', $value['link']);
		}
		if(!$success){
			$exit += 4;
		}
	}else{
		syslog(LOG_ERR, "Can't read mac table");
		$exit+=4;
	}
}

$control_security_key = 'control-port-security';
$control_security = isOn($control_security_key);
if($res = @snmprealwalk($ip, $reader, $security_oid, $timeout*3, $retries)){
	$success = true;
	foreach($res as $key=>$value){
		$index = substr(strrchr($key, "."), 1);
		$co_key = $control_security_key.'-'.$index;
		$co_security = isset($options[$co_key]) ? isPositive($co_key) : $control_security;
		if($co_security){
			$state = substr(trim($value),-1,1);
			$change = 0;
			if(isset($cfg_ports[$index])){
				if($state!='2'){ //accept
					$change = "3"; //woodoo
				}
			}else{
				if($state!='1'){ // not "no security"
					$change = "1";
				}
			}
			if($change > 0){
				syslog(LOG_INFO, sprintf("change port [%d] security to [%s]", $index, $change==3 ? 'on' : 'off'));
				$success = $success && snmp_set($security_oid.'.'.$index, 'i', $change);
			}
		}
	}
	if(!$success){
		$exit += 8;
	}
}else{
	syslog(LOG_ERR, "Can't read port security");
	$exit+=8;
}

$control_state_key = 'control-port-state';
$control_state = isOn($control_state_key);
if($res = @snmprealwalk($ip, $reader, $state_oid, $timeout*3, $retries)){
	$success = true;
	foreach($res as $key=>$value){
		$index = substr(strrchr($key, "."), 1);
		$co_key = $control_state_key.'-'.$index;
		$co_state = isset($options[$co_key]) ? isPositive($co_key) : $control_state;
		if($co_state){
			$state = substr(trim($value),-1,1);
			if(isset($cfg_ports[$index])){
				$cfg_state = $cfg_ports[$index]!=0 ? 1 : 2;
			}else{
				$cfg_state = 1;
			}
			if($cfg_state!=$state){
				syslog(LOG_INFO, sprintf("change port [%d] state to [%s]", $index, $cfg_state==1 ? 'on' : 'off'));
				$success = $success && snmp_set($state_oid.'.'.$index, 'i', $cfg_state);
			}
		}
	}
	if(!$success){
		$exit += 16;
	}
}else{
	syslog(LOG_ERR, "Can't read port security");
	$exit+=16;
}
if($exit){
	syslog(LOG_INFO, "Complete with errors");
	exit($exit);
}else{
	syslog(LOG_INFO, "Complete");
}

function isPositive($str){
	global $positives;
	return in_array(strtolower($str), $positives);
}

function isOn($str){
	global $options;
	return isset($options[$str]) && isPositive($options[$str]);
}

function snmp_set($oid, $type, $value){
	global $ip, $writer, $timeout, $retries;
	$result = @snmpset($ip, $writer, $oid, $type, $value, $timeout, $retries);
	if($result){
		syslog(LOG_INFO, sprintf("snmpset %s => %s", $oid, $value));
	}else{
		syslog(LOG_ERR, sprintf("Failed snmpset %s => %s", $oid, $value));
	}
	return $result;
}
?>