#!/usr/bin/php -q
<?php
if(count($_SERVER['argv']) < 3){
	die("Request incomplete");
}
$ip = $_SERVER['argv'][1];
$cfg = $_SERVER['argv'][2];
$secure = true;
$changed = false;

ini_set('include_path', DIRNAME(__FILE__). '/../');
include("includes/options.php");
include("classes/context/include.php");
$beanFactory = new CompositBeanFactory(array(
	new ArrayBeanFactory('boot/caban/log.php'),
));
$loggerFactory = $beanFactory->getBean('loggerFactory');
$logger = $loggerFactory->getLogger('switch.dlink');

$logger->info("configuring: %s", $ip);

$positives = array('on','true','1','yes','enable');
$option_pattern = "|([\w\-]+)\s*=\s*(\S.*)|ix";
$item_pattern = "|(\w\w\:\w\w\:\w\w\:\w\w\:\w\w\:\w\w)\s+(\d+)\s+(\w+)\s+([\w\-_\.]+)\s+(\d+\.\d+\.\d+\.\d+)\s+(\d+)|ix";

$options = array();
$cfg_macs = array();
$cfg_ports = array();
$cfg_names = array();
$cfg_bindings = array();
$cfg_vlans = array();
if($file = fopen ($cfg, "r")){
	while (!feof($file)) {
		$line = trim(fgets($file, 1024));
		$pos = strpos($line, '#');
		if(!($pos===false)){
			$line = trim(substr($line,0,$pos));
		}
		if($line){
			if(preg_match($option_pattern, $line, $m)){
				$options[strtolower($m[1])] = trim($m[2]);
			}elseif(preg_match($item_pattern, $line, $m)){
				list($all, $mac, $port, $state, $login, $host_ip, $host_vlan) = $m;
				if($port > 0){
					$mac = strtolower(str_replace(":","",$mac));
					$state = in_array(strtolower($state), $positives) ? 1 : 0;
					$cfg_names[$port] = $login;
					if($mac!='000000000000'){
						$cfg_macs[$mac.":".$port] = $state;
						$cfg_bindings[$port][$mac][$host_ip] = 1;
					}
					$cfg_macs[$mac.":".$port] = $state;
					if(isset($cfg_ports[$port])){
						if($cfg_ports[$port]!=$state){
							$cfg_ports[$port] = -1;
						}
					}else{
						$cfg_ports[$port] = $state;
					}
					if($host_vlan > 0){
						$cfg_vlans[$port] = $host_vlan;
					}
				}
			}
		}
	}
	fclose($file);
}else{
	$logger->error("Can't open %s for reading", $cfg);
	exit(1);
}

//print_r($cfg_bindings);

$reader = $options['public-community'];
$writer = $options['private-community'];
$timeout = isset($options['timeout']) ? $options['timeout'] * 1000 : 1000000;
$retries = isset($options['retries']) ? $options['retries'] + 0 : 2;
$def_vlan = $options['vlan'];

//snmp_set_oid_numeric_print(1);
//snmp_set_quick_print(1);
//snmp_set_valueretrieval(SNMP_VALUE_PLAIN);

$basic_oid_location = isset($options['basic-oid-location']) ? $options['basic-oid-location'] : ".1.3.6.1.2.1.1.2.0";
$basic_oid = substr(@snmpget($ip, $reader, $basic_oid_location, $timeout, $retries), 5);
if($basic_oid=="SNMPv2-SMI::enterprises.259.8.1.6"){
		$basic_oid = "SNMPv2-SMI::enterprises.259.8.1.5";
}
if(!$basic_oid){
	$logger->error("No response from %s", $ip);
	exit(2);
}
/*
$edgecore_models = array(
	'SNMPv2-SMI::enterprises.259.6.10.57'=>'ES4612',
	'SNMPv2-SMI::enterprises.259.6.10.74'=>'ES3526XA-1',
	'SNMPv2-SMI::enterprises.259.6.10.94'=>'ES3552M',
	'SNMPv2-SMI::enterprises.259.8.1.5'=>'ES3526XA-2',
	'SNMPv2-SMI::enterprises.259.8.1.6'=>'ES3510',
);
if(!isset($edgecore_models[$basic_oid])){
	$logger->error("this is not edge-core model %s", $ip);
	exit(2);
}
*/
$oid = array(
	//.1.3.6.1.4.1.259.6.10.75
	'port_status'=>".1.3.6.1.2.1.2.2.1.7",
	'port_desc'=>".1.3.6.1.2.1.31.1.1.1.18",
	//'port_desc'=>$basic_oid.".1.2.1.1.2", //relative
	'port_security'=>$basic_oid.".1.17.2.1.1.2",  //relative
	'port_security_action'=>$basic_oid.".1.17.2.1.1.3",  //relative
	'port_security_count'=>$basic_oid.".1.17.2.1.1.4",  //relative
	'mapping'=>".1.3.6.1.2.1.17.7.1.3.1.1.3",
	'mapping_type'=>".1.3.6.1.2.1.17.7.1.3.1.1.4",
	'config_src'=>$basic_oid.".1.24.1.1.0",  //relative
	'config_dst'=>$basic_oid.".1.24.1.3.0",  //relative
	'config_dst_name'=>$basic_oid.".1.24.1.4.0",  //relative
	'config_action'=>$basic_oid.".1.24.1.8.0",  //relative
	'ipsrc_guard_type'=>$basic_oid.'.1.48.1.1.2',
	'ipsrc_guard_binding'=>$basic_oid.'.1.48.2.1.3',
	'ipsrc_guard_table'=>$basic_oid.'.1.48.2.1.4',
	'ipsrc_guard_ip'=>$basic_oid.'.1.48.2.1.5',
	'ipsrc_guard_port'=>$basic_oid.'.1.48.2.1.6',
	'ipsrc_guard_status'=>$basic_oid.'.1.48.2.1.8',
	'vlan'=>'.1.3.6.1.2.1.17.7.1.4.3.1.2',
	'vlan_untagged'=>'.1.3.6.1.2.1.17.7.1.4.3.1.4',
	'vlan_pvid'=>'.1.3.6.1.2.1.17.7.1.4.5.1.1',
	'vlan_frame'=>'.1.3.6.1.2.1.17.7.1.4.5.1.2',
	'vlan_filter'=>'.1.3.6.1.2.1.17.7.1.4.5.1.3',
);

$exit = 0;

//print_r($options);
//print_r($cfg_macs);
//print_r($cfg_ports);
//print_r($cfg_names);

$control_state_key = 'control-port-state';
$control_state = isOn($control_state_key);
if($res = @snmprealwalk($ip, $reader, $oid['port_status'], $timeout, $retries)){
	$success = true;
	foreach($res as $key=>$value){
		$index = substr(strrchr($key, "."), 1);
		if(isset($cfg_ports[$index])){
			$co_key = $control_state_key.'-'.$index;
			$co_state = isset($options[$co_key]) ? isPositive($co_key) : $control_state;
			if($co_state){
				$state = substr(trim($value),-2,1);
				$cfg_state = $cfg_ports[$index]!=0 ? 1 : 2;
				if($cfg_state!=$state){
					$logger->info("change port [%d] state to [%s]", $index, $cfg_state==1 ? 'on' : 'off');
					$success = $success && snmp_set($oid['port_status'].'.'.$index, 'i', $cfg_state);
				}
			}
		}
	}
	if(!$success){
		$exit += 16;
	}
}else{
	$logger->error("Can't read port status");
	$exit+=16;
	$logger->info("Aborting due to errors");
        exit($exit);
}

if($res = @snmprealwalk($ip, $reader, $oid['port_desc'], $timeout, $retries)){
	$success = true;
	print_r($res);
	foreach($res as $key=>$value){
		$check = false;
		$index = substr(strrchr($key, "."), 1);
		$pn_key = 'port-name-'.$index;
		if(isset($options[$pn_key])){
			$cfg_desc = $options[$pn_key];
			$check = true;
		}
		if(isset($cfg_names[$index])){
			$cfg_desc = $cfg_names[$index];
			$check = true;
		}
		if($check){
			$value = trim($value);
			$desc = strlen($value) > 2 ? substr($value, 8) : "";
			if($cfg_desc!=$desc){
				$logger->info("change port [%d] desc to [%s]", $index, $cfg_desc);
				$success = $success && snmp_set($oid['port_desc'].'.'.$index, 's', $cfg_desc);
			}
		}
	}
	if(!$success){
		$exit += 32;
	}
}else{
	$logger->error("Can't read port names");
	$exit+=32;
}

/*
$final_hexes = array();
$up_hexes = array();
foreach(array('vlan'=>128, 'vlan_untagged'=>256) as $param=>$err_code){
	if($res = @snmprealwalk($ip, $reader, $oid[$param], $timeout, $retries)){
		$success = true;
		foreach($res as $key=>$value){
			$vlan = substr(strrchr($key, '.'), 1);
			if(($vlan>1) && ($vlan<4000)){
				list($type, $val) = explode(":", $value);
				$hex = str_replace(" ", "", trim($val));
				$orig_hex = $hex;
				$final_hexes[$vlan][$param] = $hex;
				$up_hexes[$vlan][$param] = $hex;
				foreach($cfg_ports as $port=>$state){
					$cfg_vlan = isset($cfg_vlans[$port]) ? $cfg_vlans[$port] : $def_vlan;
					if($vlan==$cfg_vlan){
						$hex = setLeftBit($hex, $port, 1);
						$up_hexes[$vlan][$param] = $hex;
						$final_hexes[$vlan][$param] = setLeftBit($final_hexes[$vlan][$param], $port, 1);
					}else{
						$final_hexes[$vlan][$param] = setLeftBit($final_hexes[$vlan][$param], $port, 0);
					}
				}
				if(strtoupper($hex) != $orig_hex){
					$logger->info("change %s %s from [%s] up to [%s]", $param, $vlan, $orig_hex, $hex);
					$success = snmp_set($oid[$param].'.'.$vlan, 'x', $hex) && $success;
				}
			}
		}
		if(!$success){
			$exit += $err_code;
		}
	}else{
		$logger->error("Can't read port vlans");
		$exit+=$err_code;
	}
}

if($res = @snmprealwalk($ip, $reader, $oid['vlan_pvid'], $timeout, $retries)){
	$success = true;
	foreach($res as $key=>$value){
		$port = substr(strrchr($key, '.'), 1);
		if(isset($cfg_ports[$port])){
			list($type, $val) = explode(":", $value);
			$vlan = trim($val);
			$cfg_vlan = isset($cfg_vlans[$port]) ? $cfg_vlans[$port] : $def_vlan;
			if($vlan != $cfg_vlan){
				$logger->info("change [%s] port pvid to [%s]", $port, $cfg_vlan);
				$success = snmp_set($oid['vlan_pvid'].'.'.$port, 'u', $cfg_vlan) && $success;
			}
		}
	}
	if(!$success){
		$exit += 512;
	}
}else{
	$logger->error("Can't read port pvids");
	$exit+=512;
}

$success = true;
foreach($final_hexes as $vlan=>$arr){
	foreach($arr as $param=>$hex){
		if(strcasecmp($hex, $up_hexes[$vlan][$param])!=0){
			$logger->info("change %s %s from [%s] down to [%s]", $param, $vlan, $up_hexes[$vlan][$param], $hex);
			$success = snmp_set($oid[$param].'.'.$vlan, 'x', $hex) && $success;
		}
	}
}
if(!$success){
	$exit += 1024;
}

function setLeftBit($hex, $offset, $value){
	static $mul = array(8, 4, 2, 1);
	$pos = intval(ceil($offset / 4)) - 1;
	$m = $mul[($offset - 1) %4];
	if($value){
		$tetra = hexdec($hex[$pos]) | $m;
	}else{
		$tetra = hexdec($hex[$pos]) & (15 - $m);
	}
	$hex[$pos] = dechex($tetra);
	return $hex;
}

function dec2hex(&$i){
	$i = sprintf("%02s", dechex($i));
}

function findOne($hex){
	$len = strlen($hex);
	$dec = '';
	for($i=0; $i<$len; $i+=2){
		$dec .= sprintf( "%08s" ,decbin(hexdec(substr($hex, $i, 2))));
	}
	return strpos('0'.$dec, '1');
}

function packOne($pos, $len=8){
	$l = $len*8;
	$dec = substr(str_repeat('0', $l).'1'.str_repeat('0', $l - $pos), -$l);
	$hex = '';
	for($i=0; $i<$len; $i++){
		$b = substr($dec, $i*8, 8);
		$hex .= sprintf("%02s", dechex(bindec(bin2hex($b))));
	}
	return $hex;
}

if(isOn('mac-table')){
	$mac_out = isOn('mac-out');
	$write = array();
	$mac_table = array();
	$moid = $oid['mapping'];
	$res = @snmprealwalk($ip, $reader, $moid, $timeout, $retries);
	if(is_array($res)){
		//print_r($res);
		$must_links = array();
		foreach($cfg_macs as $link=>$x){
			list($mac, $port) = explode(":", $link);
			if(isset($cfg_vlans[$port])){
				$vlan = $cfg_vlans[$port];
			}else{
				$vlan = $def_vlan;
			}
			$mo_key = 'mac-out-'.$port;
			$port_mo = isset($options[$mo_key]) ? isPositive($mo_key) : $mac_out;
			if(($cfg_ports[$port]==1) || !$port_mo){
				$must_links[$link.':'.$vlan] = 1;
			}
		}
		//print_r($must_links);
		$success = true;
		foreach($res as $key=>$value){
			$split = explode(".", $key);
			$vlan = $split[8];
			$macd = array_slice($split, 9, 6);
			$mach = $macd;
			array_walk($mach, 'dec2hex');
			$mac = implode($mach);
			$port = findOne(str_replace(" ", "", substr($value, 12, 26)));;
			$link = $mac.':'.$port.':'.$vlan;
			$action = '';
			if(isset($must_links[$link])){
				unset($must_links[$link]);
			}else{
				if(isset($cfg_ports[$port])){
					$logger->info("remove link: mac [%s] to port [%s]", $mac, $port);
					$roid = $oid['mapping_type'].'.'.$vlan.'.'.implode('.',$macd).'.0';
					//echo "remove: $roid ($mac)\n";
					$success = snmp_set($roid, 'i', 2) && $success;
				}
			}
		}
		//print_r($must_links);
		foreach($must_links as $link=>$x){
			list($mac, $port, $vlan) = explode(":", $link);
			$logger->info("add link: mac [%s] to port [%s]", $mac, $port);
			$macd = array();
			for($i=0; $i < 12; $i+=2){
				$macd[] = hexdec(substr($mac, $i, 2));
			}
			$macd = implode('.',$macd);
			$moid = $oid['mapping'].'.'.$vlan.'.'.$macd.'.0';
			$soid = $oid['mapping_type'].'.'.$vlan.'.'.$macd.'.0';
			$phex = packOne($port, intval(ceil($port/8)));
			//echo "soid: $soid, moid: $moid, phex: $phex\n";
			$success = snmp_set($soid, 'i', 3) && $success;
			$success = snmp_set($moid, 'x', $phex) && $success;
		}
		if(!$success){
			$exit += 4;
		}
	}else{
		$logger->error("Can't read mac table");
		$exit+=4;
	}
}

$control_security_key = 'control-port-security';
$control_security = isOn($control_security_key);

$security_map = array(
	$oid['port_security']=>array(1, 'port security'),
	$oid['port_security_action']=>array(1, 'port security action'),
	$oid['port_security_count']=>array(0, 'port security count'),
);
$success = true;
foreach($security_map as $xoid=>$arr){
	if($res = @snmprealwalk($ip, $reader, $xoid, $timeout, $retries)){
		foreach($res as $key=>$value){
			$index = substr(strrchr($key, "."), 1);
			if(isset($cfg_ports[$index])){
				$co_key = $control_security_key.'-'.$index;
				$co_security = isset($options[$co_key]) ? isPositive($co_key) : $control_security;
				if($co_security){
					$state = substr(trim($value), -1, 1);
					$cfg_state = $cfg_ports[$index]!=0 ? 0 : 1;
					if($state!=$arr[0]){
						$logger->info("Change [%d] %s to [%s]", $index, $arr[1], $arr[0]);
						$success = snmp_set($xoid.'.'.$index, 'i', $arr[0]) && $success;
					}
				}
			}
		}
	}else{
		$logger->error("Can't read %s", $arr[1]);
		$success = false;
	}
}
if(!$success){
	$exit += 8;
}

$ipsrc_guard_key = 'ip-source-guard';
$ipsrc_guard = isOn($ipsrc_guard_key);
if($ipsrc_guard){
	$success = true;
	if($res = @snmprealwalk($ip, $reader, $oid['ipsrc_guard_type'], $timeout, $retries)){
		//print_r($res);
		foreach($res as $key=>$value){
			$index = substr(strrchr($key, "."), 1);
			if(isset($cfg_bindings[$index])){
				$type = substr(trim($value), -1, 1);
				if($type!='2'){
					$logger->info("Change [%d] ip-src-guard type to srcIpMac", $index);
					$success = snmp_set($oid['ipsrc_guard_type'].'.'.$index, 'i', 2) && $success;
				}
			}
		}
		$ipsrc_quard_data = array();
		foreach(array('ip', 'port', 'table', 'status') as $req){
			if($res = @snmprealwalk($ip, $reader, $oid['ipsrc_guard_'.$req], $timeout, $retries)){
				//print_r($res);
				foreach($res as $key=>$value){
					$split = explode(".", $key);
					$vlanarr = array_slice($split, -7, 1);
					$vlan = $vlanarr[0];
					$macd = array_slice($split, -6);
					$mach = $macd;
					array_walk($mach, 'dec2hex');
					$mac = implode($mach);
					list($type, $val) = explode(":", $value);
					$ipsrc_quard_data[$vlan][$mac][$req] = trim($val);
				}
			}
		}
		foreach($ipsrc_quard_data as $vlan=>$vlanarr){
			foreach($vlanarr as $mac=>$data){
				$port = $data['port'];
				$host_ip = $data['ip'];
				$is_static = $data['table']=='3';
				if(isset($cfg_vlans[$port])){
					$cfg_vlan = $cfg_vlans[$port];
				}else{
					$cfg_vlan = $def_vlan;
				}
				if(isset($cfg_bindings[$port][$mac][$host_ip]) && ($cfg_vlan==$vlan) && $is_static && ($data['status']==1)){
					unset($cfg_bindings[$port][$mac][$host_ip]);
				}else{
					if(isset($cfg_bindings[$port])){
						// remove
						$success = snmp_set($oid['ipsrc_guard_status'].'.'.$vlan.'.'.macD($mac), 'i', 6) && $success;
					}
				}
			}
		}
		foreach($cfg_bindings as $port=>$data){
			foreach($data as $mac=>$arr){
				foreach($arr as $host_ip=>$x){
					// add
					if(isset($cfg_vlans[$port])){
						$vlan = $cfg_vlans[$port];
					}else{
						$vlan = $def_vlan;
					}
					$macD = macD($mac);
					$success = snmp_set($oid['ipsrc_guard_status'].'.'.$vlan.'.'.$macD, 'i', 5) && $success;
					$success = snmp_set($oid['ipsrc_guard_binding'].'.'.$vlan.'.'.$macD, 'i', 1) && $success;
					$success = snmp_set($oid['ipsrc_guard_ip'].'.'.$vlan.'.'.$macD, 'a', $host_ip) && $success;
					$success = snmp_set($oid['ipsrc_guard_port'].'.'.$vlan.'.'.$macD, 'i', $port) && $success;
					$success = snmp_set($oid['ipsrc_guard_status'].'.'.$vlan.'.'.$macD, 'i', 1) && $success;
				}
			}
		}
	}else{
		$logger->error("Can't read ip src guard types");
		$success = false;
	}
	if(!$success){
		$exit += 64;
	}
}

function macD($macH){
	$macd = array();
	for($i=0; $i < 12; $i+=2){
		$macd[] = hexdec(substr($macH, $i, 2));
	}
	return implode('.',$macd);
}

if($changed){
	$success = true;
	$target = isset($options['target-config']) ? $options['target-config'] : 'startup';
	$logger->info("Saving config...");
	$success = $success && snmp_set($oid['config_src'], 'i', 2);
	$success = $success && snmp_set($oid['config_dst'], 'i', 3);
	$success = $success && snmp_set($oid['config_dst_name'], 's', $target);
	$success = $success && snmp_set($oid['config_action'], 'i', 2);
	if(!$success){
		$exit += 4096;
	}
}else{
	$logger->info("No changes");
}
*/
if($exit){
	$logger->info("Complete with errors");
	exit($exit);
}else{
	$logger->info("Complete");
}

function isPositive($str){
	global $positives;
	return in_array(strtolower($str), $positives);
}

function isOn($str){
	global $options;
	return isset($options[$str]) && isPositive($options[$str]);
}

function snmp_set($oid, $type, $value){
	global $ip, $writer, $timeout, $retries, $changed, $logger;
	echo "snmpset -c $writer $ip $oid $type $value";
	$result = @snmpset($ip, $writer, $oid, $type, $value, $timeout, $retries);
	if($result){
		$logger->info("snmpset %s => %s", $oid, $value);
		$changed = true;
		echo " OK\n";
	}else{
		$logger->error("Failed snmpset %s => %s", $oid, $value);
		echo " FAILED\n";
	}
	return $result;
}
?>