#!/usr/bin/php -q
<?php
if(count($_SERVER['argv']) < 3){
	die("Request incomplete");
}
$ip = $_SERVER['argv'][1];
$cfg = $_SERVER['argv'][2];

openlog("isp.switch.edimax.http", LOG_PID + LOG_CONS, LOG_USER);
syslog(LOG_INFO, sprintf("Profiling %s", $ip));

$positives = array('on','true','1','yes','enable');
$option_pattern = "|([\w\-]+)\s*=\s*(\S.*)|ix";
$item_pattern = "|(\w\w\:\w\w\:\w\w\:\w\w\:\w\w\:\w\w)\s+(\d+)\s+(\w+)|ix";

$options = array();
$cfg_macs = array();
$cfg_ports = array();
if($file = fopen ($cfg, "r")){
	while (!feof($file)) {
	    $line = trim(fgets($file, 1024));
		$pos = strpos($line, '#');
		if(!($pos===false)){
			$line = trim(substr($line,0,$pos));
		}
		if($line){
			if(preg_match($option_pattern, $line, $m)){
				$options[strtolower($m[1])] = trim($m[2]);
			}elseif(preg_match($item_pattern, $line, $m)){
				list($all, $mac, $port, $state) = $m;
				$mac = strtolower(str_replace(":","",$mac));
				$state = in_array(strtolower($state), $positives) ? 1 : 0;
				$cfg_macs[$mac.":".$port] = $state;
				if(isset($cfg_ports[$port])){
					if($cfg_ports[$port]!=$state){
						$cfg_ports[$port] = -1;
					}
				}else{
					$cfg_ports[$port] = $state;
				}
			}
		}
	}
	fclose($file);
}else{
	syslog(LOG_ERR, sprintf("Can't open %s for reading", $cfg));
	exit(1);
}

$ch = curl_init();
if(isset($options['timeout'])){
	curl_setopt($ch, CURLOPT_TIMEOUT, $options['timeout']);
}
if(isset($options['http-user'])){
	curl_setopt($ch, CURLOPT_HTTPHEADER, array("Authorization: Basic ".base64_encode(sprintf("%s:%s", $options['http-user'], $options['http-password']))));
}

$exit = 0;

if(isOn('mac-table')){
	$mac_page = get_cmd($ch, sprintf("http://%s/filterS.htm", $ip));
	if($mac_page){
		$start = strpos($mac_page, "<SELECT name=FilterS_LIST size=10 width=100>");
		$end = strpos($mac_page, "</SELECT>", $start);
		$body = substr($mac_page, $start, $end - $start);
		$mac_table = array();
		if(preg_match_all("|\<OPTION[^\>]+value=([^\>]+)\>([^\_]+)_+PORT([^\_]+)|m", $body, $m, PREG_SET_ORDER)){
			foreach($m as $match){
				$mac_table[$match[2].":".$match[3]] = $match[1];
			}
		}
		$new_macs = array();
		$mac_out = isOn('mac-out');
		foreach($cfg_macs as $pair=>$state){
			list($mac, $port) = explode(":", $pair);
			$mo_key = 'mac-out-'.$port;
			$port_mo = isset($options[$mo_key]) ? isPositive($mo_key) : $mac_out;
			if($state==1 || !$port_mo){
				if(isset($mac_table[$pair])){
					unset($mac_table[$pair]);
				}else{
					$new_macs[$mac] = $port;
				}
			}
		}
		foreach($mac_table as $pair=>$index){
			list($mac, $port) = explode(":", $pair);
			syslog(LOG_INFO, sprintf("delete %s => %s", $mac, $port));
			$cmd = sprintf("FilterS_LIST=%d&FilterS_OP=3", $index);
			post_cmd($ch, sprintf("http://%s/filterS", $ip), $cmd);
		}
		foreach($new_macs as $mac=>$port){
			syslog(LOG_INFO, sprintf("add %s => %s", $mac, $port));
			$cmd = sprintf("Filter_Port=PORT%d&Filter_Mac=%s&FilterS_OP=1", $port, $mac);
			post_cmd($ch, sprintf("http://%s/filterS", $ip), $cmd);
		}
	}else{
		syslog(LOG_ERR, "Can't read mac table");
		$exit+=2;
	}
}

$control_security_key = 'control-port-security';
$control_state_key = 'control-port-state';
$control_security = isOn($control_security_key);
$control_state = isOn($control_state_key);
$port_page = get_cmd($ch, sprintf("http://%s/hidden1.htm", $ip));
if($port_page){
	$port_controls = array();
	$port_pattern = "|S\( (\d+)\, (\d+)\, \"([^\"]+)\" \)\;|imxs";
	if(preg_match_all($port_pattern, $port_page, $m, PREG_SET_ORDER)){
		foreach($m as $match){
			$port_controls[$match[1]][$match[2]] = $match[3];
		}
	}
	$nego = array('Force'=>0,'Auto'=>1,'Nway'=>2);
	$speed = array('10'=>1,'100'=>2);
	$duplex = array('Half'=>0,'Full'=>1);
	$on_off = array('Off'=>0,'On'=>1);
	$priority = array('Disable'=>0, 'Low'=>1, 'High'=>2);
	foreach($port_controls as $info){
		$port_name = $info[0];
		if(substr($port_name, 0, 4)=='PORT'){
			$port = substr($port_name, 4);
			$co_security_key = $control_security_key.'-'.$port;
			$co_state_key = $control_state_key.'-'.$port;
			$co_security = isset($options[$co_security_key]) ? isPositive($co_security_key) : $control_security;
			$co_state = isset($options[$co_state_key]) ? isPositive($co_state_key) : $control_state;
			
			$actual_state = $on_off[$info[1]];
			if(isset($cfg_ports[$port])){
				$cfg_state = $cfg_ports[$port]!=0 ? 1 : 0;
			}else{
				$cfg_state = 1;
			}
			$diff_state = $co_state && ($cfg_state!=$actual_state);
			
			$actual_security = $on_off[$info[16]];
			$cfg_security = isset($cfg_ports[$port]) ? 1 : 0;
			$diff_security = $co_security && ($cfg_security != $actual_security);
			
			if($diff_state || $diff_security){
				$msg = sprintf("change port [%d]:", $port);
				$cmd = array(
					'PortNum'=>$port_name,
					'Nego'=>$nego[$info[4]],
					'Speed'=>$speed[$info[6]],
					'Duplex'=>$duplex[$info[8]],
					'FlwCtlFl'=>$on_off[$info[10]],
					'FlwCtlHf'=>$on_off[$info[11]],
					'RCtlIn'=>$info[13]=='Off' ? 0 : $info[13],
					'RCtlE'=>$info[14]=='Off' ? 0 : $info[14],
					'StatPri'=>$priority[$info[15]],
					'Submit'=>'Apply',
				);
				$cmd['PortState'] = $co_state ? $cfg_state : $actual_state;
				if($diff_state){
					$msg .= sprintf(" state to [%s]", $cfg_state ? 'On' : 'Off');
				}
				if(($co_security ? $cfg_security : $actual_security)){
					$cmd['PSecuri'] = 'On';
				}
				if($diff_security){
					$msg .= sprintf(" security to [%s]", $cfg_security ? 'On' : 'Off');
				}
				syslog(LOG_INFO, $msg);
				post_cmd($ch, sprintf("http://%s/CfgPortSet", $ip), $cmd);
			}
		}
	}
}else{
	syslog(LOG_ERR, "Can't read port controls");
	$exit+=4;
}


curl_close($ch);
if($exit){
	syslog(LOG_INFO, "Complete with errors");
	exit($exit);
}else{
	syslog(LOG_INFO, "Complete");
}

function isPositive($str){
	global $positives;
	return in_array(strtolower($str), $positives);
}

function isOn($str){
	global $options;
	return isset($options[$str]) && isPositive($options[$str]);
}

function cmd($ch){
	ob_start();
    curl_exec($ch);
	$err = curl_errno($ch);
	if($err){
		syslog(LOG_ERR, sprintf("curl error #%d: %s", $err, curl_error($ch)));
	}
	$result = ob_get_contents();
	ob_end_clean();
	return $result;
}

function get_cmd($ch, $address){
    curl_setopt($ch, CURLOPT_URL, $address);
	curl_setopt($ch, CURLOPT_POST, 0);
	return cmd($ch);
}

function post_cmd($ch, $address, $cmd){
	if(is_array($cmd)){
		$items = array();
		foreach($cmd as $k=>$v){
			$items[] = sprintf("%s=%s", $k, $v);
		}
		$cmd = implode("&", $items);
	}
    curl_setopt($ch, CURLOPT_URL, $address);
	curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $cmd);
	syslog(LOG_INFO, sprintf("curl: %s?%s", $address, $cmd));
	return cmd($ch);
}
?>

